<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    recommend
 */

/**
 * Block class.
 */
class Block_main_screen_actions
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 2;
        $info['locked'] = false;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'recommend';
        $info['parameters'] = ['title', 'url'];
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        $error_msg = new Tempcode();
        if (!addon_installed__messaged('recommend', $error_msg)) {
            return $error_msg;
        }

        require_css('screen_actions');

        require_lang('recommend');

        $block_id = get_block_id($map);

        $url = isset($map['url']) ? $map['url'] : '';
        if ($url == '') {
            $url = get_self_url(true);
        }

        $title = isset($map['title']) ? $map['title'] : '';
        if ($title == '') {
            global $METADATA, $DISPLAYED_TITLE;
            if (isset($METADATA['title'])) {
                $title = $METADATA['title'];
            }
            if (isset($DISPLAYED_TITLE)) {
                $title = $DISPLAYED_TITLE;
            }
        }

        $url_map = ['page' => 'recommend', 'from_url' => protect_url_parameter($url), 'from_title' => $title];
        $recommend_url = build_url($url_map, get_module_zone('recommend'));

        return do_template('BLOCK_MAIN_SCREEN_ACTIONS', [
            '_GUID' => '2f5ceee4e1cc3d31c184c62e0710b1c3',
            'BLOCK_ID' => $block_id,
            'PRINT_URL' => get_self_url(true, false, ['wide_print' => 1]),
            'RECOMMEND_URL' => $recommend_url,
            'URL_ENCODED_CANONICAL_URL' => urlencode(get_canonical_url()),
            'TITLE' => array_key_exists('title', $map) ? $map['title'] : '',
        ]);
    }
}
