<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_adminzone_dashboard
 */

/**
 * Block class.
 */
class Block_main_staff_new_version
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 2;
        $info['locked'] = false;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'core_adminzone_dashboard';
        $info['parameters'] = [];
        return $info;
    }

    /**
     * Find caching details for the block.
     *
     * @return ?array Map of cache details (cache_on and ttl) (null: block is disabled)
     */
    public function caching_environment() : ?array
    {
        $info = [];
        $info['cache_on'] = <<<'PHP'
        [
        ]
PHP;
        $info['ttl'] = 60 * 3;
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        require_lang('version');
        require_code('version2');
        require_css('adminzone_dashboard');

        $block_id = get_block_id($map);

        $table = get_future_version_information();

        require_code('addons2');
        $updated_addons = find_updated_addons();
        $has_updated_addons = (!empty($updated_addons));

        return do_template('BLOCK_MAIN_STAFF_NEW_VERSION', [
            '_GUID' => '43c7b18d3d44e825247579df23a2ad9c',
            'BLOCK_ID' => $block_id,
            'VERSION' => cms_version_pretty(),
            'VERSION_TABLE' => $table,
            'HAS_UPDATED_ADDONS' => $has_updated_addons,
        ]);
    }
}
