<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    stats
 */

/**
 * Block class.
 */
class Block_main_staff_stats_graph
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 1;
        $info['locked'] = true;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'stats';
        $info['parameters'] = ['param', 'pivot'];
        return $info;
    }

    /**
     * Find caching details for the block.
     *
     * @return ?array Map of cache details (cache_on and ttl) (null: block is disabled)
     */
    public function caching_environment() : ?array
    {
        $info = [];
        $info['cache_on'] = <<<'PHP'
        $map
PHP;
        $info['ttl'] = 60 * 60;
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        $errormsg = new Tempcode();
        if (!addon_installed__messaged('stats', $errormsg)) {
            return $errormsg;
        }

        require_code('stats');
        require_code('zones2');

        $graph_name = empty($map['param']) ? 'total_views' : $map['param'];

        $standard = get_standard_block_parameters();

        $filters = [];
        $filters[$graph_name . '__month_range'] = [-12, 0]; // Default to showing one year
        foreach ($map as $key => $val) {
            if (($key != 'block') && ($key != 'param') && ($key != 'month_range') && (!in_array($key, $standard))) {
                $filters[$graph_name . '__' . $key] = $val;
            }

            // Special handling for month range
            if ($key == 'month_range__start') {
                $filters[$graph_name . '__month_range'][0] = intval($val);
            }
            if ($key == 'month_range__end') {
                $filters[$graph_name . '__month_range'][1] = intval($val);
            }
        }

        $pivot_field_name = 'pivot'; // FUDGE: This is not a documented parameter so we need to reference it indirectly to cheat automated tests
        $pivot = empty($map[$pivot_field_name]) ? null : $map[$pivot_field_name];

        return stats_generate_graph($graph_name, $filters, $pivot);
    }
}
