<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    chat
 */

/**
 * Block class.
 */
class Block_side_friends
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 2;
        $info['locked'] = false;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'chat';
        $info['parameters'] = ['max'];
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        $error_msg = new Tempcode();
        if (!addon_installed__messaged('chat', $error_msg)) {
            return $error_msg;
        }

        if (is_guest()) {
            return new Tempcode(); // Guest has no friends
        }

        if ((get_page_name() == 'chat') && (get_param_string('type', 'browse') == 'browse')) { // Don't want to show if actually on chat lobby, which already has this functionality
            return new Tempcode();
        }

        require_code('chat');
        require_code('chat_lobby');
        require_lang('chat');
        require_css('chat');
        require_javascript('chat');

        $block_id = get_block_id($map);

        $max = array_key_exists('max', $map) ? intval($map['max']) : 15;

        $friends = show_im_contacts(null, true, $max);

        return do_template('BLOCK_SIDE_FRIENDS', [
            '_GUID' => 'ce94db14f9a212f38d0fce1658866e2c',
            'BLOCK_ID' => $block_id,
            'FRIENDS' => $friends,
        ]);
    }
}
