<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    printer_friendly_block
 */

/**
 * Block class.
 */
class Block_side_printer_friendly
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 2;
        $info['locked'] = false;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'printer_friendly_block';
        $info['parameters'] = [];
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        $error_msg = new Tempcode();
        if (!addon_installed__messaged('printer_friendly_block', $error_msg)) {
            return $error_msg;
        }

        $block_id = get_block_id($map);

        $url = get_self_url(true, false, ['wide_print' => 1]);

        return do_template('BLOCK_SIDE_PRINTER_FRIENDLY', [
            '_GUID' => 'db1d2db67f07a3d6bd130f4cef4c5e9d',
            'BLOCK_ID' => $block_id,
            'URL' => $url,
        ]);
    }
}
