<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    stats_block
 */

/**
 * Block class.
 */
class Block_side_stats
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 3;
        $info['locked'] = false;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'stats_block';
        $info['parameters'] = [];
        return $info;
    }

    /**
     * Find caching details for the block.
     *
     * @return ?array Map of cache details (cache_on and ttl) (null: block is disabled)
     */
    public function caching_environment() : ?array
    {
        $info = [];
        $info['cache_on'] = <<<'PHP'
        [
        ]
PHP;
        $info['ttl'] = 15;
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        $error_msg = new Tempcode();
        if (!addon_installed__messaged('stats_block', $error_msg)) {
            return $error_msg;
        }

        $block_id = get_block_id($map);

        $full_tpl = new Tempcode();

        // Inbuilt
        $bits = new Tempcode();
        $on_forum = $GLOBALS['FORUM_DRIVER']->get_num_users_forums();
        if ($on_forum !== null) {
            if (get_option('activity_show_stats_count_users_online') == '1') {
                $users_site = get_num_users_site();

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => '5ac97313d4c83e8afdeec09a48cea030',
                    'KEY' => do_lang_tempcode('COUNT_ONSITE'),
                    'RAW_VALUE' => strval($users_site),
                    'VALUE' => integer_format($users_site, 0),
                ]));
            }
            if (get_option('activity_show_stats_count_users_online_record') == '1') {
                $users_peak = get_num_users_peak();

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => 'dc6d0893cf98703a951da168c6a9d0ac',
                    'KEY' => do_lang_tempcode('COUNT_ONSITE_RECORD'),
                    'RAW_VALUE' => strval($users_peak),
                    'VALUE' => integer_format($users_peak, 0),
                ]));
            }
            if (get_option('activity_show_stats_count_users_online_forum') == '1') {
                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => '14f2fdbf59e86c34d93cbf16bed3f0eb',
                    'KEY' => do_lang_tempcode('COUNT_ONFORUMS'),
                    'RAW_VALUE' => strval($on_forum),
                    'VALUE' => integer_format($on_forum, 0),
                ]));
            }
            $title = do_lang_tempcode('USERS_ONLINE');
        } else {
            if (get_option('activity_show_stats_count_users_online') == '1') {
                $users_site = get_num_users_site();

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => '9c9760b2ed9e985e96b53c91c511e84e',
                    'KEY' => do_lang_tempcode('USERS_ONLINE'),
                    'RAW_VALUE' => strval($users_site),
                    'VALUE' => integer_format($users_site, 0),
                ]));
            }
            if (get_option('activity_show_stats_count_users_online_record') == '1') {
                $users_peak = get_num_users_peak();

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => 'd18068d747fe1fe364042133e4b3ba84',
                    'KEY' => do_lang_tempcode('USERS_ONLINE_RECORD'),
                    'RAW_VALUE' => strval($users_peak),
                    'VALUE' => integer_format($users_peak, 0),
                ]));
            }
            $title = do_lang_tempcode('ACTIVITY');
        }
        if (addon_installed('stats')) {
            if (get_option('activity_show_stats_count_page_views_today') === '1') {
                $today = $GLOBALS['SITE_DB']->query_value_if_there('SELECT COUNT(*) FROM ' . get_table_prefix() . 'stats WHERE date_and_time>' . strval(time() - 60 * 60 * 24));

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => 'fc9760b2ed9e985e96b53c91c511e84e',
                    'KEY' => do_lang_tempcode('PAGE_VIEWS_TODAY'),
                    'RAW_VALUE' => strval($today),
                    'VALUE' => integer_format($today, 0),
                ]));
            }
            if (get_option('activity_show_stats_count_page_views_this_week') === '1') {
                $week = $GLOBALS['SITE_DB']->query_value_if_there('SELECT COUNT(*) FROM ' . get_table_prefix() . 'stats WHERE date_and_time>' . strval(time() - 60 * 60 * 24 * 7));

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => '11293e62f4f4514697781e31dde82acf',
                    'KEY' => do_lang_tempcode('PAGE_VIEWS_THIS_WEEK'),
                    'RAW_VALUE' => strval($week),
                    'VALUE' => integer_format($week, 0),
                ]));
            }
            if (get_option('activity_show_stats_count_page_views_this_month') === '1') {
                $month = $GLOBALS['SITE_DB']->query_value_if_there('SELECT COUNT(*) FROM ' . get_table_prefix() . 'stats WHERE date_and_time>' . strval(time() - 60 * 60 * 24 * 31));

                $bits->attach(do_template('BLOCK_SIDE_STATS_SUBLINE', [
                    '_GUID' => '0e022498b078585582d7a58f2ac15841',
                    'KEY' => do_lang_tempcode('PAGE_VIEWS_THIS_MONTH'),
                    'RAW_VALUE' => strval($month),
                    'VALUE' => integer_format($month, 0),
                ]));
            }
        }
        if (!$bits->is_empty_shell()) {
            $full_tpl->attach(do_template('BLOCK_SIDE_STATS_SECTION', ['_GUID' => 'e2408c71a7c74f1d14089412d4538b6d', 'SECTION' => $title, 'CONTENT' => $bits]));
        }

        $_hooks = find_all_hook_obs('blocks', 'side_stats', 'Hook_stats_');
        if (array_key_exists('stats_forum', $_hooks)) { // Fudge the order
            $forum_hook = $_hooks['stats_forum'];
            unset($_hooks['stats_forum']);
            $_hooks = array_merge(['stats_forum' => $forum_hook], $_hooks);
        }
        foreach ($_hooks as $object) {
            $bits = $object->run();
            if (!$bits->is_empty_shell()) {
                $full_tpl->attach($bits);
            }
        }

        return do_template('BLOCK_SIDE_STATS', [
            '_GUID' => '0e9986c117c2a3c04690840fedcbddcd',
            'BLOCK_ID' => $block_id,
            'CONTENT' => $full_tpl,
        ]);
    }
}
