<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_notifications
 */

/**
 * Block class.
 */
class Block_top_notifications
{
    /**
     * Find details of the block.
     *
     * @return ?array Map of block info (null: block is disabled)
     */
    public function info() : ?array
    {
        $info = [];
        $info['author'] = 'Chris Graham';
        $info['organisation'] = 'Composr';
        $info['hacked_by'] = null;
        $info['hack_version'] = null;
        $info['version'] = 1;
        $info['locked'] = false;
        $info['min_cms_version'] = 11.0;
        $info['addon'] = 'core_notifications';
        $info['parameters'] = ['max'];
        return $info;
    }

    /**
     * Execute the block.
     *
     * @param  array $map A map of parameters
     * @return Tempcode The result of execution
     */
    public function run(array $map) : object
    {
        if (is_guest()) {
            return new Tempcode();
        }

        $block_id = get_block_id($map);

        $max = isset($map['max']) ? intval($map['max']) : 5;

        require_lang('notifications');

        require_code('notification_poller');

        list($notifications, , $num_unread_web_notifications) = get_web_notifications($max);

        list($pts, $num_unread_pts) = get_pts($max);

        return do_template('BLOCK_TOP_NOTIFICATIONS', [
            '_GUID' => '3fa04eb827741561440dbf1a65988b33',
            'BLOCK_ID' => $block_id,

            'NUM_UNREAD_WEB_NOTIFICATIONS' => strval($num_unread_web_notifications),
            'NOTIFICATIONS' => $notifications,

            'NUM_UNREAD_PTS' => strval($num_unread_pts),
            'PTS' => $pts,

            'MAX' => strval($max),
        ]);
    }
}
