<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_cns
 */

/**
 * Add a Multi Moderation to the system.
 *
 * @param  SHORT_TEXT $name The name of the Multi Moderation
 * @param  LONG_TEXT $post_text The post text to add when applying (blank: don't add a post)
 * @param  ?AUTO_LINK $move_to The forum to move the topic when applying (null: do not move)
 * @param  ?BINARY $pin_state The pin state after applying (null: unchanged)
 * @param  ?BINARY $open_state The open state after applying (null: unchanged)
 * @param  SHORT_TEXT $forum_multi_code The forum multi code for where this Multi Moderation may be applied
 * @param  SHORT_TEXT $title_suffix The title suffix
 * @return AUTO_LINK The ID of the Multi Moderation just added
 */
function cns_make_multi_moderation(string $name, string $post_text, ?int $move_to, ?int $pin_state, ?int $open_state, string $forum_multi_code = '*', string $title_suffix = '') : int
{
    if (!addon_installed('cns_multi_moderations')) {
        warn_exit(do_lang_tempcode('MISSING_ADDON', escape_html('cns_multi_moderations')));
    }

    require_code('global4');
    prevent_double_submit('ADD_MULTI_MODERATION', null, $name);

    $map = [
        'mm_post_text' => $post_text,
        'mm_move_to_forum_id' => $move_to,
        'mm_pin_state' => $pin_state,
        'mm_open_state' => $open_state,
        'mm_forum_multi_code' => $forum_multi_code,
        'mm_title_suffix' => $title_suffix,
    ];
    $map += insert_lang('mm_name', $name, 3, $GLOBALS['FORUM_DB']);
    $id = $GLOBALS['FORUM_DB']->query_insert('f_multi_moderations', $map, true);

    require_lang('cns_multi_moderations');
    log_it('ADD_MULTI_MODERATION', strval($id), $name);

    if ((addon_installed('commandr')) && (!running_script('install')) && (!get_mass_import_mode())) {
        require_code('resource_fs');
        generate_resource_fs_moniker('multi_moderation', strval($id), null, null, true);
    }

    return $id;
}
