<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    stats
 */

/**
 * Hook class.
 */
class Hook_checklist_kpis
{
    /**
     * Find items to include on the staff checklist.
     *
     * @return array An array of tuples: The task row to show, the number of seconds until it is due (or null if not on a timer), the number of things to sort out (or null if not on a queue), The name of the config option that controls the schedule (or null if no option)
     */
    public function run() : array
    {
        if (!addon_installed('stats')) {
            return [];
        }

        require_lang('stats');
        require_code('stats');

        $kpis = gather_kpis();

        $num_missed = 0;
        foreach ($kpis as $kpi) {
            list(, , , , , , , , , , $hits_target) = $kpi;
            if (!$hits_target) {
                $num_missed++;
            }
        }

        $status = ($num_missed > 0) ? 0 : 1;

        $_status = ($status == 0) ? do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_0') : do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_1');

        $task_label = do_lang_tempcode('REACH_KPI_TARGETS');

        $url = build_url(['page' => 'admin_stats', 'type' => 'kpis'], get_module_zone('admin_stats'));

        $tpl = do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM', [
            '_GUID' => '769eaf74214a8f4d5d50850c2c318c02',
            'URL' => $url,
            'STATUS' => $_status,
            'TASK' => $task_label,
        ]);
        return [[$tpl, null, $num_missed, null]];
    }
}
