<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    newsletter
 */

/**
 * Hook class.
 */
class Hook_checklist_newsletter_paused
{
    /**
     * Find items to include on the staff checklist.
     *
     * @return array An array of tuples: The task row to show, the number of seconds until it is due (or null if not on a timer), the number of things to sort out (or null if not on a queue), The name of the config option that controls the schedule (or null if no option)
     */
    public function run() : array
    {
        if (!addon_installed('newsletter')) {
            return [];
        }

        $url = build_url(['page' => 'admin_config', 'type' => 'category', 'id' => 'FEATURE'], get_module_zone('admin_config'), [], false, false, false, 'group-NEWSLETTER');
        $task = do_lang_tempcode('newsletter:NAG_UNPAUSE_NEWSLETTER', escape_html_tempcode($url));

        $status = (get_option('newsletter_paused') == '1' && $GLOBALS['SITE_DB']->query_select_value('newsletter_drip_send', 'COUNT(*)') > 0) ? 0 : 1;

        if ($status == 0) {
            return []; // Don't bloat things up
        }

        $_status = ($status == 0) ? do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_0') : do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_1');

        $tpl = do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM', ['_GUID' => 'd0ab18b027fc4b8af2cea6b55ceb2f9b', 'URL' => '', 'STATUS' => $_status, 'TASK' => $task]);

        return [[$tpl, ($status == 0) ? -1 : 0, 1, null]];
    }
}
