<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    tickets
 */

/**
 * Hook class.
 */
class Hook_checklist_tickets
{
    /**
     * Find items to include on the staff checklist.
     *
     * @return array An array of tuples: The task row to show, the number of seconds until it is due (or null if not on a timer), the number of things to sort out (or null if not on a queue), The name of the config option that controls the schedule (or null if no option)
     */
    public function run() : array
    {
        if (!addon_installed('tickets')) {
            return [];
        }

        require_lang('tickets');
        require_code('tickets');

        $outstanding = 0;

        $tickets = get_tickets(['only_open' => true], false, true);
        if ($tickets !== null) {
            foreach ($tickets as $topic) {
                if ($topic['closed'] == 0) {
                    $outstanding++;
                }
            }
        }

        if ($outstanding > 0) {
            $status = do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_0', ['_GUID' => 'db3320fdb98152a39beb0115865d028d']);
        } else {
            $status = do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_1', ['_GUID' => '0b722907fd5656e88e26429f63f78e86']);
        }

        $url = build_url(['page' => 'tickets', 'type' => 'browse'], get_module_zone('tickets'));

        $tpl = do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM', [
            '_GUID' => '8202af47a2f1d24675acbe4c6d20c8b4',
            'URL' => $url,
            'STATUS' => $status,
            'TASK' => do_lang_tempcode('SUPPORT_TICKETS'),
            'INFO' => do_lang_tempcode('NUM_QUEUE', escape_html(integer_format($outstanding, 0))),
        ]);
        return [[$tpl, null, $outstanding, null]];
    }
}
