<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    validation
 */

/**
 * Hook class.
 */
class Hook_checklist_validation
{
    /**
     * Find items to include on the staff checklist.
     *
     * @return array An array of tuples: The task row to show, the number of seconds until it is due (or null if not on a timer), the number of things to sort out (or null if not on a queue), The name of the config option that controls the schedule (or null if no option)
     */
    public function run() : array
    {
        if (!addon_installed('validation')) {
            return [];
        }

        require_code('validation');

        // Validate/delete submissions

        list($num_not_validated_1, $num_not_validated_2) = get_count_content_needing_validation();
        if ($num_not_validated_1 >= 1) {
            $status = 0;
        } else {
            $status = 1;
        }
        $_status = ($status == 0) ? do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_0') : do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM_STATUS_1');

        $url = build_url(['page' => 'admin_validation'], get_module_zone('admin_validation'));

        $tpl = do_template('BLOCK_MAIN_STAFF_CHECKLIST_ITEM', [
            '_GUID' => '48f2bc149dca356c8b6bd87092f70d3c',
            'URL' => '',
            'STATUS' => $_status,
            'TASK' => do_lang_tempcode('NAG_VALIDATE', escape_html_tempcode($url)),
            'INFO' => do_lang_tempcode('NOT_VALIDATED_ENTRIES', escape_html(integer_format($num_not_validated_1, 0)), escape_html(integer_format($num_not_validated_2, 0))),
        ]);

        return [[$tpl, null, $num_not_validated_1, null]];
    }
}
