<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    stats
 */

/**
 * Hook class.
 */
class Hook_admin_stats_redirects_google_analytics extends CMSStatsRedirect
{
    /**
     * Find metadata about stats icons that are provided by this stats hook.
     *
     * @param  boolean $for_kpi Whether this is for setting up a KPI
     * @return ?array Map of metadata (null: hook is disabled)
     */
    public function info(bool $for_kpi = false) : ?array
    {
        if (get_option('google_analytics') == '') {
            return null;
        }

        return [
            'google_analytics' => [
                'label_lang_string' => '_GOOGLE_ANALYTICS',
                'icon' => 'spare/analytics',
            ],
        ];
    }

    /**
     * Get the hook's redirect URL.
     *
     * @param  string $bucket_name The bucket name
     * @return URLPATH Redirect URL
     */
    public function get_redirect_url(string $bucket_name) : string
    {
        return 'https://google.com/analytics';
    }
}
