<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    aggregate_types
 */

/**
 * Hook class.
 */
class Hook_actionlog_aggregate_types extends Hook_actionlog
{
    /**
     * Get details of action log entry types handled by this hook.
     *
     * @return array Map of handler data in standard format
     */
    public function get_handlers() : array
    {
        if (!addon_installed('aggregate_types')) {
            return [];
        }

        require_lang('aggregate_types');

        return [
            'EDIT_AGGREGATE_TYPES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'AGGREGATE_TYPES' => '_SEARCH:admin_aggregate_types',
                    'ADD_AGGREGATE_TYPE_INSTANCE' => '_SEARCH:admin_aggregate_types:add',
                ],
            ],
            'ADD_AGGREGATE_TYPE_INSTANCE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'aggregate_type_instance',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'EDIT_THIS_AGGREGATE_TYPE_INSTANCE' => '_SEARCH:admin_aggregate_types:_edit:{ID}',
                    'ADD_AGGREGATE_TYPE_INSTANCE' => '_SEARCH:admin_aggregate_types:add',
                    'AGGREGATE_TYPES' => '_SEARCH:admin_aggregate_types',
                ],
            ],
            'EDIT_AGGREGATE_TYPE_INSTANCE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'aggregate_type_instance',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'EDIT_THIS_AGGREGATE_TYPE_INSTANCE' => '_SEARCH:admin_aggregate_types:_edit:{ID}',
                    'ADD_AGGREGATE_TYPE_INSTANCE' => '_SEARCH:admin_aggregate_types:add',
                    'AGGREGATE_TYPES' => '_SEARCH:admin_aggregate_types',
                ],
            ],
            'DELETE_AGGREGATE_TYPE_INSTANCE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'aggregate_type_instance',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_AGGREGATE_TYPE_INSTANCE' => '_SEARCH:admin_aggregate_types:add',
                ],
            ],
        ];
    }
}
