<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    chat
 */

/**
 * Hook class.
 */
class Hook_actionlog_chat extends Hook_actionlog
{
    /**
     * Get details of action log entry types handled by this hook.
     *
     * @return array Map of handler data in standard format
     */
    public function get_handlers() : array
    {
        if (!addon_installed('chat')) {
            return [];
        }

        require_lang('chat');

        return [
            'ADD_CHATROOM' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'CHATROOM' => '_SEARCH:chat:room:{ID}',
                    'EDIT_THIS_CHATROOM' => '_SEARCH:admin_chat:_edit:{ID}',
                    'ADD_CHATROOM' => '_SEARCH:admin_chat:add',
                ],
            ],
            'EDIT_CHATROOM' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'CHATROOM' => '_SEARCH:chat:room:{ID}',
                    'EDIT_THIS_CHATROOM' => '_SEARCH:admin_chat:_edit:{ID}',
                    'ADD_CHATROOM' => '_SEARCH:admin_chat:add',
                ],
            ],
            'DELETE_CHATROOM' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_CHATROOM' => '_SEARCH:admin_chat:add',
                ],
            ],
            'DELETE_ALL_CHATROOMS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'ADD_CHATROOM' => '_SEARCH:admin_chat:add',
                ],
            ],
            'EDIT_MESSAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'CHAT_MODERATION' => '_SEARCH:cms_chat',
                ],
            ],
            'DELETE_MESSAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'CHAT_MODERATION' => '_SEARCH:cms_chat',
                ],
            ],
            'DELETE_ALL_MESSAGES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'CHATROOM' => '_SEARCH:chat:room:{ID}',
                    'EDIT_THIS_CHATROOM' => '_SEARCH:admin_chat:_edit:{ID}',
                    'ADD_CHATROOM' => '_SEARCH:admin_chat:add',
                ],
            ],
            'CHAT_BAN' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => null,
                'followup_page_links' => [
                    'CHATROOM' => '_SEARCH:chat:room:{ID}',
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'CHAT_UNBAN' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => null,
                'followup_page_links' => [
                    'CHATROOM' => '_SEARCH:chat:room:{ID}',
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'BLOCK_MEMBER' => [
                'flags' => ACTIONLOG_FLAGS_NONE | ACTIONLOG_FLAG__USER_ACTION,
                'cma_hook' => 'member',
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'SUBMITTER' => ['FORUM_DRIVER__PROFILE_URL', '{ID}'],
                    'TARGET' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'UNBLOCK_MEMBER' => [
                'flags' => ACTIONLOG_FLAGS_NONE | ACTIONLOG_FLAG__USER_ACTION,
                'cma_hook' => 'member',
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'SUBMITTER' => ['FORUM_DRIVER__PROFILE_URL', '{ID}'],
                    'TARGET' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'MAKE_FRIEND' => [
                'flags' => ACTIONLOG_FLAGS_NONE | ACTIONLOG_FLAG__USER_ACTION,
                'cma_hook' => 'member',
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'SUBMITTER' => ['FORUM_DRIVER__PROFILE_URL', '{ID}'],
                    'TARGET' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'DUMP_FRIEND' => [
                'flags' => ACTIONLOG_FLAGS_NONE | ACTIONLOG_FLAG__USER_ACTION,
                'cma_hook' => 'member',
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'SUBMITTER' => ['FORUM_DRIVER__PROFILE_URL', '{ID}'],
                    'TARGET' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            '_INSTANT_MESSAGING' => [
                'flags' => ACTIONLOG_FLAGS_NONE | ACTIONLOG_FLAG__USER_ACTION,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                ],
            ],
            'CHAT' => [
                'flags' => ACTIONLOG_FLAGS_NONE | ACTIONLOG_FLAG__USER_ACTION,
                'cma_hook' => 'chat',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'CHATROOM' => '_SEARCH:chat:room:{ID}',
                ],
            ],
        ];
    }

    /**
     * Get written context for an action log entry handled by this hook.
     *
     * @param  array $actionlog_row Action log row
     * @param  array $handler_data Handler data
     * @param  ?string $identifier Identifier (null: none)
     * @return ?string Written context (null: none)
     */
    protected function get_written_context(array $actionlog_row, array $handler_data, ?string $identifier) : ?string
    {
        switch ($actionlog_row['the_type']) {
            case 'CHAT_BAN':
            case 'CHAT_UNBAN':
                $username = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_b']));
                if ($username === null) {
                    $username = '#' . $actionlog_row['param_b'];
                }

                $room_name = $GLOBALS['SITE_DB']->query_select_value_if_there('chat_rooms', 'room_name', ['id' => intval($actionlog_row['param_a'])]);
                if ($room_name === null) {
                    $room_name = '#' . $actionlog_row['param_a'];
                }

                $written_context = do_lang('SOMETHING_IN', $username, $room_name);
                return $written_context;

            case 'MAKE_FRIEND':
                $username_likes = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_a']));
                if ($username_likes === null) {
                    $username_likes = '#' . $actionlog_row['param_a'];
                }

                $username_liked = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_b']));
                if ($username_liked === null) {
                    $username_liked = '#' . $actionlog_row['param_b'];
                }

                $written_context = do_lang('MEMBER_BEFRIENDED', $username_likes, $username_liked);
                return $written_context;

            case 'DUMP_FRIEND':
                $username_likes = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_a']));
                if ($username_likes === null) {
                    $username_likes = '#' . $actionlog_row['param_a'];
                }

                $username_liked = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_b']));
                if ($username_liked === null) {
                    $username_liked = '#' . $actionlog_row['param_b'];
                }

                $written_context = do_lang('MEMBER_UNFRIENDED', $username_likes, $username_liked);
                return $written_context;

            case 'BLOCK_MEMBER':
                $username_blocker = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_a']));
                if ($username_blocker === null) {
                    $username_blocker = '#' . $actionlog_row['param_a'];
                }

                $username_blocked = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_b']));
                if ($username_blocked === null) {
                    $username_blocked = '#' . $actionlog_row['param_b'];
                }

                $written_context = do_lang('MEMBER_BLOCKED', $username_blocker, $username_blocked);
                return $written_context;

            case 'UNBLOCK_MEMBER':
                $username_blocker = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_a']));
                if ($username_blocker === null) {
                    $username_blocker = '#' . $actionlog_row['param_a'];
                }

                $username_blocked = $GLOBALS['FORUM_DRIVER']->get_username(intval($actionlog_row['param_b']));
                if ($username_blocked === null) {
                    $username_blocked = '#' . $actionlog_row['param_b'];
                }

                $written_context = do_lang('MEMBER_UNBLOCKED', $username_blocker, $username_blocked);
                return $written_context;
        }

        return parent::get_written_context($actionlog_row, $handler_data, $identifier);
    }
}
