<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_actionlog_core extends Hook_actionlog
{
    /**
     * Get details of action log entry types handled by this hook.
     *
     * @return array Map of handler data in standard format
     */
    public function get_handlers() : array
    {
        require_lang('zones');
        require_lang('addons');
        require_lang('staff_checklist');
        require_lang('cleanup');
        require_lang('config');
        require_lang('lang');
        require_lang('menus');
        require_lang('notifications');
        require_lang('permissions');
        require_lang('themes');
        require_lang('upgrade');
        require_lang('group_member_timeouts');
        require_lang('trackbacks');
        require_lang('privacy');

        return [
            'ADD_ZONE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'zone',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '{ID}:',
                    'EDIT_THIS_ZONE' => '_SEARCH:admin_zones:_edit:{ID}',
                    'ZONE_EDITOR' => '_SEARCH:admin_zones:_editor:{ID}',
                    'ADD_ZONE' => '_SEARCH:admin_zones:add',
                ],
            ],
            'EDIT_ZONE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'zone',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '{ID}:',
                    'EDIT_THIS_ZONE' => '_SEARCH:admin_zones:_edit:{ID}',
                    'ZONE_EDITOR' => '_SEARCH:admin_zones:_editor:{ID}',
                    'ADD_ZONE' => '_SEARCH:admin_zones:add',
                ],
            ],
            'DELETE_ZONE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'zone',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_ZONE' => '_SEARCH:admin_zones:add',
                ],
            ],
            'COMCODE_PAGE_EDIT' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'comcode_page',
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'VIEW' => '{1}:{0}',
                    'COMCODE_PAGE_EDIT_THIS' => '_SEARCH:cms_comcode_pages:_edit:page_link={1__EVEN_EMPTY}%3A{0__EVEN_EMPTY}:lang=' . get_site_default_lang(),
                    'COMCODE_PAGE_MANAGEMENT' => '_SEARCH:cms_comcode_pages',
                ],
            ],
            'MOVE_PAGES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                ],
            ],
            'DELETE_PAGES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => null,
                'followup_page_links' => [
                    'COMCODE_PAGE_MANAGEMENT' => '_SEARCH:cms_comcode_pages',
                ],
            ],
            'EXPORT_ADDON' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'ADDONS' => '_SEARCH:admin_addons',
                ],
            ],
            'INSTALL_ADDON' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'ADDONS' => '_SEARCH:admin_addons',
                ],
            ],
            'UNINSTALL_ADDON' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'ADDONS' => '_SEARCH:admin_addons',
                ],
            ],
            'CHECK_LIST_ADD' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'DASHBOARD' => 'adminzone:',
                ],
            ],
            'CHECK_LIST_DELETE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'DASHBOARD' => 'adminzone:',
                ],
            ],
            'CHECK_LIST_MARK_DONE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'DASHBOARD' => 'adminzone:',
                ],
            ],
            'CHECK_LIST_MARK_UNDONE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'DASHBOARD' => 'adminzone:',
                ],
            ],
            'STAFF_LINKS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'DASHBOARD' => 'adminzone:',
                ],
            ],
            'NOTES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'DASHBOARD' => 'adminzone:',
                ],
            ],
            'CLEANUP_TOOLS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'CLEANUP_TOOLS' => '_SEARCH:admin_cleanup',
                ],
            ],
            'CONFIGURATION' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => null,
                'followup_page_links' => [
                    'EDIT_LINK' => '{CONFIG_URL}',
                    'OPTION_CATEGORIES' => '_SEARCH:admin_config',
                ],
            ],
            'TRANSLATE_CODE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'TRANSLATE_CODE' => '_SEARCH:admin_lang',
                ],
            ],
            'TRANSLATE_CONTENT' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'TRANSLATE_CONTENT' => (multi_lang_content() && multi_lang()) ? '_SEARCH:admin_lang:content' : null,
                ],
            ],
            'ADD_MENU' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'menu',
                'identifier_index' => 0,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'EDIT_MENU' => '_SEARCH:admin_menus:edit:{ID}',
                ],
            ],
            'EDIT_MENU' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'menu',
                'identifier_index' => 0,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'EDIT_MENU' => '_SEARCH:admin_menus:edit:{ID}',
                ],
            ],
            'DELETE_MENU' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'menu',
                'identifier_index' => 0,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'MENU_MANAGEMENT' => '_SEARCH:admin_menus',
                ],
            ],
            'ADD_MENU_ITEM' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'menu_item',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'EDIT_MENU' => '_SEARCH:admin_menus:edit:{MENU}',
                ],
            ],
            'EDIT_MENU_ITEM' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'menu_item',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'EDIT_MENU' => '_SEARCH:admin_menus:edit:{MENU}',
                ],
            ],
            'DELETE_MENU_ITEM' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'menu_item',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'EDIT_MENU' => '_SEARCH:admin_menus',
                ],
            ],
            'NOTIFICATIONS_LOCKDOWN' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'NOTIFICATIONS_LOCKDOWN' => '_SEARCH:admin_notifications:lockdown',
                ],
            ],
            'NOTIFICATIONS_DEFAULT' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'NOTIFICATIONS_DEFAULT' => '_SEARCH:admin_notifications:default',
                ],
            ],
            'PRIVILEGES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'PRIVILEGES' => '_SEARCH:admin_permissions:privileges',
                    'PERMISSIONS_TREE' => '_SEARCH:admin_permissions',
                ],
            ],
            'PAGE_ACCESS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'PAGE_ACCESS' => '_SEARCH:admin_permissions:page',
                    'PERMISSIONS_TREE' => '_SEARCH:admin_permissions',
                ],
            ],
            'ADD_THEME' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => null,
                'followup_page_links' => [
                    'EDIT_THEME' => '_SEARCH:admin_themes:edit_theme:theme={ID}',
                    'THEMEWIZARD' => '_SEARCH:admin_themewizard',
                    'MANAGE_THEMES' => '_SEARCH:admin_themes',
                ],
            ],
            'EDIT_THEME' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'EDIT_THEME' => '_SEARCH:admin_themes:edit_theme:theme={ID}',
                    'MANAGE_THEMES' => '_SEARCH:admin_themes',
                ],
            ],
            'DELETE_THEME' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => null,
                'followup_page_links' => [
                    'MANAGE_THEMES' => '_SEARCH:admin_themes',
                ],
            ],
            'COPY_THEME' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'EDIT_THEME' => '_SEARCH:admin_themes:edit_theme:theme={ID}',
                    'MANAGE_THEMES' => '_SEARCH:admin_themes',
                ],
            ],
            'RENAME_THEME' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 1,
                'written_context_index' => null,
                'followup_page_links' => [
                    'EDIT_THEME' => '_SEARCH:admin_themes:edit_theme:theme={ID}',
                    'MANAGE_THEMES' => '_SEARCH:admin_themes',
                ],
            ],
            'EDIT_TEMPLATES' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'EDIT_TEMPLATE' => '_SEARCH:admin_themes:edit_template:f0file={0}:theme={1}',
                    'MANAGE_THEMES' => '_SEARCH:admin_themes',
                ],
            ],
            'ADD_THEME_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'EDIT_THEME_IMAGE' => '_SEARCH:admin_themes:edit_image:{ID}:theme={1}:lang=' . get_site_default_lang(),
                    'ADD_THEME_IMAGE' => '_SEARCH:admin_themes:add_image:theme={1}',
                ],
            ],
            'EDIT_THEME_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'EDIT_THEME_IMAGE' => '_SEARCH:admin_themes:edit_image:{ID}:theme={1}:lang=' . get_site_default_lang(),
                    'ADD_THEME_IMAGE' => '_SEARCH:admin_themes:add_image:theme={1}',
                ],
            ],
            'DELETE_THEME_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => 0,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'ADD_THEME_IMAGE' => '_SEARCH:admin_themes:add_image:theme={1}',
                ],
            ],
            'DELETE_TRACKBACKS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'MANAGE_TRACKBACKS' => '_SEARCH:admin_trackbacks',
                ],
            ],
            'GROUP_MEMBER_TIMEOUTS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'GROUP_MEMBER_TIMEOUTS' => '_SEARCH:admin_group_member_timeouts',
                ],
            ],
            'UPGRADER_OPEN_SITE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'UPGRADER_UPGRADER_TITLE' => '_SEARCH:admin_config:upgrader',
                ],
            ],
            'UPGRADER_CLOSE_SITE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'UPGRADER_UPGRADER_TITLE' => '_SEARCH:admin_config:upgrader',
                ],
            ],
            'UPGRADER_DOWNLOAD' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'UPGRADER_UPGRADER_TITLE' => '_SEARCH:admin_config:upgrader',
                ],
            ],
            'UPGRADER_DATABASE_UPGRADE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'UPGRADER_UPGRADER_TITLE' => '_SEARCH:admin_config:upgrader',
                ],
            ],
            'PERSONAL_DATA_DOWNLOAD' => [
                'flags' => ACTIONLOG_FLAG__GDPR,
                'cma_hook' => 'member',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'PERSONAL_DATA_PURGING' => [
                'flags' => ACTIONLOG_FLAG__GDPR,
                'cma_hook' => 'member',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{1}'],
                ],
            ],
            'ACCESSED_ADMIN_ZONE' => [
                'flags' => ACTIONLOG_FLAG__GDPR,
                'cma_hook' => 'zone',
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'VIEW' => 'adminzone:',
                ],
            ],
        ];
    }

    /**
     * Get written context for an action log entry handled by this hook.
     *
     * @param  array $actionlog_row Action log row
     * @param  array $handler_data Handler data
     * @param  ?string $identifier Identifier (null: none)
     * @return ?string Written context (null: none)
     */
    protected function get_written_context(array $actionlog_row, array $handler_data, ?string $identifier) : ?string
    {
        switch ($actionlog_row['the_type']) {
            case 'CONFIGURATION':
                $_hook = 'hooks/systems/config/' . $identifier;
                if ((!is_file(get_file_base() . '/sources/' . $_hook . '.php')) && (!is_file(get_file_base() . '/sources_custom/' . $_hook . '.php'))) {
                    return $identifier;
                }
                require_code('hooks/systems/config/' . filter_naughty_harsh($identifier));
                $ob = object_factory('Hook_config_' . filter_naughty_harsh($identifier), true);
                if ($ob === null) {
                    return $identifier;
                }
                $details = $ob->get_details();
                return do_lang($details['human_name']);

            case 'ADD_THEME':
            case 'DELETE_THEME':
                $theme = $actionlog_row['param_a'];
                $path = get_custom_file_base() . '/themes/' . $theme . '/theme.ini';
                if (!is_file($path)) {
                    $path = get_file_base() . '/themes/' . $theme . '/theme.ini';
                }
                if (is_file($path)) {
                    $details = cms_parse_ini_file_fast($path);
                    if (array_key_exists('title', $details)) {
                        return $details['title'];
                    }
                }
                return $theme;

            case 'COPY_THEME':
            case 'RENAME_THEME':
                $theme = $actionlog_row['param_b'];
                $path = get_custom_file_base() . '/themes/' . $theme . '/theme.ini';
                if (!is_file($path)) {
                    $path = get_file_base() . '/themes/' . $theme . '/theme.ini';
                }
                if (is_file($path)) {
                    $details = cms_parse_ini_file_fast($path);
                    if (array_key_exists('title', $details)) {
                        return $details['title'];
                    }
                }
                return $theme;

            case 'CLEANUP_TOOLS':
                $hook = $actionlog_row['param_a'];
                if ($hook != '') {
                    $_hook = 'hooks/systems/cleanup/' . $hook;
                    if ((!is_file(get_file_base() . '/sources/' . $_hook . '.php')) && (!is_file(get_file_base() . '/sources_custom/' . $_hook . '.php'))) {
                        return $hook;
                    }
                    require_code($_hook);
                    $ob = object_factory('Hook_cleanup_' . $hook, true);
                    if ($ob === null) {
                        return $hook;
                    }
                    $info = $ob->info();
                    if ($info === null) {
                        return $hook;
                    }
                    return $info['title']->evaluate();
                }
                break;

            case 'COMCODE_PAGE_EDIT':
                $written_context = $actionlog_row['param_b'] . ':' . $actionlog_row['param_a'];
                return $written_context;

            case 'EDIT_CSS':
                $written_context = do_lang('SOMETHING_IN', $actionlog_row['param_b'], $actionlog_row['param_a']);
                return $written_context;

            case 'EDIT_TEMPLATES':
                $written_context = do_lang('SOMETHING_IN', $actionlog_row['param_a'], $actionlog_row['param_b']);
                return $written_context;

            case 'MOVE_PAGES':
                $written_context = do_lang('SOMETHING_TO', $actionlog_row['param_a'], $actionlog_row['param_b']);
                return $written_context;
        }

        return parent::get_written_context($actionlog_row, $handler_data, $identifier);
    }

    /**
     * Get details of action log entry types handled by this hook.
     *
     * @param  array $actionlog_row Action log row
     * @param  ?string $identifier The identifier associated with this action log entry (null: unknown / none)
     * @param  ?string $written_context The written context associated with this action log entry (null: unknown / none)
     * @param  array $bindings Default bindings
     */
    protected function get_extended_actionlog_bindings(array $actionlog_row, ?string $identifier, ?string $written_context, array &$bindings)
    {
        switch ($actionlog_row['the_type']) {
            case 'CONFIGURATION':
                if ($identifier !== null) {
                    require_code('config2');
                    $url = config_option_url($identifier);
                    $bindings += [
                        'CONFIG_URL' => ($url === null) ? static_evaluate_tempcode(build_url(['page' => ''], '')) : static_evaluate_tempcode($url),
                    ];
                }
                break;

            case 'ADD_MENU_ITEM':
            case 'EDIT_MENU_ITEM':
                if ($identifier !== null) {
                    $menu = $GLOBALS['SITE_DB']->query_select_value_if_there('menu_items', 'i_menu', ['id' => intval($identifier)]);
                    if ($menu !== null) {
                        $bindings += [
                            'MENU' => $menu,
                        ];
                    }
                }
                break;
        }
    }
}
