<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    galleries
 */

/**
 * Hook class.
 */
class Hook_actionlog_galleries extends Hook_actionlog
{
    /**
     * Get details of action log entry types handled by this hook.
     *
     * @return array Map of handler data in standard format
     */
    public function get_handlers() : array
    {
        if (!addon_installed('galleries')) {
            return [];
        }

        require_lang('galleries');

        return [
            'GALLERY_IMPORT' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'gallery',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:browse:{ID}',
                    'EDIT_THIS_GALLERY' => '_SEARCH:cms_galleries:_edit_category:{ID}',
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add:cat={ID}',
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other:cat={ID}',
                ],
            ],
            'ADD_GALLERY' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'gallery',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:browse:{ID}',
                    'EDIT_THIS_GALLERY' => '_SEARCH:cms_galleries:_edit_category:{ID}',
                    'ADD_GALLERY' => '_SEARCH:cms_galleries:add_category',
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add:cat={ID}',
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other:cat={ID}',
                ],
            ],
            'EDIT_GALLERY' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'gallery',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:browse:{ID}',
                    'EDIT_THIS_GALLERY' => '_SEARCH:cms_galleries:_edit_category:{ID}',
                    'ADD_GALLERY' => '_SEARCH:cms_galleries:add_category',
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add:cat={ID}',
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other:cat={ID}',
                ],
            ],
            'DELETE_GALLERY' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'gallery',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_GALLERY' => '_SEARCH:cms_galleries:add_category',
                ],
            ],
            'ADD_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'image',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:image:{ID}',
                    'EDIT_THIS_IMAGE' => '_SEARCH:cms_galleries:_edit:{ID}',
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add:cat={CAT,OPTIONAL}',
                ],
            ],
            'VALIDATE_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'image',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:image:{ID}',
                    'EDIT_THIS_IMAGE' => '_SEARCH:cms_galleries:_edit:{ID}',
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add:cat={CAT,OPTIONAL}',
                ],
            ],
            'EDIT_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'image',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:image:{ID}',
                    'EDIT_THIS_IMAGE' => '_SEARCH:cms_galleries:_edit:{ID}',
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add:cat={CAT,OPTIONAL}',
                ],
            ],
            'DELETE_IMAGE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'image',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_IMAGE' => '_SEARCH:cms_galleries:add',
                ],
            ],
            'ADD_VIDEO' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'video',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:video:{ID}',
                    'EDIT_THIS_VIDEO' => '_SEARCH:cms_galleries:_edit_other:{ID}',
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other:cat={CAT,OPTIONAL}',
                ],
            ],
            'VALIDATE_VIDEO' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'video',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:video:{ID}',
                    'EDIT_THIS_VIDEO' => '_SEARCH:cms_galleries:_edit_other:{ID}',
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other:cat={CAT,OPTIONAL}',
                ],
            ],
            'EDIT_VIDEO' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'video',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:galleries:video:{ID}',
                    'EDIT_THIS_VIDEO' => '_SEARCH:cms_galleries:_edit_other:{ID}',
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other:cat={CAT,OPTIONAL}',
                ],
            ],
            'DELETE_VIDEO' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'video',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_VIDEO' => '_SEARCH:cms_galleries:add_other',
                ],
            ],
        ];
    }

    /**
     * Get details of action log entry types handled by this hook.
     *
     * @param  array $actionlog_row Action log row
     * @param  ?string $identifier The identifier associated with this action log entry (null: unknown / none)
     * @param  ?string $written_context The written context associated with this action log entry (null: unknown / none)
     * @param  array $bindings Default bindings
     */
    protected function get_extended_actionlog_bindings(array $actionlog_row, ?string $identifier, ?string $written_context, array &$bindings)
    {
        switch ($actionlog_row['the_type']) {
            case 'ADD_IMAGE':
            case 'EDIT_IMAGE':
                if ($identifier !== null) {
                    $cat = $GLOBALS['SITE_DB']->query_select_value_if_there('images', 'cat', ['id' => intval($identifier)]);
                    if ($cat !== null) {
                        $bindings += [
                            'CAT' => $cat,
                        ];
                    }
                }
                break;

            case 'ADD_VIDEO':
            case 'EDIT_VIDEO':
                if ($identifier !== null) {
                    $cat = $GLOBALS['SITE_DB']->query_select_value_if_there('videos', 'cat', ['id' => intval($identifier)]);
                    if ($cat !== null) {
                        $bindings += [
                            'CAT' => $cat,
                        ];
                    }
                }
                break;
        }
    }
}
