<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    news
 */

/**
 * Hook class.
 */
class Hook_actionlog_news extends Hook_actionlog
{
    /**
     * Get details of action log entry types handled by this hook.
     *
     * @return array Map of handler data in standard format
     */
    public function get_handlers() : array
    {
        if (!addon_installed('news')) {
            return [];
        }

        require_lang('news');

        return [
            'ADD_NEWS_CATEGORY' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news_category',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:news:browse:{ID}',
                    'EDIT_THIS_NEWS_CATEGORY' => '_SEARCH:cms_news:_edit_category:{ID}',
                    'ADD_NEWS_CATEGORY' => '_SEARCH:cms_news:add_category',
                    'ADD_NEWS' => '_SEARCH:cms_news:add:cat={ID}',
                ],
            ],
            'EDIT_NEWS_CATEGORY' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news_category',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:news:browse:{ID}',
                    'EDIT_THIS_NEWS_CATEGORY' => '_SEARCH:cms_news:_edit_category:{ID}',
                    'ADD_NEWS_CATEGORY' => '_SEARCH:cms_news:add_category',
                    'ADD_NEWS' => '_SEARCH:cms_news:add:cat={ID}',
                ],
            ],
            'DELETE_NEWS_CATEGORY' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news_category',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_NEWS_CATEGORY' => '_SEARCH:cms_news:add_category',
                ],
            ],
            'ADD_NEWS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:news:view:{ID}',
                    'EDIT_THIS_NEWS' => '_SEARCH:cms_news:_edit:{ID}',
                    'ADD_NEWS' => '_SEARCH:cms_news:add:cat={CAT,OPTIONAL}',
                ],
            ],
            'EDIT_NEWS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:news:view:{ID}',
                    'EDIT_THIS_NEWS' => '_SEARCH:cms_news:_edit:{ID}',
                    'ADD_NEWS' => '_SEARCH:cms_news:add:cat={CAT,OPTIONAL}',
                ],
            ],
            'VALIDATE_NEWS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW' => '_SEARCH:news:view:{ID}',
                    'EDIT_THIS_NEWS' => '_SEARCH:cms_news:_edit:{ID}',
                    'ADD_NEWS' => '_SEARCH:cms_news:add:cat={CAT,OPTIONAL}',
                ],
            ],
            'DELETE_NEWS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'news',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'ADD_NEWS' => '_SEARCH:cms_news:add',
                ],
            ],
            'IMPORT_NEWS' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'MANAGE_NEWS' => '_SEARCH:cms_news',
                ],
            ],
        ];
    }

    /**
     * Get details of action log entry types handled by this hook.
     *
     * @param  array $actionlog_row Action log row
     * @param  ?string $identifier The identifier associated with this action log entry (null: unknown / none)
     * @param  ?string $written_context The written context associated with this action log entry (null: unknown / none)
     * @param  array $bindings Default bindings
     */
    protected function get_extended_actionlog_bindings(array $actionlog_row, ?string $identifier, ?string $written_context, array &$bindings)
    {
        switch ($actionlog_row['the_type']) {
            case 'ADD_NEWS':
            case 'EDIT_NEWS':
                if ($identifier !== null) {
                    $category_id = $GLOBALS['SITE_DB']->query_select_value_if_there('news', 'news_category', ['id' => intval($identifier)]);
                    if ($category_id !== null) {
                        $bindings += [
                            'CAT' => strval($category_id),
                        ];
                    }
                }
                break;
        }
    }
}
