<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    securitylogging
 */

/**
 * Hook class.
 */
class Hook_actionlog_securitylogging extends Hook_actionlog
{
    /**
     * Get details of action log entry types handled by this hook.
     *
     * @return array Map of handler data in standard format
     */
    public function get_handlers() : array
    {
        if (!addon_installed('securitylogging')) {
            return [];
        }

        require_lang('submitban');
        require_lang('lookup');
        require_lang('config');

        return [
            'IP_BANNED' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'IP_BANS' => '_SEARCH:admin_ip_ban',
                    'VIEW_ACTIONLOGS' => '_SEARCH:admin_actionlog',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'IP_UNBANNED' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'IP_BANS' => '_SEARCH:admin_ip_ban',
                    'VIEW_ACTIONLOGS' => '_SEARCH:admin_actionlog',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'SYNDICATED_IP_BAN' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'IP_BANS' => '_SEARCH:admin_ip_ban',
                    'VIEW_ACTIONLOGS' => '_SEARCH:admin_actionlog',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'MADE_IP_BANNABLE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'IP_BANS' => '_SEARCH:admin_ip_ban',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'MADE_IP_UNBANNABLE' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => 0,
                'followup_page_links' => [
                    'IP_BANS' => '_SEARCH:admin_ip_ban',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'SUBMITTER_BANNED' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'member',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{ID}'],
                    'VIEW_ACTIONLOGS' => '_SEARCH:admin_actionlog',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'SUBMITTER_UNBANNED' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => 'member',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{ID}'],
                    'VIEW_ACTIONLOGS' => '_SEARCH:admin_actionlog',
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'INVESTIGATE_USER' => [
                'flags' => ACTIONLOG_FLAG__GDPR,
                'cma_hook' => 'member',
                'identifier_index' => 0,
                'written_context_index' => 1,
                'followup_page_links' => [
                    'VIEW_PROFILE' => ['FORUM_DRIVER__PROFILE_URL', '{0}'],
                    'INVESTIGATE_USER' => '_SEARCH:admin_lookup:results:param={0}',
                ],
            ],
            'ADVANCED_BANNING' => [
                'flags' => ACTIONLOG_FLAGS_NONE,
                'cma_hook' => null,
                'identifier_index' => null,
                'written_context_index' => null,
                'followup_page_links' => [
                    'BREADCRUMB_OVERRIDES' => '_SEARCH:admin_config:advanced_banning',
                ],
            ],
        ];
    }
}
