<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    cns_thematic_avatars
 */

/**
 * Hook class.
 */
class Hook_addon_registry_cns_thematic_avatars
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the version of the software this addon is for (used in generating the TAR filename).
     *
     * @return float Version number
     */
    public function get_version() : float
    {
        return cms_version_number();
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'Graphical';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'A selection of avatars for Conversr';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [
            'tut_members',
        ];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => [
                'cns_member_avatars',
            ],
            'recommends' => [],
            'conflicts_with' => [],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/admin/component.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'sources/hooks/systems/addon_registry/cns_thematic_avatars.php',
            'themes/default/images/cns_default_avatars/default_set/thematic/animals.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/books.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/business.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/chess.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/food.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/games.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/index.html',
            'themes/default/images/cns_default_avatars/default_set/thematic/money.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/music.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/nature.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/outdoors.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/space.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/sports.png',
            'themes/default/images/cns_default_avatars/default_set/thematic/tech.png',
        ];
    }
}
