<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_cns
 */

/**
 * Hook class.
 */
class Hook_addon_registry_core_cns
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the version of the software this addon is for (used in generating the TAR filename).
     *
     * @return float Version number
     */
    public function get_version() : float
    {
        return cms_version_number();
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'Architecture';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'The ' . brand_name() . ' member/usergroup system.';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [
            'tut_members',
            'tut_adv_members',
            'tut_users',
            'tut_httpauth',
            'tut_emoticons',
        ];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => [],
            'recommends' => [],
            'conflicts_with' => [],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/menu/social/profile.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'adminzone/pages/modules/admin_cns_emoticons.php',
            'adminzone/pages/modules/admin_cns_groups.php',
            'adminzone/pages/modules/admin_cns_members.php',
            'adminzone/pages/modules/admin_cns_merge_members.php',
            'data/approve_ip.php',
            'data/username_check.php',
            'lang/EN/cns.ini',
            'lang/EN/cns_autofill.ini',
            'lang/EN/cns_components.ini',
            'lang/EN/cns_config.ini',
            'lang/EN/cns_lost_password.ini',
            'lang/EN/cns_lurkers.ini',
            'lang/EN/cns_mailinglists.ini',
            'lang/EN/cns_member_directory.ini',
            'lang/EN/cns_polls.ini',
            'lang/EN/cns_ranks.ini',
            'lang/EN/cns_special_cpf.ini',
            'lang/EN/password_rules.ini',
            'pages/modules/join.php',
            'pages/modules/lost_password.php',
            'site/pages/modules/groups.php',
            'site/pages/modules/members.php',
            'site/pages/modules/users_online.php',
            'sources/blocks/main_join.php',
            'sources/blocks/main_members.php',
            'sources/cns_field_editability.php',
            'sources/cns_forum_driver_helper.php',
            'sources/cns_forum_driver_helper_auth.php',
            'sources/cns_forum_driver_helper_install.php',
            'sources/cns_forums.php',
            'sources/cns_forums2.php',
            'sources/cns_forums_action.php',
            'sources/cns_forums_action2.php',
            'sources/cns_general.php',
            'sources/cns_general_action.php',
            'sources/cns_general_action2.php',
            'sources/cns_groups.php',
            'sources/cns_groups2.php',
            'sources/cns_groups_action.php',
            'sources/cns_groups_action2.php',
            'sources/cns_install.php',
            'sources/cns_join.php',
            'sources/cns_lost_password.php',
            'sources/cns_members.php',
            'sources/cns_members2.php',
            'sources/cns_members_action.php',
            'sources/cns_members_action2.php',
            'sources/cns_moderation.php',
            'sources/cns_moderation_action.php',
            'sources/cns_moderation_action2.php',
            'sources/cns_notifications.php',
            'sources/cns_polls.php',
            'sources/cns_polls_action.php',
            'sources/cns_polls_action2.php',
            'sources/cns_polls_action3.php',
            'sources/cns_popups.php',
            'sources/cns_posts.php',
            'sources/cns_posts2.php',
            'sources/cns_posts_action.php',
            'sources/cns_posts_action2.php',
            'sources/cns_posts_action3.php',
            'sources/cns_profiles.php',
            'sources/cns_topics.php',
            'sources/cns_topics_action.php',
            'sources/cns_topics_action2.php',
            'sources/forum/cns.php',
            'sources/hooks/blocks/main_staff_checklist/usergroup_membership.php',
            'sources/hooks/modules/admin_import/emoticons.php',
            'sources/hooks/modules/admin_import/spreadsheet_members.php',
            'sources/hooks/modules/admin_setupwizard/core_cns.php',
            'sources/hooks/modules/admin_stats/cns_members.php',
            'sources/hooks/modules/admin_stats/invites.php',
            'sources/hooks/modules/members/.htaccess',
            'sources/hooks/modules/members/index.html',
            'sources/hooks/modules/search/cns_members.php',
            'sources/hooks/systems/actionlog/core_cns.php',
            'sources/hooks/systems/addon_registry/core_cns.php',
            'sources/hooks/systems/cleanup/cns_members.php',
            'sources/hooks/systems/cleanup/rebuild_all_cpf_indices.php',
            'sources/hooks/systems/cns_auth/converge.php',
            'sources/hooks/systems/cns_auth/index.html',
            'sources/hooks/systems/cns_auth/phpbb3.php',
            'sources/hooks/systems/cns_auth/smf.php',
            'sources/hooks/systems/cns_auth/smf2.php',
            'sources/hooks/systems/cns_auth/vb3.php',
            'sources/hooks/systems/cns_cpf_filter/.htaccess',
            'sources/hooks/systems/cns_cpf_filter/index.html',
            'sources/hooks/systems/commandr_fs/emoticons.php',
            'sources/hooks/systems/commandr_fs/groups.php',
            'sources/hooks/systems/commandr_fs/members.php',
            'sources/hooks/systems/commandr_fs_extended_member/known_login_ips.php',
            'sources/hooks/systems/config/allow_alpha_search.php',
            'sources/hooks/systems/config/complex_privacy_options.php',
            'sources/hooks/systems/config/decryption_key.php',
            'sources/hooks/systems/config/display_name_generator.php',
            'sources/hooks/systems/config/dobs.php',
            'sources/hooks/systems/config/email_confirm_join.php',
            'sources/hooks/systems/config/enable_birthdays.php',
            'sources/hooks/systems/config/enable_highlight_name.php',
            'sources/hooks/systems/config/enable_language_selection.php',
            'sources/hooks/systems/config/enable_privacy_tab.php',
            'sources/hooks/systems/config/enable_timezones.php',
            'sources/hooks/systems/config/enable_user_online_groups.php',
            'sources/hooks/systems/config/encryption_key.php',
            'sources/hooks/systems/config/finish_profile.php',
            'sources/hooks/systems/config/hot_topic_definition.php',
            'sources/hooks/systems/config/httpauth_is_enabled.php',
            'sources/hooks/systems/config/important_groups_per_page.php',
            'sources/hooks/systems/config/invites_per_day.php',
            'sources/hooks/systems/config/is_on_automatic_mark_topic_read.php',
            'sources/hooks/systems/config/is_on_invisibility.php',
            'sources/hooks/systems/config/is_on_invites.php',
            'sources/hooks/systems/config/is_on_parental_consent.php',
            'sources/hooks/systems/config/is_on_post_map.php',
            'sources/hooks/systems/config/is_on_timezone_detection.php',
            'sources/hooks/systems/config/join_declarations.php',
            'sources/hooks/systems/config/maximum_password_length.php',
            'sources/hooks/systems/config/maximum_username_length.php',
            'sources/hooks/systems/config/md_default_sort_order.php',
            'sources/hooks/systems/config/member_email_receipt_configurability.php',
            'sources/hooks/systems/config/members_per_page.php',
            'sources/hooks/systems/config/minimum_password_length.php',
            'sources/hooks/systems/config/minimum_password_strength.php',
            'sources/hooks/systems/config/minimum_username_length.php',
            'sources/hooks/systems/config/new_member_default_email_message.php',
            'sources/hooks/systems/config/new_member_default_email_subject.php',
            'sources/hooks/systems/config/new_member_default_temporary_password.php',
            'sources/hooks/systems/config/normal_groups_per_page.php',
            'sources/hooks/systems/config/one_per_email_address.php',
            'sources/hooks/systems/config/page_after_join.php',
            'sources/hooks/systems/config/parental_consent_age.php',
            'sources/hooks/systems/config/password_change_days.php',
            'sources/hooks/systems/config/password_expiry_days.php',
            'sources/hooks/systems/config/password_reset_minutes.php',
            'sources/hooks/systems/config/password_reset_privacy.php',
            'sources/hooks/systems/config/password_reset_process.php',
            'sources/hooks/systems/config/primary_members_per_page.php',
            'sources/hooks/systems/config/privacy_fax.php',
            'sources/hooks/systems/config/privacy_postal_address.php',
            'sources/hooks/systems/config/probation_usergroup.php',
            'sources/hooks/systems/config/prohibit_password_whitespace.php',
            'sources/hooks/systems/config/prohibit_username_whitespace.php',
            'sources/hooks/systems/config/require_new_member_validation.php',
            'sources/hooks/systems/config/restricted_usernames.php',
            'sources/hooks/systems/config/search_cns_members.php',
            'sources/hooks/systems/config/secondary_members_per_page.php',
            'sources/hooks/systems/config/show_empty_cpfs.php',
            'sources/hooks/systems/config/show_first_join_page.php',
            'sources/hooks/systems/config/staff_email_receipt_configurability.php',
            'sources/hooks/systems/config/use_joindate.php',
            'sources/hooks/systems/config/use_lastondate.php',
            'sources/hooks/systems/config/username_profile_links.php',
            'sources/hooks/systems/config/valid_email_domains.php',
            'sources/hooks/systems/content_meta_aware/group.php',
            'sources/hooks/systems/content_meta_aware/member.php',
            'sources/hooks/systems/content_meta_aware/post.php',
            'sources/hooks/systems/content_meta_aware/topic.php',
            'sources/hooks/systems/cron/cns_birthdays.php',
            'sources/hooks/systems/cron/cns_confirm_reminder.php',
            'sources/hooks/systems/disposable_values/cns_member_count.php',
            'sources/hooks/systems/disposable_values/cns_newest_member_id.php',
            'sources/hooks/systems/disposable_values/cns_newest_member_username.php',
            'sources/hooks/systems/disposable_values/cns_post_count.php',
            'sources/hooks/systems/disposable_values/cns_topic_count.php',
            'sources/hooks/systems/notifications/cns.php',
            'sources/hooks/systems/notifications/cns_staff.php',
            'sources/hooks/systems/points/member__join.php',
            'sources/hooks/systems/points/member__visit.php',
            'sources/hooks/systems/preview/cns_emoticon.php',
            'sources/hooks/systems/privacy/core_cns.php',
            'sources/hooks/systems/profiles_tabs/.htaccess',
            'sources/hooks/systems/profiles_tabs/about.php',
            'sources/hooks/systems/profiles_tabs/edit.php',
            'sources/hooks/systems/profiles_tabs/index.html',
            'sources/hooks/systems/profiles_tabs/rank.php',
            'sources/hooks/systems/profiles_tabs_edit/.htaccess',
            'sources/hooks/systems/profiles_tabs_edit/delete.php',
            'sources/hooks/systems/profiles_tabs_edit/index.html',
            'sources/hooks/systems/profiles_tabs_edit/profile.php',
            'sources/hooks/systems/profiles_tabs_edit/settings.php',
            'sources/hooks/systems/realtime_rain/cns.php',
            'sources/hooks/systems/resource_meta_aware/emoticon.php',
            'sources/hooks/systems/rss/cns_birthdays.php',
            'sources/hooks/systems/rss/cns_members.php',
            'sources/hooks/systems/sitemap/group.php',
            'sources/hooks/systems/sitemap/member.php',
            'sources/hooks/systems/snippets/exists_email.php',
            'sources/hooks/systems/snippets/exists_emoticon.php',
            'sources/hooks/systems/snippets/exists_usergroup.php',
            'sources/hooks/systems/snippets/invite_missing.php',
            'sources/hooks/systems/snippets/member_tooltip.php',
            'sources/hooks/systems/snippets/password_strength.php',
            'sources/hooks/systems/snippets/profile_tab.php',
            'sources/hooks/systems/snippets/pt_rules.php',
            'sources/hooks/systems/symbols/CPF_LIST.php',
            'sources/hooks/systems/tasks/cns_delete_member.php',
            'sources/hooks/systems/tasks/cns_members_recache.php',
            'sources/hooks/systems/tasks/export_members.php',
            'sources/hooks/systems/tasks/import_members.php',
            'sources/hooks/systems/tasks/rebuild_all_cpf_indices.php',
            'sources/password_rules.php',
            'sources/two_factor.php',
            'sources_custom/hooks/modules/members/.htaccess',
            'sources_custom/hooks/modules/members/index.html',
            'sources_custom/hooks/systems/cns_auth/index.html',
            'sources_custom/hooks/systems/cns_cpf_filter/.htaccess',
            'sources_custom/hooks/systems/cns_cpf_filter/index.html',
            'sources_custom/hooks/systems/profiles_tabs/.htaccess',
            'sources_custom/hooks/systems/profiles_tabs/index.html',
            'sources_custom/hooks/systems/profiles_tabs_edit/.htaccess',
            'sources_custom/hooks/systems/profiles_tabs_edit/index.html',
            'themes/default/css/_cns_colours.css',
            'themes/default/css/cns.css',
            'themes/default/css/cns_admin.css',
            'themes/default/css/cns_editor.css',
            'themes/default/css/cns_footer.css',
            'themes/default/css/cns_header.css',
            'themes/default/css/cns_member_directory.css',
            'themes/default/css/cns_member_profiles.css',
            'themes/default/css/stepper.css',
            'themes/default/images/EN/cns_rank_images/admin.svg',
            'themes/default/images/EN/cns_rank_images/index.html',
            'themes/default/images/EN/cns_rank_images/mod.svg',
            'themes/default/images/cns_default_avatars/default.png',
            'themes/default/images/cns_emoticons/angel.png',
            'themes/default/images/cns_emoticons/angry.png',
            'themes/default/images/cns_emoticons/birthday.png',
            'themes/default/images/cns_emoticons/blink.gif',
            'themes/default/images/cns_emoticons/blink.gif.png',
            'themes/default/images/cns_emoticons/blush.png',
            'themes/default/images/cns_emoticons/boat.png',
            'themes/default/images/cns_emoticons/cheeky.png',
            'themes/default/images/cns_emoticons/christmas.png',
            'themes/default/images/cns_emoticons/confused.png',
            'themes/default/images/cns_emoticons/constipated.png',
            'themes/default/images/cns_emoticons/cool.png',
            'themes/default/images/cns_emoticons/cowboy.png',
            'themes/default/images/cns_emoticons/cry.png',
            'themes/default/images/cns_emoticons/cyborg.png',
            'themes/default/images/cns_emoticons/depressed.png',
            'themes/default/images/cns_emoticons/devil.gif',
            'themes/default/images/cns_emoticons/devil.gif.png',
            'themes/default/images/cns_emoticons/drool.png',
            'themes/default/images/cns_emoticons/dry.png',
            'themes/default/images/cns_emoticons/fight.png',
            'themes/default/images/cns_emoticons/fishing.png',
            'themes/default/images/cns_emoticons/glee.png',
            'themes/default/images/cns_emoticons/goodbye.png',
            'themes/default/images/cns_emoticons/grin.png',
            'themes/default/images/cns_emoticons/guitar.gif',
            'themes/default/images/cns_emoticons/guitar.gif.png',
            'themes/default/images/cns_emoticons/hand.png',
            'themes/default/images/cns_emoticons/hippie.png',
            'themes/default/images/cns_emoticons/hug.gif',
            'themes/default/images/cns_emoticons/hug.gif.png',
            'themes/default/images/cns_emoticons/idea.png',
            'themes/default/images/cns_emoticons/index.html',
            'themes/default/images/cns_emoticons/king.png',
            'themes/default/images/cns_emoticons/kiss.png',
            'themes/default/images/cns_emoticons/lol.gif',
            'themes/default/images/cns_emoticons/lol.gif.png',
            'themes/default/images/cns_emoticons/mellow.png',
            'themes/default/images/cns_emoticons/nerd.png',
            'themes/default/images/cns_emoticons/ninja2.gif',
            'themes/default/images/cns_emoticons/ninja2.gif.png',
            'themes/default/images/cns_emoticons/nod.gif',
            'themes/default/images/cns_emoticons/nod.gif.png',
            'themes/default/images/cns_emoticons/none.png',
            'themes/default/images/cns_emoticons/offtopic.png',
            'themes/default/images/cns_emoticons/party.png',
            'themes/default/images/cns_emoticons/ph34r.png',
            'themes/default/images/cns_emoticons/puppyeyes.png',
            'themes/default/images/cns_emoticons/reallybadday.gif',
            'themes/default/images/cns_emoticons/reallybadday.gif.png',
            'themes/default/images/cns_emoticons/rockon.gif',
            'themes/default/images/cns_emoticons/rockon.gif.png',
            'themes/default/images/cns_emoticons/rolleyes.gif',
            'themes/default/images/cns_emoticons/rolleyes.gif.png',
            'themes/default/images/cns_emoticons/sad.png',
            'themes/default/images/cns_emoticons/sarcy.png',
            'themes/default/images/cns_emoticons/shake.gif',
            'themes/default/images/cns_emoticons/shake.gif.png',
            'themes/default/images/cns_emoticons/shocked.png',
            'themes/default/images/cns_emoticons/shutup.gif',
            'themes/default/images/cns_emoticons/shutup.gif.png',
            'themes/default/images/cns_emoticons/sick.png',
            'themes/default/images/cns_emoticons/sinner.png',
            'themes/default/images/cns_emoticons/smile.png',
            'themes/default/images/cns_emoticons/thumbs.png',
            'themes/default/images/cns_emoticons/tired.gif',
            'themes/default/images/cns_emoticons/tired.gif.png',
            'themes/default/images/cns_emoticons/upsidedown.png',
            'themes/default/images/cns_emoticons/whew.gif',
            'themes/default/images/cns_emoticons/whew.gif.png',
            'themes/default/images/cns_emoticons/whistle.png',
            'themes/default/images/cns_emoticons/wink.png',
            'themes/default/images/cns_emoticons/wub.png',
            'themes/default/images/cns_emoticons/zzz.png',
            'themes/default/images/cns_post_map/index.html',
            'themes/default/images/cns_post_map/last_mesg_level.svg',
            'themes/default/images/cns_post_map/mesg_level.svg',
            'themes/default/images/cns_post_map/middle_mesg_level.svg',
            'themes/default/images/cns_rank_images/0.svg',
            'themes/default/images/cns_rank_images/1.svg',
            'themes/default/images/cns_rank_images/2.svg',
            'themes/default/images/cns_rank_images/3.svg',
            'themes/default/images/cns_rank_images/4.svg',
            'themes/default/images/cns_rank_images/index.html',
            'themes/default/images/icons/birthday.svg',
            'themes/default/images/icons/buttons/ignore.svg',
            'themes/default/images/icons/cns_general/index.html',
            'themes/default/images/icons/cns_general/is_off.svg',
            'themes/default/images/icons/cns_general/is_on.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/demographics.svg',
            'themes/default/images/icons/menu/adminzone/security/usergroups_temp.svg',
            'themes/default/images/icons/menu/adminzone/style/emoticons.svg',
            'themes/default/images/icons/menu/adminzone/tools/users/delete_lurkers.svg',
            'themes/default/images/icons/menu/adminzone/tools/users/member_add.svg',
            'themes/default/images/icons/menu/adminzone/tools/users/member_edit.svg',
            'themes/default/images/icons/menu/adminzone/tools/users/merge_members.svg',
            'themes/default/images/icons/menu/site_meta/user_actions/lost_password.svg',
            'themes/default/images/icons/menu/social/groups.svg',
            'themes/default/images/icons/menu/social/profile.svg',
            'themes/default/images/icons/tabs/member_account/edit/data.svg',
            'themes/default/images/icons/tabs/member_account/edit/delete.svg',
            'themes/default/images/icons/tabs/member_account/edit/index.html',
            'themes/default/images/icons/tabs/member_account/edit/profile.svg',
            'themes/default/images/icons/tabs/member_account/edit/settings.svg',
            'themes/default/images/icons/tabs/member_account/index.html',
            'themes/default/images/icons/tool_buttons/inbox.svg',
            'themes/default/images/icons/tool_buttons/inbox2.svg',
            'themes/default/images/icons_monochrome/birthday.svg',
            'themes/default/images/icons_monochrome/buttons/ignore.svg',
            'themes/default/images/icons_monochrome/cns_general/index.html',
            'themes/default/images/icons_monochrome/cns_general/is_off.svg',
            'themes/default/images/icons_monochrome/cns_general/is_on.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/demographics.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/security/usergroups_temp.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/style/emoticons.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/users/delete_lurkers.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/users/member_add.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/users/member_edit.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/users/merge_members.svg',
            'themes/default/images/icons_monochrome/menu/site_meta/user_actions/lost_password.svg',
            'themes/default/images/icons_monochrome/menu/social/groups.svg',
            'themes/default/images/icons_monochrome/menu/social/profile.svg',
            'themes/default/images/icons_monochrome/tabs/member_account/edit/data.svg',
            'themes/default/images/icons_monochrome/tabs/member_account/edit/delete.svg',
            'themes/default/images/icons_monochrome/tabs/member_account/edit/index.html',
            'themes/default/images/icons_monochrome/tabs/member_account/edit/profile.svg',
            'themes/default/images/icons_monochrome/tabs/member_account/edit/settings.svg',
            'themes/default/images/icons_monochrome/tabs/member_account/index.html',
            'themes/default/images/icons_monochrome/tool_buttons/inbox.svg',
            'themes/default/images/icons_monochrome/tool_buttons/inbox2.svg',
            'themes/default/javascript/core_cns.js',
            'themes/default/templates/BLOCK_MAIN_JOIN.tpl',
            'themes/default/templates/BLOCK_MAIN_JOIN_DONE.tpl',
            'themes/default/templates/BLOCK_MAIN_MEMBERS.tpl',
            'themes/default/templates/BLOCK_MAIN_MEMBERS_COMPLEX.tpl',
            'themes/default/templates/CNS_DELURK_CONFIRM_SCREEN.tpl',
            'themes/default/templates/CNS_EMOTICON_TABLE.tpl',
            'themes/default/templates/CNS_GROUP_DIRECTORY_SCREEN.tpl',
            'themes/default/templates/CNS_JOIN_FORM.tpl',
            'themes/default/templates/CNS_JOIN_REVIEW_RULES_SCREEN.tpl',
            'themes/default/templates/CNS_JOIN_STEP1_SCREEN.tpl',
            'themes/default/templates/CNS_JOIN_STEP2_SCREEN.tpl',
            'themes/default/templates/CNS_MEMBER_ACTION.tpl',
            'themes/default/templates/CNS_MEMBER_BOX.tpl',
            'themes/default/templates/CNS_MEMBER_BOX_CUSTOM_FIELD.tpl',
            'themes/default/templates/CNS_MEMBER_DIRECTORY_SCREEN.tpl',
            'themes/default/templates/CNS_MEMBER_DIRECTORY_SCREEN_FILTER.tpl',
            'themes/default/templates/CNS_MEMBER_DIRECTORY_SCREEN_FILTERS.tpl',
            'themes/default/templates/CNS_MEMBER_DIRECTORY_USERNAME.tpl',
            'themes/default/templates/CNS_MEMBER_PROFILE_ABOUT.tpl',
            'themes/default/templates/CNS_MEMBER_PROFILE_EDIT.tpl',
            'themes/default/templates/CNS_MEMBER_PROFILE_FIELD.tpl',
            'themes/default/templates/CNS_MEMBER_PROFILE_FIELDS.tpl',
            'themes/default/templates/CNS_MEMBER_PROFILE_RANK.tpl',
            'themes/default/templates/CNS_MEMBER_PROFILE_SCREEN.tpl',
            'themes/default/templates/CNS_POST_MAP.tpl',
            'themes/default/templates/CNS_POST_MAP_ITEM.tpl',
            'themes/default/templates/CNS_RANK_IMAGE.tpl',
            'themes/default/templates/CNS_TOPIC_POST_AVATAR.tpl',
            'themes/default/templates/CNS_USERS_ONLINE_SCREEN.tpl',
            'themes/default/templates/CNS_USER_MEMBER.tpl',
            'themes/default/templates/CNS_VIEW_GROUP_MEMBER.tpl',
            'themes/default/templates/CNS_VIEW_GROUP_MEMBER_SECONDARY.tpl',
            'themes/default/templates/CNS_VIEW_GROUP_SCREEN.tpl',
            'themes/default/templates/POSTING_FIELD.tpl',
            'themes/default/text/PARENTAL_CONSENT_MAIL.txt',
        ];
    }

    /**
     * Get mapping between template names and the method of this class that can render a preview of them.
     *
     * @return array The mapping
     */
    public function tpl_previews() : array
    {
        return [
            'templates/CNS_DELURK_CONFIRM_SCREEN.tpl' => 'administrative__cns_delurk_confirm_screen',
            'templates/CNS_JOIN_STEP1_SCREEN.tpl' => 'cns_join_step1_screen',
            'templates/CNS_JOIN_STEP2_SCREEN.tpl' => 'cns_join_step2_screen',
            'templates/CNS_JOIN_REVIEW_RULES_SCREEN.tpl' => 'cns_join_review_rules_screen',
            'templates/CNS_USER_MEMBER.tpl' => 'cns_user_member',
            'templates/CNS_MEMBER_ACTION.tpl' => 'cns_member_profile_screen',
            'templates/CNS_EMOTICON_TABLE.tpl' => 'cns_emoticon_table',
            'templates/CNS_MEMBER_DIRECTORY_SCREEN.tpl' => 'cns_member_directory_screen',
            'templates/CNS_MEMBER_PROFILE_SCREEN.tpl' => 'cns_member_profile_screen',
            'templates/CNS_MEMBER_PROFILE_ABOUT.tpl' => 'cns_member_profile_screen',
            'templates/CNS_MEMBER_PROFILE_FIELDS.tpl' => 'cns_member_profile_screen',
            'templates/CNS_MEMBER_PROFILE_FIELD.tpl' => 'cns_member_profile_screen',
            'templates/CNS_MEMBER_PROFILE_EDIT.tpl' => 'cns_member_profile_screen',
            'templates/CNS_MEMBER_PROFILE_RANK.tpl' => 'cns_member_profile_rank',
            'templates/CNS_USERS_ONLINE_SCREEN.tpl' => 'cns_users_online_screen',
            'templates/CNS_GROUP_DIRECTORY_SCREEN.tpl' => 'cns_group_directory_screen',
            'templates/CNS_VIEW_GROUP_MEMBER.tpl' => 'cns_view_group_screen',
            'templates/CNS_VIEW_GROUP_MEMBER_SECONDARY.tpl' => 'cns_view_group_screen',
            'templates/CNS_VIEW_GROUP_SCREEN.tpl' => 'cns_view_group_screen',
            'templates/BLOCK_MAIN_MEMBERS.tpl' => 'block_main_members_listing',
            'templates/BLOCK_MAIN_MEMBERS_COMPLEX.tpl' => 'block_main_members_complex_listing',
            'templates/CNS_MEMBER_DIRECTORY_SCREEN_FILTERS.tpl' => 'block_main_members_listing',
            'templates/CNS_MEMBER_DIRECTORY_SCREEN_FILTER.tpl' => 'block_main_members_listing',
            'templates/CNS_MEMBER_DIRECTORY_USERNAME.tpl' => 'block_main_members_listing',
            'templates/CNS_POST_MAP.tpl' => 'cns_post_map',
            'templates/CNS_POST_MAP_ITEM.tpl' => 'cns_post_map',
            'templates/BLOCK_MAIN_JOIN_DONE.tpl' => 'block_main_join_done',
            'templates/BLOCK_MAIN_JOIN.tpl' => 'block_main_join',
            'templates/CNS_JOIN_FORM.tpl' => 'cns_join_form',
            'text/PARENTAL_CONSENT_MAIL.txt' => 'parental_consent_mail',
        ];
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__cns_delurk_confirm_screen() : object
    {
        $lurkers = [
            [
                'ID' => '1',
                'USERNAME' => lorem_word(),
                'PROFILE_URL' => placeholder_url(),
            ],
            [
                'ID' => '2',
                'USERNAME' => lorem_word_2(),
                'PROFILE_URL' => placeholder_url(),
            ],
        ];

        return lorem_globalise(do_lorem_template('CNS_DELURK_CONFIRM_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'LURKERS' => $lurkers,
            'URL' => placeholder_url(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_join_step1_screen() : object
    {
        $group_select = new Tempcode();

        foreach (placeholder_array() as $key => $value) {
            $group_select->attach(form_input_list_entry(strval($key), false, $value));
        }

        return lorem_globalise(do_lorem_template('CNS_JOIN_STEP1_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'RULES' => lorem_chunk_html(),
            'URL' => placeholder_url(),
            'HIDDEN' => '',
            'GROUP_SELECT' => $group_select,
            'DECLARATIONS' => [do_lang('I_AGREE')],
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_join_step2_screen() : object
    {
        require_css('forms');

        require_lang('dates');

        $fields = new Tempcode();

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_LINE', [
            'MAXLENGTH' => '3',
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
            'SIZE' => '40',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_phrase(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_PASSWORD', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'VALUE' => '',
            'PASSWORD_STRENGTH' => true,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_phrase(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_PASSWORD', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'VALUE' => '',
            'PASSWORD_STRENGTH' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_phrase(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_EMAIL', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_phrase(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD_SPACER', [
            'TITLE' => lorem_phrase(),
            'SECTION_HIDDEN' => false,
            'HELP' => lorem_paragraph(),
        ]));

        $form = do_lorem_template('FORM', [
            'TEXT' => '',
            'HIDDEN' => '',
            'FIELDS' => $fields,
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => do_lang_tempcode('PROCEED'),
            'SUBMIT_BUTTON_CLASS' => null,
            'URL' => placeholder_url(),
            'JS_FUNCTION_CALLS' => [],
            'SKIP_REQUIRED' => null,
            'SECONDARY_FORM' => false,
            'TABINDEX' => placeholder_number(),
            'SUPPORT_AUTOSAVE' => null,
            'ANALYTIC_EVENT_CATEGORY' => null,
            'MODSECURITY_WORKAROUND' => null,
            'SKIP_WEBSTANDARDS' => null,
            'GET' => null,
            'NO_SIZING' => null,
            'PREVIEW' => null,
            'TITLE' => null,
        ]);

        return lorem_globalise(do_lorem_template('CNS_JOIN_STEP2_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'FORM' => $form,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_join_review_rules_screen() : object
    {
        $group_select = new Tempcode();

        foreach (placeholder_array() as $key => $value) {
            $group_select->attach(form_input_list_entry(strval($key), false, $value));
        }

        return lorem_globalise(do_lorem_template('CNS_JOIN_REVIEW_RULES_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'RULES' => lorem_chunk_html(),
            'URL' => placeholder_url(),
            'URL_DISAGREE' => placeholder_url(),
            'HIDDEN' => '',
            'DECLARATIONS' => [do_lang('I_AGREE')],
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_user_member() : object
    {
        return lorem_globalise(do_lorem_template('CNS_USER_MEMBER', [
            'FIRST' => true,
            'COLOUR' => '',
            'PROFILE_URL' => placeholder_url(),
            'USERNAME' => lorem_word(),
            'MEMBER_ID' => placeholder_numeric_id(),
            'USERGROUP' => lorem_word_2(),
            'AT' => lorem_phrase(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_emoticon_table() : object
    {
        $rows = [];
        $cols = 4;
        $cells = [];
        for ($i = 0; $i < 10; $i++) {
            if (($i % $cols == 0) && ($i != 0)) {
                $rows[] = [
                    'CELLS' => $cells,
                ];
                $cells = [];
            }
            $cells[] = [
                'FIELD_NAME' => lorem_word(),
                'COLS' => strval($cols),
                'CODE_ESC' => '',
                'THEME_IMG_CODE' => 'cns_emoticons/' . placeholder_img_code('cns_emoticons'),
                'CODE' => ':)',
            ];
        }
        if (!empty($cells)) {
            $rows[] = [
                'CELLS' => $cells,
            ];
        }

        return lorem_globalise(do_lorem_template('CNS_EMOTICON_TABLE', [
            'ROWS' => $rows,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_member_directory_screen() : object
    {
        if (get_forum_type() != 'cns') {
            return do_lorem_template('RED_ALERT', ['TEXT' => do_lang_tempcode('NO_CNS')]);
        }

        return lorem_globalise(do_lorem_template('CNS_MEMBER_DIRECTORY_SCREEN', [
            'TITLE' => lorem_screen_title(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_listing() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS', 'listing');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_boxes() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS', 'boxes');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_photos() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS', 'photos');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_media() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS', 'media');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_avatars() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS', 'avatars');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_complex_listing() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS_COMPLEX', 'listing');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_complex_boxes() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS_COMPLEX', 'boxes');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_complex_photos() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS_COMPLEX', 'photos');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_complex_media() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS_COMPLEX', 'media');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_members_complex_avatars() : object
    {
        $tpl = $this->do_block_main_members_preview('BLOCK_MAIN_MEMBERS_COMPLEX', 'avatars');

        return lorem_globalise($tpl, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @param  string $tpl Template
     * @param  ID_TEXT $display_mode The display mode for the block
     * @set listing boxes photos media avatars
     * @return Tempcode Preview
     */
    protected function do_block_main_members_preview(string $tpl, string $display_mode) : object
    {
        $custom_fields = do_lorem_template('CNS_MEMBER_BOX_CUSTOM_FIELD', [
            'NAME' => lorem_phrase(),
            'RAW' => placeholder_ip(),
            'VALUE' => placeholder_ip(),
            'MEMBER_ID' => placeholder_first_admin_id(),
        ]);
        $poster_details = do_lorem_template('CNS_GUEST_DETAILS', [
            'CUSTOM_FIELDS' => $custom_fields,
        ]);

        $map = [
            'GIVE_CONTEXT' => false,
            'MEMBER_ID' => placeholder_first_admin_id(),
            'USERNAME' => lorem_phrase(),
            '_POSTS' => placeholder_number(),
            'POSTS' => placeholder_number(),
            '_POINTS' => placeholder_number(),
            'POINTS' => placeholder_number(),
            'JOIN_DATE_RAW' => placeholder_date_raw(),
            'JOIN_DATE' => placeholder_date(),
            'PRIMARY_GROUP_NAME' => lorem_phrase(),
            'SECONDARY_GROUPS' => [
                lorem_word(),
            ],
            'CUSTOM_FIELDS' => lorem_phrase(),
            'ONLINE' => false,
            'AVATAR_URL' => placeholder_image_url(),
            'DOB_LABEL' => do_lang('DATE_OF_BIRTH'),
            'DOB' => placeholder_date(),
            '_DOB' => placeholder_date_raw(),
            '_DOB_CENSORED' => placeholder_date_raw(),
        ];
        if (addon_installed('cns_warnings')) {
            $map['_NUM_WARNINGS'] = placeholder_number();
            $map['NUM_WARNINGS']  = placeholder_number();
        }
        $box = do_lorem_template('CNS_MEMBER_BOX', $map);

        $member_boxes = [];
        $member_boxes[] = [
            'I' => '0',
            'BREAK' => false,
            'BOX' => $box,
            'MEMBER_ID' => placeholder_first_admin_id(),
            'GALLERY_NAME' => '',
            'GALLERY_TITLE' => '',
        ];

        $per_row = 6;

        $usergroups = [];
        $usergroups[placeholder_numeric_id()] = ['USERGROUP' => lorem_phrase(), 'NUM' => placeholder_number()];

        $symbols = [
            [
                'START' => '0',
                'SYMBOL' => 'a',
            ],
            [
                'START' => '1',
                'SYMBOL' => 'b',
            ],
            [
                'START' => '3',
                'SYMBOL' => 'c',
            ]
        ];

        $result_entries = new Tempcode();
        $header_row = new Tempcode();
        foreach (placeholder_array(6) as $i => $v) {
            $username = do_lorem_template('CNS_MEMBER_DIRECTORY_USERNAME', [
                'ID' => placeholder_first_admin_id(),
                'USERNAME' => lorem_word(),
                'URL' => placeholder_url(),
                'AVATAR_URL' => placeholder_avatar(),
                'PHOTO_URL' => placeholder_image(),
                'VALIDATED' => ($i > 1),
                'CONFIRMED' => ($i > 0),
            ]);

            $cells = new Tempcode();

            $cells->attach(do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => $username,
            ]));
            $cells->attach(do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => lorem_word(),
            ]));
            $cells->attach(do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => placeholder_number(),
            ]));
            $cells->attach(do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => placeholder_number(),
            ]));
            $cells->attach(do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => placeholder_date(),
            ]));
            $cells->attach(do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => placeholder_date(),
            ]));

            $entries = do_lorem_template('RESULTS_TABLE_ENTRY', [
                'VALUES' => $cells,
            ]);

            $result_entries->attach($entries);

            $header_row->attach(do_lorem_template('RESULTS_TABLE_FIELD_TITLE', [
                'VALUE' => lorem_word(),
            ]));
        }

        $results = do_lorem_template('RESULTS_TABLE', [
            'WIDTHS' => [],
            'TEXT_ID' => placeholder_random_id(),
            'HEADER_ROW' => $header_row,
            'RESULT_ENTRIES' => $result_entries,
            'FOOTER_ROW' => null,
            'MESSAGE' => '',
            'SORT' => '',
            'PAGINATION' => '',
            'NONRESPONSIVE' => false,
        ]);

        return do_lorem_template($tpl, [
            'BLOCK_ID' => '',
            'START' => strval(0),
            'MAX' => strval(30),
            'SORTABLE' => 'm_join_time',
            'SORT_ORDER' => 'DESC',
            'ITEM_WIDTH' => ($per_row === null) ? '' : (float_to_raw_string(99.0/*avoid possibility of rounding issues as pixels will not divide perfectly*/ / floatval($per_row)) . '%'),
            'PER_ROW' => strval($per_row),
            'DISPLAY_MODE' => $display_mode,
            'MEMBER_BOXES' => $member_boxes,
            'PAGINATION' => placeholder_pagination(),
            'RESULTS_TABLE' => $results,
            'USERGROUPS' => $usergroups,
            'SYMBOLS' => $symbols,
            'HAS_ACTIVE_FILTER' => true,
            'INCLUDE_FORM' => true,
            'SORT' => '',
            'FILTERS_ROW_A' => placeholder_comma_list(),
            'FILTERS_ROW_B' => placeholder_comma_list_b(),
        ]);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_member_profile_screen() : object
    {
        $sections = [
            'contact' => lorem_word(),
            'profile' => lorem_word_2(),
            'views' => lorem_word(),
            'audit' => lorem_word_2(),
            'content' => lorem_word(),
        ];
        $actions = [];
        $i = 0;
        foreach ($sections as $section_code => $section_title) {
            $actions[$section_code] = do_lorem_template('CNS_MEMBER_ACTION', [
                'ID' => strval($i),
                'URL' => placeholder_url(),
                'ICON' => 'links/facebook',
                'LANG' => lorem_word(),
                'REL' => '',
            ]);
            $i++;
        }

        $custom_fields = [
            [
                'FIELD_ID' => placeholder_numeric_id(),
                'NAME_FULL' => lorem_phrase(),
                'NAME' => lorem_phrase(),
                'RENDERED' => lorem_phrase(),
                'RAW' => lorem_phrase(),
                'FIELD_TYPE' => 'short_text',
                'EDITABILITY' => null,
                'EDIT_TYPE' => 'line',
                'ENCRYPTED_VALUE' => '',
                'ICON' => '',
                'SECTION' => '',
            ],
        ];

        $tabs = [];
        $tab_content = do_lorem_template('CNS_MEMBER_PROFILE_ABOUT', [
            'RIGHT_MARGIN' => lorem_phrase(),
            'AVATAR_WIDTH' => placeholder_number(),
            'PHOTO_WIDTH' => placeholder_number(),
            'MOST_ACTIVE_FORUM' => lorem_phrase(),
            'TIME_FOR_THEM' => placeholder_date(),
            'TIME_FOR_THEM_RAW' => placeholder_date_raw(),
            '_SUBMIT_DAYS_AGO' => placeholder_number(),
            'SUBMIT_DAYS_AGO' => placeholder_number(),
            'SUBMIT_TIME_RAW' => placeholder_date_raw(),
            'LAST_VISIT_TIME_RAW' => placeholder_date_raw(),
            'ONLINE_NOW' => lorem_phrase(),
            '_ONLINE_NOW' => false,
            'BANNED' => lorem_phrase(),
            'USER_AGENT' => lorem_phrase(),
            'OPERATING_SYSTEM' => lorem_phrase(),
            'DOB_LABEL' => lorem_phrase(),
            'DOB' => lorem_phrase(),
            '_DOB' => placeholder_date_raw(),
            '_DOB_CENSORED' => placeholder_date_raw(),
            'IP_ADDRESS' => lorem_phrase(),
            'COUNT_POSTS' => placeholder_number(),
            '_COUNT_POINTS' => placeholder_number(),
            'COUNT_POINTS' => placeholder_number(),
            'PRIMARY_GROUP' => lorem_phrase(),
            'PRIMARY_GROUP_ID' => placeholder_numeric_id(),
            'PHOTO_URL' => placeholder_image_url(),
            'EMAIL_ADDRESS' => lorem_word() . '@example.com',
            'AVATAR_URL' => placeholder_avatar(),
            'SIGNATURE' => lorem_phrase(),
            'JOIN_DATE' => placeholder_date(),
            'JOIN_DATE_RAW' => placeholder_date_raw(),
            'CUSTOM_FIELDS' => $custom_fields,
            'ACTIONS_contact' => $actions['contact'],
            'ACTIONS_profile' => $actions['profile'],
            'ACTIONS_views' => $actions['views'],
            'ACTIONS_audit' => $actions['audit'],
            'ACTIONS_content' => $actions['content'],
            'USERNAME' => lorem_word(),
            'MEMBER_ID' => placeholder_first_admin_id(),
            'SECONDARY_GROUPS' => placeholder_array(),
            'VIEW_PROFILES' => true,
            'ON_PROBATION' => placeholder_date_raw(),
            'CLUBS' => lorem_phrase(),
            'EXTRA_INFO_DETAILS' => [],
            'EXTRA_TRACKING_DETAILS' => [],
            'USERS_TIMEZONE' => lorem_word(),
            'USERS_TIMEZONE_RAW' => lorem_word(),
            'EXTRA_SECTIONS' => [],
            '__COUNT_POSTS' => placeholder_number(),
            '_COUNT_POSTS' => placeholder_number(),
        ]);
        $tabs[] = [
            'TAB_CODE' => placeholder_codename() . '0',
            'TAB_TITLE' => lorem_phrase(),
            'TAB_ICON' => 'menu/social/profile',
            'TAB_CONTENT' => $tab_content,
            'TAB_FIRST' => true,
            'TAB_LAST' => false,
        ];
        $tabs2 = [];
        $fields = new Tempcode();
        $name = placeholder_codename();
        $input = do_lorem_template('FORM_SCREEN_INPUT_INTEGER', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
            'READ_ONLY' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_phrase(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));
        $tabs2[] = [
            'TAB_TITLE' => lorem_phrase(),
            'TAB_ICON' => 'menu/social/profile',
            'TAB_FIELDS' => $fields,
            'TAB_TEXT' => lorem_paragraph(),
            'TAB_SINGLE_FIELD' => false,
            'TAB_FIRST' => true,
            'TAB_LAST' => true,
        ];
        $tab_content = do_lorem_template('CNS_MEMBER_PROFILE_EDIT', [
            'URL' => placeholder_url(),
            'SUBMIT_ICON' => 'admin/edit',
            'SUBMIT_NAME' => lorem_phrase(),
            'SKIP_WEBSTANDARDS' => true,
            'TABS' => $tabs2,
            'HIDDEN' => '',
            'JS_FUNCTION_CALLS' => [],
        ]);
        $tabs[] = [
            'TAB_CODE' => placeholder_codename() . '1',
            'TAB_TITLE' => lorem_phrase(),
            'TAB_ICON' => 'menu/social/profile',
            'TAB_CONTENT' => $tab_content,
            'TAB_FIRST' => false,
            'TAB_LAST' => true,
        ];

        return lorem_globalise(do_lorem_template('CNS_MEMBER_PROFILE_SCREEN', [
            'TITLE' => get_screen_title('MEMBER_ACCOUNT', true, [lorem_phrase(), lorem_phrase()]),
            'MEMBER_ID' => placeholder_first_admin_id(),
            'TABS' => $tabs,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_member_profile_rank() : object
    {
        $groups = [];
        $groups[] = [
            'RANK_GROUP' => '1',
            'RANK_NAME' => lorem_word(),
            'RANK_THRESHOLD' => placeholder_number(),
            'RANK_STATUS' => '2',
        ];
        $groups[] = [
            'RANK_GROUP' => '2',
            'RANK_NAME' => lorem_word_2(),
            'RANK_THRESHOLD' => placeholder_number(),
            'RANK_STATUS' => '1',
        ];
        $groups[] = [
            'RANK_GROUP' => '2',
            'RANK_NAME' => lorem_word(),
            'RANK_THRESHOLD' => placeholder_number(),
            'RANK_STATUS' => '0',
        ];
        $locked_privileges = [];
        $unlocked_privileges = [];
        $locked_privileges[] = [
            'PRIVILEGE' => lorem_phrase(),
            'SCOPE' => lorem_word(),
        ];
        $unlocked_privileges[] = [
            'PRIVILEGE' => lorem_phrase(),
            'SCOPE' => lorem_word(),
        ];
        return lorem_globalise(do_template('CNS_MEMBER_PROFILE_RANK', [
            '_GUID' => '08c9853175987d67e96f77d912e9e809',
            'SIMPLE_STEPPER' => '0',
            'RANKS' => $groups,
            'CURRENT_RANK' => lorem_word_2(),
            '_NUM_POINTS_ADVANCE' => placeholder_number(),
            'NUM_POINTS_ADVANCE' => placeholder_number(),
            'LOCKED_THEMES' => lorem_phrase(),
            'UNLOCKED_THEMES' => lorem_phrase(),
            'LOCKED_PRIVILEGES' => $locked_privileges,
            'UNLOCKED_PRIVILEGES' => $unlocked_privileges,
            'HAS_DAILY_UPLOAD_QUOTA' => '1',
            'DAILY_UPLOAD_QUOTA' => placeholder_number(),
            '_DAILY_UPLOAD_QUOTA' => placeholder_number(),
            'MAX_ATTACHMENTS' => placeholder_number(),
            '_MAX_ATTACHMENTS' => placeholder_number(),
            'POST_LENGTH' => placeholder_number(),
            '_POST_LENGTH' => placeholder_number(),
            'SIGNATURE_LENGTH' => placeholder_number(),
            '_SIGNATURE_LENGTH' => placeholder_number(),
            'CAN_UPLOAD_AVATARS' => '1',
            'MAXIMUM_AVATAR_DIMENSIONS' => placeholder_number(),
            'INFINITE_PERSONAL_GALLERY_ENTRIES' => '0',
            'PERSONAL_GALLERY_ENTRIES_IMAGES' => placeholder_number(),
            '_PERSONAL_GALLERY_ENTRIES_IMAGES' => placeholder_number(),
            'PERSONAL_GALLERY_ENTRIES_VIDEOS' => placeholder_number(),
            '_PERSONAL_GALLERY_ENTRIES_VIDEOS' => placeholder_number(),
            'GIFT_POINTS' => placeholder_number(),
            '_GIFT_POINTS' => placeholder_number(),
            'GIFT_POINTS_PER_DAY' => placeholder_number(),
            '_GIFT_POINTS_PER_DAY' => placeholder_number(),
            'FLOOD_CONTROL_SUBMIT' => placeholder_number(),
            '_FLOOD_CONTROL_SUBMIT' => placeholder_number(),
            'FLOOD_CONTROL_ACCESS' => placeholder_number(),
            '_FLOOD_CONTROL_ACCESS' => placeholder_number(),
        ]));
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_users_online_screen() : object
    {
        $rows = [];
        foreach (placeholder_array() as $key => $value) {
            $rows[] = [
                'IP' => placeholder_ip(),
                'AT_URL' => placeholder_url(),
                'LOCATION' => lorem_word(),
                'MEMBER' => placeholder_link(),
                '_TIME' => placeholder_number(),
                'TIME' => placeholder_number(),
            ];
        }

        return lorem_globalise(do_lorem_template('CNS_USERS_ONLINE_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'ROWS' => $rows,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_group_directory_screen() : object
    {
        return lorem_globalise(do_lorem_template('CNS_GROUP_DIRECTORY_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'STAFF' => placeholder_table(),
            'OTHERS' => placeholder_table(),
            'CLUBS' => placeholder_table(),
            'RANKS' => [placeholder_table(), placeholder_table()],
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_view_group_screen() : object
    {
        $_primary_members = placeholder_array();
        $primary_members = new Tempcode();
        $_secondary_members = new Tempcode();
        $secondary_members = new Tempcode();
        $prospective_members = new Tempcode();
        $_prospective_members = new Tempcode();
        foreach ($_primary_members as $i => $primary_member) {
            $temp = do_lorem_template('CNS_VIEW_GROUP_MEMBER', [
                'ID' => placeholder_first_admin_id(),
                'NAME' => $primary_member,
                'URL' => placeholder_url(),
            ]);

            $cells = do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => $temp,
            ]);
            $entries = do_lorem_template('RESULTS_TABLE_ENTRY', [
                'VALUES' => $cells,
            ]);

            $primary_members->attach($entries);
        }

        $header_row = do_lorem_template('RESULTS_TABLE_FIELD_TITLE', [
            'VALUE' => lorem_word(),
        ]);

        $primary_members = do_lorem_template('RESULTS_TABLE', [
            'WIDTHS' => [],
            'TEXT_ID' => placeholder_random_id(),
            'HEADER_ROW' => $header_row,
            'RESULT_ENTRIES' => $primary_members,
            'FOOTER_ROW' => null,
            'MESSAGE' => '',
            'SORT' => '',
            'PAGINATION' => '',
            'NONRESPONSIVE' => false,
        ]);

        $temp = new Tempcode();
        foreach (placeholder_array() as $i => $v) {
            $temp = do_lorem_template('CNS_VIEW_GROUP_MEMBER_SECONDARY', [
                'ID' => placeholder_first_admin_id(),
                'URL' => placeholder_url(),
                'REMOVE_URL' => placeholder_url(),
                'NAME' => $v,
            ]);
            $cells = do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => $temp,
            ]);
            $entries = do_lorem_template('RESULTS_TABLE_ENTRY', [
                'VALUES' => $cells,
            ]);

            $_secondary_members->attach($entries);
        }
        $header_row = do_lorem_template('RESULTS_TABLE_FIELD_TITLE', [
            'VALUE' => lorem_word(),
        ]);

        $secondary_members = do_lorem_template('RESULTS_TABLE', [
            'WIDTHS' => [],
            'TEXT_ID' => placeholder_random_id(),
            'HEADER_ROW' => $header_row,
            'RESULT_ENTRIES' => $_secondary_members,
            'FOOTER_ROW' => null,
            'MESSAGE' => '',
            'SORT' => '',
            'PAGINATION' => '',
            'NONRESPONSIVE' => false,
        ]);

        foreach (placeholder_array() as $i => $v) {
            $cells = do_lorem_template('RESULTS_TABLE_FIELD', [
                'VALUE' => $temp,
            ]);
            $entries = do_lorem_template('RESULTS_TABLE_ENTRY', [
                'VALUES' => $cells,
            ]);

            $_prospective_members->attach($entries);
        }
        $header_row = do_lorem_template('RESULTS_TABLE_FIELD_TITLE', [
            'VALUE' => lorem_word(),
        ]);

        $prospective_members = do_lorem_template('RESULTS_TABLE', [
            'WIDTHS' => [],
            'TEXT_ID' => placeholder_random_id(),
            'HEADER_ROW' => $header_row,
            'RESULT_ENTRIES' => $_prospective_members,
            'FOOTER_ROW' => null,
            'MESSAGE' => '',
            'SORT' => '',
            'PAGINATION' => '',
            'NONRESPONSIVE' => false,
        ]);

        return lorem_globalise(do_lorem_template('CNS_VIEW_GROUP_SCREEN', [
            'GROUP_NAME' => lorem_phrase(),
            'ID' => placeholder_numeric_id(),
            'FORUM' => '',
            'CLUB' => false,
            'EDIT_URL' => placeholder_url(),
            'TITLE' => lorem_screen_title(),
            'LEADER' => lorem_phrase(),
            'PROMOTION_INFO' => new Tempcode(),
            'ADD_URL' => placeholder_url(),
            'APPLY_URL' => placeholder_url(),
            'APPLY_TEXT' => lorem_phrase(),
            'PRIMARY_MEMBERS' => $primary_members,
            'SECONDARY_MEMBERS' => $secondary_members,
            'PROSPECTIVE_MEMBERS' => $prospective_members,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_post_map() : object
    {
        $items = new Tempcode();

        $items->attach(do_lorem_template('CNS_POST_MAP_ITEM', [
            'TITLE' => lorem_phrase(),
            'URL' => placeholder_url(),
            'POSTER_ID' => placeholder_number(),
            'POSTER_IS_GUEST' => false,
            'POSTER_URL' => placeholder_url(),
            'POSTER_NAME' => lorem_phrase(),
            'LEVEL_HAS_ADJACENT_SIBLING' => [true],
            'POST_LEVEL' => '0',
            'DATE' => placeholder_date(),
            'IS_UNREAD' => true,
            'POST_NUMBER' => '1',
        ]));
        $items->attach(do_lorem_template('CNS_POST_MAP_ITEM', [
            'TITLE' => lorem_phrase(),
            'URL' => placeholder_url(),
            'POSTER_ID' => placeholder_number(),
            'POSTER_IS_GUEST' => false,
            'POSTER_URL' => placeholder_url(),
            'POSTER_NAME' => lorem_phrase(),
            'LEVEL_HAS_ADJACENT_SIBLING' => [true],
            'POST_LEVEL' => '0',
            'DATE' => placeholder_date(),
            'IS_UNREAD' => false,
            'POST_NUMBER' => '2',
        ]));
        $items->attach(do_lorem_template('CNS_POST_MAP_ITEM', [
            'TITLE' => lorem_phrase(),
            'URL' => placeholder_url(),
            'POSTER_ID' => placeholder_number(),
            'POSTER_IS_GUEST' => false,
            'POSTER_URL' => placeholder_url(),
            'POSTER_NAME' => lorem_phrase(),
            'LEVEL_HAS_ADJACENT_SIBLING' => [false, true],
            'POST_LEVEL' => '1',
            'DATE' => placeholder_date(),
            'IS_UNREAD' => false,
            'POST_NUMBER' => '3',
        ]));

        return lorem_globalise(do_lorem_template('CNS_POST_MAP', [
            'ITEMS' => $items,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_join() : object
    {
        return lorem_globalise(do_lorem_template('BLOCK_MAIN_JOIN', [
            'FORM' => placeholder_form(),
            'HAS_EMAIL_TO_SEND' => true,
            'BLOCK_PARAMS' => '',
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_join_done() : object
    {
        return lorem_globalise(do_lorem_template('BLOCK_MAIN_JOIN_DONE', [
            'MESSAGE' => lorem_paragraph_html(),
            'LOGGED_IN' => true,
            'HAS_EMAIL_TO_SEND' => true,
            'EMAIL_SENT' => true,
            'EMAIL_ADDRESS' => lorem_word(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__cns_join_form() : object
    {
        return lorem_globalise(do_lorem_template('CNS_JOIN_FORM', [
            'HIDDEN' => '',
            'URL' => placeholder_url(),
            'FIELDS' => placeholder_fields(),
            'SUBMIT_ICON' => 'menu/site_meta/user_actions/join',
            'SUBMIT_NAME' => lorem_word(),
            'TEXT' => lorem_sentence_html(),
            'USERNAME_CHECK_SCRIPT' => placeholder_url(),
            'SNIPPET_SCRIPT' => placeholder_url(),
            'INVITES_ENABLED' => true,
            'ONE_PER_EMAIL_ADDRESS' => true,
            'USE_CAPTCHA' => true,
            'ANALYTIC_EVENT_CATEGORY' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__parental_consent_mail() : object
    {
        $fields_done = [];
        $fields_done[] = ['LABEL' => lorem_phrase(), 'VALUE' => lorem_phrase()];

        return lorem_globalise(do_lorem_template('PARENTAL_CONSENT_MAIL', [
            'FAX' => lorem_phrase(),
            'POSTAL_ADDRESS' => lorem_phrase(),
            'EMAIL_ADDRESS' => lorem_phrase(),
            'FIELDS_DONE' => $fields_done,
            'PRIVACY_POLICY_URL' => placeholder_url(),
            'RULES_URL' => placeholder_url(),
        ], null, false, null, '.txt', 'text'), null, '', true);
    }

    /**
     * Get details of extended emoticons.
     *
     * @return array A pair: Details of core extended emoticons, Details of non-core extended emoticons
     */
    protected function extended_emoticons() : array
    {
        $core_emoticons = [
            ':dry:' => ['dry', false],
            ':$' => ['blush', false],
            'O_o' => ['blink', false],
            ':wub:' => ['wub', false],
            ':cool:' => ['cool', false],
            ':lol:' => ['lol', false],
            ':|' => ['mellow', true],
            ':ninja:' => ['ph34r', false],
        ];

        $supported_emoticons = [
            ':offtopic:' => ['offtopic', true], // Larger than normal, so don't put in core set
            ':rolleyes:' => ['rolleyes', false],
            ':D' => ['grin', false],
            '^_^' => ['glee', false],
            '(K)' => ['kiss', true],
            ':S' => ['confused', false],
            ':@' => ['angry', false],
            ':shake:' => ['shake', false],
            ':hand:' => ['hand', false],
            ':drool:' => ['drool', false],
            ':devil:' => ['devil', false],
            ':party:' => ['party', true],
            ':constipated:' => ['constipated', false],
            ':depressed:' => ['depressed', false],
            ':zzz:' => ['zzz', false],
            ':whistle:' => ['whistle', true],
            ':upsidedown:' => ['upsidedown', false],
            ':sick:' => ['sick', false],
            ':shutup:' => ['shutup', true],
            ':sarcy:' => ['sarcy', false],
            ':puppyeyes:' => ['puppyeyes', false],
            ':nod:' => ['nod', true],
            ':nerd:' => ['nerd', false],
            ':king:' => ['king', false],
            ':birthday:' => ['birthday', false],
            ':cyborg:' => ['cyborg', true],
            ':hippie:' => ['hippie', false],
            ':ninja2:' => ['ninja2', false],
            ':rockon:' => ['rockon', true],
            ':sinner:' => ['sinner', true],
            ':guitar:' => ['guitar', true],
            ':angel:' => ['angel', false],
            ':cowboy:' => ['cowboy', false],
            ':fight:' => ['fight', false],
            ':goodbye:' => ['goodbye', false],
            ':idea:' => ['idea', true],
            ':boat:' => ['boat', true],
            ':fishing:' => ['fishing', true],
            ':reallybadday:' => ['reallybadday', true],
            ':hug:' => ['hug', true],
            ':tired:' => ['tired', true],
            ':whew:' => ['whew', true],
            ':christmas:' => ['christmas', true],
        ];

        return [$core_emoticons, $supported_emoticons];
    }

    /**
     * Find available predefined content, and what is installed.
     *
     * @return array A map of available predefined content codenames, and details (if installed, and title)
     */
    public function enumerate_predefined_content() : array
    {
        $ret = [];

        require_lang('cns');
        require_code('cns_general_action2');
        list($core_emoticons, $supported_emoticons) = $this->extended_emoticons();
        $_extended_emoticons = [];
        foreach (array_keys($core_emoticons) as $code) {
            $_extended_emoticons[] = db_string_equal_to('e_code', $code);
        }
        foreach (array_keys($supported_emoticons) as $code) {
            $_extended_emoticons[] = db_string_equal_to('e_code', $code);
        }
        $or_list = implode(' OR ', $_extended_emoticons);
        $installed = ($GLOBALS['FORUM_DB']->query_select_value('f_emoticons', 'COUNT(*)', [], ' AND (' . $or_list . ')') > 0);
        $ret['have_default_full_emoticon_set'] = [
            'title' => do_lang_tempcode('HAVE_DEFAULT_FULL_EMOTICON_SET'),
            'description' => do_lang_tempcode('DESCRIPTION_HAVE_DEFAULT_FULL_EMOTICON_SET', escape_html(integer_format(cns_get_num_emoticons_on_disk()))),
            'installed' => $installed,
        ];

        $_existing_fields = $GLOBALS['FORUM_DB']->query_select('f_custom_fields', ['id', 'cf_name']);
        $existing_fields = [];
        foreach ($_existing_fields as $field) {
            $existing_fields[get_translated_text($field['cf_name'])] = $field['id'];
        }
        require_lang('cns');
        require_lang('cns_special_cpf');
        require_lang('fields');
        require_code('cns_members_action2');
        $cpfs = cns_predefined_custom_field_details();
        foreach ($cpfs as $codename => $details) {
            $ret[$codename] = [
                'title' => do_lang_tempcode('DEFAULT_CPF_' . $codename . '_NAME'),
                'description' => do_lang_tempcode('DEFAULT_CPF_' . $codename . '_DESCRIPTION'),
                'installed' => array_key_exists(do_lang('DEFAULT_CPF_' . $codename . '_NAME'), $existing_fields),
                '_details' => $details,
            ];
        }

        sort_maps_by($ret, 'title', false, true);

        return $ret;
    }

    /**
     * Install predefined content.
     *
     * @param  ?array $content A list of predefined content labels to install (null: all)
     */
    public function install_predefined_content(?array $content = null)
    {
        if (($content === null) || (in_array('have_default_full_emoticon_set', $content))) {
            if (!has_predefined_content('core_cns', 'have_default_full_emoticon_set')) {
                list($core_emoticons, $supported_emoticons) = $this->extended_emoticons();
                foreach ($core_emoticons as $type_code => $_) {
                    list($theme_image_code, $no_use_topics) = $_;
                    cns_make_emoticon($type_code, 'cns_emoticons/' . $theme_image_code, 0, $no_use_topics ? 0 : 1);
                }
                foreach ($supported_emoticons as $type_code => $_) {
                    list($theme_image_code, $no_use_topics) = $_;
                    cns_make_emoticon($type_code, 'cns_emoticons/' . $theme_image_code, 1, $no_use_topics ? 0 : 1);
                }
            }
        }

        require_code('cns_members_action2');

        $cpfs = cns_predefined_custom_field_details();
        foreach (array_keys($cpfs) as $codename) {
            if ((($content === null) || (in_array($codename, $content))) && (!has_predefined_content('core_cns', $codename))) {
                cns_make_predefined_content_field($codename);
            }
        }
    }

    /**
     * Uninstall predefined content.
     *
     * @param  ?array $content A list of predefined content labels to uninstall (null: all)
     */
    public function uninstall_predefined_content(?array $content = null)
    {
        if ((($content === null) || (in_array('have_default_full_emoticon_set', $content))) && (has_predefined_content('core_cns', 'have_default_full_emoticon_set'))) {
            list($core_emoticons, $supported_emoticons) = $this->extended_emoticons();
            $_extended_emoticons = [];
            foreach (array_keys($core_emoticons) as $code) {
                $_extended_emoticons[] = db_string_equal_to('e_code', $code);
            }
            foreach (array_keys($supported_emoticons) as $code) {
                $_extended_emoticons[] = db_string_equal_to('e_code', $code);
            }
            $GLOBALS['FORUM_DB']->query('DELETE FROM ' . $GLOBALS['FORUM_DB']->get_table_prefix() . 'f_emoticons WHERE (' . implode(' OR ', $_extended_emoticons) . ')');
        }

        require_code('cns_members_action2');

        $cpfs = cns_predefined_custom_field_details();
        foreach (array_keys($cpfs) as $codename) {
            if ((($content === null) || (in_array($codename, $content))) && (has_predefined_content('core_cns', $codename))) {
                $this->delete_predefined_content_field($codename);
            }
        }
    }

    /**
     * Delete a Custom Profile Field from one of the predefined templates (this is often used by importers).
     *
     * @param  ID_TEXT $codename The identifier of the predefined Custom Profile Field
     */
    protected function delete_predefined_content_field(string $codename)
    {
        require_lang('cns');
        require_lang('cns_special_cpf');
        require_lang('fields');

        $test = $GLOBALS['FORUM_DB']->query_select_value_if_there('f_custom_fields', 'id', [$GLOBALS['SITE_DB']->translate_field_ref('cf_name') => do_lang('DEFAULT_CPF_' . $codename . '_NAME')]);
        if ($test !== null) {
            require_code('cns_members_action');
            cns_delete_custom_field($test);
        }
    }
}
