<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_feedback_features
 */

/**
 * Hook class.
 */
class Hook_addon_registry_core_feedback_features
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the version of the software this addon is for (used in generating the TAR filename).
     *
     * @return float Version number
     */
    public function get_version() : float
    {
        return cms_version_number();
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'Information Display';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'Features for user interaction with content.';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [
            'tut_feedback',
            'tut_adv_news',
        ];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => [],
            'recommends' => [],
            'conflicts_with' => [],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/feedback/comment.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'adminzone/pages/modules/admin_trackbacks.php',
            'data/form_to_email.php',
            'data/post_comment.php',
            'data/trackback.php',
            'lang/EN/trackbacks.ini',
            'pages/comcode/EN/feedback.txt',
            'sources/blocks/main_comments.php',
            'sources/blocks/main_contact_simple.php',
            'sources/blocks/main_rating.php',
            'sources/blocks/main_trackback.php',
            'sources/feedback.php',
            'sources/feedback2.php',
            'sources/hooks/systems/addon_registry/core_feedback_features.php',
            'sources/hooks/systems/config/allow_own_rate.php',
            'sources/hooks/systems/config/comment_topic_subject.php',
            'sources/hooks/systems/config/comments_to_show_in_thread.php',
            'sources/hooks/systems/config/default_comment_sort_order.php',
            'sources/hooks/systems/config/enable_feedback.php',
            'sources/hooks/systems/config/is_on_comments.php',
            'sources/hooks/systems/config/is_on_rating.php',
            'sources/hooks/systems/config/is_on_trackbacks.php',
            'sources/hooks/systems/config/max_thread_depth.php',
            'sources/hooks/systems/config/simplify_wysiwyg_by_permissions.php',
            'sources/hooks/systems/notifications/comment_posted.php',
            'sources/hooks/systems/notifications/like.php',
            'sources/hooks/systems/page_groupings/trackbacks.php',
            'sources/hooks/systems/points/feedback__add.php',
            'sources/hooks/systems/points/feedback__received.php',
            'sources/hooks/systems/preview/comments.php',
            'sources/hooks/systems/rss/comments.php',
            'sources/hooks/systems/snippets/comments.php',
            'sources/hooks/systems/snippets/rating.php',
            'sources/hooks/systems/symbols/FORM_TO_EMAIL_URL.php',
            'sources/hooks/systems/symbols/SHOW_RATINGS.php',
            'sources/hooks/systems/trackback/.htaccess',
            'sources/hooks/systems/trackback/index.html',
            'sources/topics.php',
            'sources_custom/hooks/systems/trackback/.htaccess',
            'sources_custom/hooks/systems/trackback/index.html',
            'themes/default/images/icons/feedback/comment.svg',
            'themes/default/images/icons/feedback/comments_topic.svg',
            'themes/default/images/icons/feedback/dislike.svg',
            'themes/default/images/icons/feedback/dislike2.svg',
            'themes/default/images/icons/feedback/like.svg',
            'themes/default/images/icons/feedback/like2.svg',
            'themes/default/images/icons/feedback/rate.svg',
            'themes/default/images/icons/menu/adminzone/audit/trackbacks.svg',
            'themes/default/images/icons_monochrome/feedback/comment.svg',
            'themes/default/images/icons_monochrome/feedback/comments_topic.svg',
            'themes/default/images/icons_monochrome/feedback/dislike.svg',
            'themes/default/images/icons_monochrome/feedback/dislike2.svg',
            'themes/default/images/icons_monochrome/feedback/like.svg',
            'themes/default/images/icons_monochrome/feedback/like2.svg',
            'themes/default/images/icons_monochrome/feedback/rate.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/trackbacks.svg',
            'themes/default/javascript/core_feedback_features.js',
            'themes/default/templates/BLOCK_MAIN_CONTACT_SIMPLE.tpl',
            'themes/default/templates/COMMENTS_DEFAULT_TEXT.tpl',
            'themes/default/templates/COMMENTS_POSTING_FORM.tpl',
            'themes/default/templates/COMMENTS_POSTING_FORM_CAPTCHA.tpl',
            'themes/default/templates/COMMENTS_WRAPPER.tpl',
            'themes/default/templates/COMMENT_AJAX_HANDLER.tpl',
            'themes/default/templates/RATINGS_SHOW.tpl',
            'themes/default/templates/RATING_BOX.tpl',
            'themes/default/templates/RATING_DISPLAY_SHARED.tpl',
            'themes/default/templates/RATING_FORM.tpl',
            'themes/default/templates/RATING_INLINE_DYNAMIC.tpl',
            'themes/default/templates/RATING_INLINE_STATIC.tpl',
            'themes/default/templates/TRACKBACK.tpl',
            'themes/default/templates/TRACKBACK_DELETE_SCREEN.tpl',
            'themes/default/templates/TRACKBACK_WRAPPER.tpl',
            'themes/default/xml/TRACKBACK_XML.xml',
            'themes/default/xml/TRACKBACK_XML_ERROR.xml',
            'themes/default/xml/TRACKBACK_XML_LISTING.xml',
            'themes/default/xml/TRACKBACK_XML_NO_ERROR.xml',
            'themes/default/xml/TRACKBACK_XML_WRAPPER.xml',
        ];
    }

    /**
     * Get mapping between template names and the method of this class that can render a preview of them.
     *
     * @return array The mapping
     */
    public function tpl_previews() : array
    {
        return [
            'templates/COMMENTS_DEFAULT_TEXT.tpl' => 'comments_default_text',
            'templates/TRACKBACK.tpl' => 'trackback_wrapper',
            'templates/TRACKBACK_DELETE_SCREEN.tpl' => 'administrative__trackback_delete_screen',
            'xml/TRACKBACK_XML_NO_ERROR.xml' => 'trackback_xml_wrapper',
            'xml/TRACKBACK_XML_ERROR.xml' => 'trackback_xml_wrapper',
            'xml/TRACKBACK_XML_WRAPPER.xml' => 'trackback_xml_wrapper',
            'templates/COMMENTS_POSTING_FORM.tpl' => 'comments',
            'templates/COMMENTS_POSTING_FORM_CAPTCHA.tpl' => 'comments',
            'templates/RATING_BOX.tpl' => 'rating',
            'templates/RATING_DISPLAY_SHARED.tpl' => 'rating_display_shared',
            'templates/RATINGS_SHOW.tpl' => 'ratings_show',
            'templates/COMMENTS_WRAPPER.tpl' => 'comments',
            'xml/TRACKBACK_XML.xml' => 'trackback_xml_wrapper',
            'templates/TRACKBACK_WRAPPER.tpl' => 'trackback_wrapper',
            'xml/TRACKBACK_XML_LISTING.xml' => 'trackback_xml_listing',
            'templates/RATING_FORM.tpl' => 'rating',
            'templates/RATING_INLINE_STATIC.tpl' => 'rating_inline_static',
            'templates/RATING_INLINE_DYNAMIC.tpl' => 'rating_inline_dynamic',
            'templates/EMOTICON_CLICK_CODE.tpl' => 'comments',
            'templates/COMMENT_AJAX_HANDLER.tpl' => 'comments',
            'templates/POST.tpl' => 'comments',
            'templates/POST_CHILD_LOAD_LINK.tpl' => 'comments',
            'templates/BLOCK_MAIN_CONTACT_SIMPLE.tpl' => 'block_main_contact_simple',
        ];
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__comments_default_text() : object
    {
        return lorem_globalise(do_lorem_template('COMMENTS_DEFAULT_TEXT', []), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__trackback_delete_screen() : object
    {
        return lorem_globalise(do_lorem_template('TRACKBACK_DELETE_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'TRACKBACKS' => placeholder_trackbacks(),
            'LOTS' => lorem_phrase(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__trackback_xml_wrapper() : object
    {
        $xml = do_lorem_template('TRACKBACK_XML', [
            'TITLE' => lorem_phrase(),
            'LINK' => placeholder_url(),
            'EXCERPT' => lorem_phrase(),
        ], null, false, null, '.xml', 'xml');
        $xml->attach(do_lorem_template('TRACKBACK_XML_NO_ERROR', [], null, false, null, '.xml', 'xml'));
        $xml->attach(do_lorem_template('TRACKBACK_XML_ERROR', ['TRACKBACK_ERROR' => lorem_phrase()], null, false, null, '.xml', 'xml'));
        return do_lorem_template('TRACKBACK_XML_WRAPPER', [
            'XML' => $xml,
        ], null, false, null, '.xml', 'xml');
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__comments() : object
    {
        $ret = placeholder_comments(placeholder_comments_form());

        return lorem_globalise($ret, null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__trackback_wrapper() : object
    {
        return lorem_globalise(do_lorem_template('TRACKBACK_WRAPPER', [
            'TRACKBACKS' => placeholder_trackbacks(),
            'TRACKBACK_FEEDBACK_TYPE' => lorem_word(),
            'TRACKBACK_ID' => placeholder_codename(),
            'TRACKBACK_TITLE' => lorem_phrase(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__trackback_xml_listing() : object
    {
        $trackback_rendered_xml = do_lorem_template('TRACKBACK_XML', [
            'TITLE' => lorem_word(),
            'LINK' => placeholder_url(),
            'EXCERPT' => lorem_paragraph(),
        ], null, false, null, '.xml', 'xml');

        $content = do_lorem_template('TRACKBACK_XML_LISTING', [
            'ITEMS' => $trackback_rendered_xml,
            'LINK_PAGE' => lorem_word(),
            'LINK_ID' => placeholder_codename(),
        ], null, false, null, '.xml', 'xml');

        return $content;
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__rating() : object
    {
        return lorem_globalise(placeholder_rating(lorem_word()), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__rating_inline_static() : object
    {
        return lorem_globalise(placeholder_rating(lorem_word(), 'RATING_INLINE_STATIC'), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__rating_inline_dynamic() : object
    {
        return lorem_globalise(placeholder_rating(lorem_word(), 'RATING_INLINE_DYNAMIC'), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__rating_display_shared() : object
    {
        $all_rating_criteria = [];
        foreach (placeholder_array() as $i => $v) {
            $all_rating_criteria[] = [
                'TITLE' => lorem_phrase(),
                'RATING' => '3',
                '_OVERALL_NUM_RATINGS' => placeholder_number(),
                'OVERALL_NUM_RATINGS' => placeholder_number(),
                'TYPE' => lorem_word() . strval($i),
            ];
        }
        return lorem_globalise(do_lorem_template('RATING_DISPLAY_SHARED', [
            'CONTENT_TYPE' => lorem_word(),
            'RATING' => '3',
            'ID' => placeholder_codename(),
            'ALL_RATING_CRITERIA' => $all_rating_criteria,
            'HAS_RATINGS' => true,
            '_NUM_RATINGS' => placeholder_number(),
            'NUM_RATINGS' => placeholder_number(),
            '_OVERALL_NUM_RATINGS' => placeholder_number(),
            'OVERALL_NUM_RATINGS' => placeholder_number(),
            'RATING_FORM' => new Tempcode(),
            'TITLE' => lorem_phrase(),
            'ERROR' => '',
            'LIKES' => false,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__ratings_show() : object
    {
        $ratings = [];
        $ratings[] = [
            'RATING_MEMBER' => placeholder_numeric_id(),
            'RATING_USERNAME' => lorem_word(),
            'RATING_IP' => lorem_word(),
            'RATING_TIME' => placeholder_date_raw(),
            'RATING_TIME_FORMATTED' => placeholder_date(),
            'RATING' => '2',
        ];

        return lorem_globalise(do_lorem_template('RATINGS_SHOW', [
            'RATINGS' => $ratings,
            'HAS_MORE' => true,
            'MAX' => '1',
            'CNT' => '1',
            'CNT_REMAINING' => '10',
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_contact_simple() : object
    {
        require_lang('cns');

        return lorem_globalise(do_lorem_template('BLOCK_MAIN_CONTACT_SIMPLE', [
            'BLOCK_ID' => lorem_word(),
            'COMMENT_DETAILS' => placeholder_comments_form(false),
            'MESSAGE' => lorem_phrase(),
        ]), null, '', true);
    }
}
