<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_form_interfaces
 */

/**
 * Hook class.
 */
class Hook_addon_registry_core_form_interfaces
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the version of the software this addon is for (used in generating the TAR filename).
     *
     * @return float Version number
     */
    public function get_version() : float
    {
        return cms_version_number();
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'Architecture';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'Core rendering functionality for forms.';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [
            'tut_themes',
        ];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => [],
            'recommends' => [],
            'conflicts_with' => [],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/admin/component.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'data/block_helper.php',
            'data/ckeditor/.htaccess',
            'data/ckeditor/LICENSE.md',
            'data/ckeditor/build-config.js',
            'data/ckeditor/ckeditor.js',
            'data/ckeditor/cms_patch.sh',
            'data/ckeditor/contents.css',
            'data/ckeditor/index.html',
            'data/ckeditor/lang/af.js',
            'data/ckeditor/lang/ar.js',
            'data/ckeditor/lang/az.js',
            'data/ckeditor/lang/bg.js',
            'data/ckeditor/lang/bn.js',
            'data/ckeditor/lang/bs.js',
            'data/ckeditor/lang/ca.js',
            'data/ckeditor/lang/cs.js',
            'data/ckeditor/lang/cy.js',
            'data/ckeditor/lang/da.js',
            'data/ckeditor/lang/de-ch.js',
            'data/ckeditor/lang/de.js',
            'data/ckeditor/lang/el.js',
            'data/ckeditor/lang/en-au.js',
            'data/ckeditor/lang/en-ca.js',
            'data/ckeditor/lang/en-gb.js',
            'data/ckeditor/lang/en.js',
            'data/ckeditor/lang/eo.js',
            'data/ckeditor/lang/es-mx.js',
            'data/ckeditor/lang/es.js',
            'data/ckeditor/lang/et.js',
            'data/ckeditor/lang/eu.js',
            'data/ckeditor/lang/fa.js',
            'data/ckeditor/lang/fi.js',
            'data/ckeditor/lang/fo.js',
            'data/ckeditor/lang/fr-ca.js',
            'data/ckeditor/lang/fr.js',
            'data/ckeditor/lang/gl.js',
            'data/ckeditor/lang/gu.js',
            'data/ckeditor/lang/he.js',
            'data/ckeditor/lang/hi.js',
            'data/ckeditor/lang/hr.js',
            'data/ckeditor/lang/hu.js',
            'data/ckeditor/lang/id.js',
            'data/ckeditor/lang/index.html',
            'data/ckeditor/lang/is.js',
            'data/ckeditor/lang/it.js',
            'data/ckeditor/lang/ja.js',
            'data/ckeditor/lang/ka.js',
            'data/ckeditor/lang/km.js',
            'data/ckeditor/lang/ko.js',
            'data/ckeditor/lang/ku.js',
            'data/ckeditor/lang/lt.js',
            'data/ckeditor/lang/lv.js',
            'data/ckeditor/lang/mk.js',
            'data/ckeditor/lang/mn.js',
            'data/ckeditor/lang/ms.js',
            'data/ckeditor/lang/nb.js',
            'data/ckeditor/lang/nl.js',
            'data/ckeditor/lang/no.js',
            'data/ckeditor/lang/oc.js',
            'data/ckeditor/lang/pl.js',
            'data/ckeditor/lang/pt-br.js',
            'data/ckeditor/lang/pt.js',
            'data/ckeditor/lang/ro.js',
            'data/ckeditor/lang/ru.js',
            'data/ckeditor/lang/si.js',
            'data/ckeditor/lang/sk.js',
            'data/ckeditor/lang/sl.js',
            'data/ckeditor/lang/sq.js',
            'data/ckeditor/lang/sr-latn.js',
            'data/ckeditor/lang/sr.js',
            'data/ckeditor/lang/sv.js',
            'data/ckeditor/lang/th.js',
            'data/ckeditor/lang/tr.js',
            'data/ckeditor/lang/tt.js',
            'data/ckeditor/lang/ug.js',
            'data/ckeditor/lang/uk.js',
            'data/ckeditor/lang/vi.js',
            'data/ckeditor/lang/zh-cn.js',
            'data/ckeditor/lang/zh.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/a11yhelp.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/index.html',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/_translationstatus.txt',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/af.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ar.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/az.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/bg.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ca.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/cs.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/cy.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/da.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/de-ch.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/de.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/el.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/en-au.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/en-gb.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/en.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/eo.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/es-mx.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/es.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/et.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/eu.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/fa.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/fi.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/fo.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/fr-ca.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/fr.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/gl.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/gu.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/he.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/hi.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/hr.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/hu.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/id.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/index.html',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/it.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ja.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/km.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ko.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ku.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/lt.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/lv.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/mk.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/mn.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/nb.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/nl.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/no.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/oc.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/pl.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/pt-br.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/pt.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ro.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ru.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/si.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/sk.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/sl.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/sq.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/sr-latn.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/sr.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/sv.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/th.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/tr.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/tt.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/ug.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/uk.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/vi.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/zh-cn.js',
            'data/ckeditor/plugins/a11yhelp/dialogs/lang/zh.js',
            'data/ckeditor/plugins/about/dialogs/about.js',
            'data/ckeditor/plugins/about/dialogs/hidpi/index.html',
            'data/ckeditor/plugins/about/dialogs/hidpi/logo_ckeditor.png',
            'data/ckeditor/plugins/about/dialogs/index.html',
            'data/ckeditor/plugins/about/dialogs/logo_ckeditor.png',
            'data/ckeditor/plugins/base64image/LICENSE.md',
            'data/ckeditor/plugins/base64image/README.md',
            'data/ckeditor/plugins/base64image/dialogs/base64image.js',
            'data/ckeditor/plugins/base64image/dialogs/index.html',
            'data/ckeditor/plugins/base64image/index.html',
            'data/ckeditor/plugins/clipboard/dialogs/index.html',
            'data/ckeditor/plugins/clipboard/dialogs/paste.js',
            'data/ckeditor/plugins/codemirror/css/codemirror.min.css',
            'data/ckeditor/plugins/codemirror/css/index.html',
            'data/ckeditor/plugins/codemirror/images/autocomplete.png',
            'data/ckeditor/plugins/codemirror/images/autoformat.png',
            'data/ckeditor/plugins/codemirror/images/commentselectedrange.png',
            'data/ckeditor/plugins/codemirror/images/index.html',
            'data/ckeditor/plugins/codemirror/images/searchcode.png',
            'data/ckeditor/plugins/codemirror/images/uncommentselectedrange.png',
            'data/ckeditor/plugins/codemirror/js/beautify-html.js',
            'data/ckeditor/plugins/codemirror/js/beautify.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.addons.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.addons.search.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.bbcode.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.bbcodemixed.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.handlebars.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.htmlmixed.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.javascript.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.php.min.js',
            'data/ckeditor/plugins/codemirror/js/codemirror.mode.twig.min.js',
            'data/ckeditor/plugins/codemirror/js/index.html',
            'data/ckeditor/plugins/codemirror/theme/3024-day.css',
            'data/ckeditor/plugins/codemirror/theme/3024-night.css',
            'data/ckeditor/plugins/codemirror/theme/abcdef.css',
            'data/ckeditor/plugins/codemirror/theme/ambiance-mobile.css',
            'data/ckeditor/plugins/codemirror/theme/ambiance.css',
            'data/ckeditor/plugins/codemirror/theme/ayu-dark.css',
            'data/ckeditor/plugins/codemirror/theme/ayu-mirage.css',
            'data/ckeditor/plugins/codemirror/theme/base16-dark.css',
            'data/ckeditor/plugins/codemirror/theme/base16-light.css',
            'data/ckeditor/plugins/codemirror/theme/bespin.css',
            'data/ckeditor/plugins/codemirror/theme/blackboard.css',
            'data/ckeditor/plugins/codemirror/theme/cobalt.css',
            'data/ckeditor/plugins/codemirror/theme/colorforth.css',
            'data/ckeditor/plugins/codemirror/theme/darcula.css',
            'data/ckeditor/plugins/codemirror/theme/dracula.css',
            'data/ckeditor/plugins/codemirror/theme/duotone-dark.css',
            'data/ckeditor/plugins/codemirror/theme/duotone-light.css',
            'data/ckeditor/plugins/codemirror/theme/eclipse.css',
            'data/ckeditor/plugins/codemirror/theme/elegant.css',
            'data/ckeditor/plugins/codemirror/theme/erlang-dark.css',
            'data/ckeditor/plugins/codemirror/theme/gruvbox-dark.css',
            'data/ckeditor/plugins/codemirror/theme/hopscotch.css',
            'data/ckeditor/plugins/codemirror/theme/icecoder.css',
            'data/ckeditor/plugins/codemirror/theme/idea.css',
            'data/ckeditor/plugins/codemirror/theme/index.html',
            'data/ckeditor/plugins/codemirror/theme/isotope.css',
            'data/ckeditor/plugins/codemirror/theme/lesser-dark.css',
            'data/ckeditor/plugins/codemirror/theme/liquibyte.css',
            'data/ckeditor/plugins/codemirror/theme/lucario.css',
            'data/ckeditor/plugins/codemirror/theme/material-darker.css',
            'data/ckeditor/plugins/codemirror/theme/material-ocean.css',
            'data/ckeditor/plugins/codemirror/theme/material-palenight.css',
            'data/ckeditor/plugins/codemirror/theme/material.css',
            'data/ckeditor/plugins/codemirror/theme/mbo.css',
            'data/ckeditor/plugins/codemirror/theme/mdn-like.css',
            'data/ckeditor/plugins/codemirror/theme/midnight.css',
            'data/ckeditor/plugins/codemirror/theme/monokai.css',
            'data/ckeditor/plugins/codemirror/theme/moxer.css',
            'data/ckeditor/plugins/codemirror/theme/neat.css',
            'data/ckeditor/plugins/codemirror/theme/neo.css',
            'data/ckeditor/plugins/codemirror/theme/night.css',
            'data/ckeditor/plugins/codemirror/theme/nord.css',
            'data/ckeditor/plugins/codemirror/theme/oceanic-next.css',
            'data/ckeditor/plugins/codemirror/theme/panda-syntax.css',
            'data/ckeditor/plugins/codemirror/theme/paraiso-dark.css',
            'data/ckeditor/plugins/codemirror/theme/paraiso-light.css',
            'data/ckeditor/plugins/codemirror/theme/pastel-on-dark.css',
            'data/ckeditor/plugins/codemirror/theme/railscasts.css',
            'data/ckeditor/plugins/codemirror/theme/rubyblue.css',
            'data/ckeditor/plugins/codemirror/theme/seti.css',
            'data/ckeditor/plugins/codemirror/theme/shadowfox.css',
            'data/ckeditor/plugins/codemirror/theme/solarized.css',
            'data/ckeditor/plugins/codemirror/theme/ssms.css',
            'data/ckeditor/plugins/codemirror/theme/the-matrix.css',
            'data/ckeditor/plugins/codemirror/theme/tomorrow-night-bright.css',
            'data/ckeditor/plugins/codemirror/theme/tomorrow-night-eighties.css',
            'data/ckeditor/plugins/codemirror/theme/ttcn.css',
            'data/ckeditor/plugins/codemirror/theme/twilight.css',
            'data/ckeditor/plugins/codemirror/theme/vibrant-ink.css',
            'data/ckeditor/plugins/codemirror/theme/xq-dark.css',
            'data/ckeditor/plugins/codemirror/theme/xq-light.css',
            'data/ckeditor/plugins/codemirror/theme/yeti.css',
            'data/ckeditor/plugins/codemirror/theme/yonce.css',
            'data/ckeditor/plugins/codemirror/theme/zenburn.css',
            'data/ckeditor/plugins/colordialog/dialogs/colordialog.css',
            'data/ckeditor/plugins/colordialog/dialogs/colordialog.js',
            'data/ckeditor/plugins/colordialog/dialogs/index.html',
            'data/ckeditor/plugins/composr/images/composr_block.png',
            'data/ckeditor/plugins/composr/images/composr_box.png',
            'data/ckeditor/plugins/composr/images/composr_code.png',
            'data/ckeditor/plugins/composr/images/composr_comcode.png',
            'data/ckeditor/plugins/composr/images/composr_image.png',
            'data/ckeditor/plugins/composr/images/composr_image__moono.png',
            'data/ckeditor/plugins/composr/images/composr_page.png',
            'data/ckeditor/plugins/composr/images/composr_quote.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_block.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_box.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_code.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_comcode.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_image.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_image__moono.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_page.png',
            'data/ckeditor/plugins/composr/images/hidpi/composr_quote.png',
            'data/ckeditor/plugins/composr/images/hidpi/index.html',
            'data/ckeditor/plugins/composr/images/index.html',
            'data/ckeditor/plugins/composr/index.html',
            'data/ckeditor/plugins/composr/plugin.js',
            'data/ckeditor/plugins/dialog/dialogDefinition.js',
            'data/ckeditor/plugins/dialog/index.html',
            'data/ckeditor/plugins/dialog/styles/dialog.css',
            'data/ckeditor/plugins/dialog/styles/index.html',
            'data/ckeditor/plugins/emojione/.bower.json',
            'data/ckeditor/plugins/emojione/.gitignore',
            'data/ckeditor/plugins/emojione/README.md',
            'data/ckeditor/plugins/emojione/bower.json',
            'data/ckeditor/plugins/emojione/dialogs/emojione.js',
            'data/ckeditor/plugins/emojione/dialogs/index.html',
            'data/ckeditor/plugins/emojione/index.html',
            'data/ckeditor/plugins/emojione/libs/emojione/emojione.min.js',
            'data/ckeditor/plugins/emojione/libs/emojione/index.html',
            'data/ckeditor/plugins/emojione/styles/emojione-awesome.css',
            'data/ckeditor/plugins/emojione/styles/emojione.css',
            'data/ckeditor/plugins/emojione/styles/index.html',
            'data/ckeditor/plugins/find/dialogs/find.js',
            'data/ckeditor/plugins/find/dialogs/index.html',
            'data/ckeditor/plugins/icons.png',
            'data/ckeditor/plugins/icons_hidpi.png',
            'data/ckeditor/plugins/image2/dialogs/image2.js',
            'data/ckeditor/plugins/image2/dialogs/index.html',
            'data/ckeditor/plugins/imageresponsive/README.md',
            'data/ckeditor/plugins/imageresponsive/index.html',
            'data/ckeditor/plugins/index.html',
            'data/ckeditor/plugins/lineheight/LICENSE',
            'data/ckeditor/plugins/lineheight/README.md',
            'data/ckeditor/plugins/lineheight/index.html',
            'data/ckeditor/plugins/lineheight/readme.txt',
            'data/ckeditor/plugins/link/dialogs/anchor.js',
            'data/ckeditor/plugins/link/dialogs/index.html',
            'data/ckeditor/plugins/link/dialogs/link.js',
            'data/ckeditor/plugins/link/images/anchor.png',
            'data/ckeditor/plugins/link/images/hidpi/anchor.png',
            'data/ckeditor/plugins/link/images/hidpi/index.html',
            'data/ckeditor/plugins/link/images/index.html',
            'data/ckeditor/plugins/liststyle/dialogs/index.html',
            'data/ckeditor/plugins/liststyle/dialogs/liststyle.js',
            'data/ckeditor/plugins/magicline/images/hidpi/icon-rtl.png',
            'data/ckeditor/plugins/magicline/images/hidpi/icon.png',
            'data/ckeditor/plugins/magicline/images/hidpi/index.html',
            'data/ckeditor/plugins/magicline/images/icon-rtl.png',
            'data/ckeditor/plugins/magicline/images/icon.png',
            'data/ckeditor/plugins/magicline/images/index.html',
            'data/ckeditor/plugins/pastecode/CHANGES.md',
            'data/ckeditor/plugins/pastecode/index.html',
            'data/ckeditor/plugins/pastefromgdocs/filter/default.js',
            'data/ckeditor/plugins/pastefromgdocs/filter/index.html',
            'data/ckeditor/plugins/pastefromlibreoffice/filter/default.js',
            'data/ckeditor/plugins/pastefromlibreoffice/filter/index.html',
            'data/ckeditor/plugins/pastefromword/filter/default.js',
            'data/ckeditor/plugins/pastefromword/filter/index.html',
            'data/ckeditor/plugins/pastetools/filter/common.js',
            'data/ckeditor/plugins/pastetools/filter/image.js',
            'data/ckeditor/plugins/pastetools/filter/index.html',
            'data/ckeditor/plugins/scayt/CHANGELOG.md',
            'data/ckeditor/plugins/scayt/LICENSE.md',
            'data/ckeditor/plugins/scayt/README.md',
            'data/ckeditor/plugins/scayt/dialogs/dialog.css',
            'data/ckeditor/plugins/scayt/dialogs/index.html',
            'data/ckeditor/plugins/scayt/dialogs/options.js',
            'data/ckeditor/plugins/scayt/dialogs/toolbar.css',
            'data/ckeditor/plugins/scayt/index.html',
            'data/ckeditor/plugins/scayt/skins/moono-lisa/index.html',
            'data/ckeditor/plugins/scayt/skins/moono-lisa/scayt.css',
            'data/ckeditor/plugins/showblocks/images/block_address.png',
            'data/ckeditor/plugins/showblocks/images/block_blockquote.png',
            'data/ckeditor/plugins/showblocks/images/block_div.png',
            'data/ckeditor/plugins/showblocks/images/block_h1.png',
            'data/ckeditor/plugins/showblocks/images/block_h2.png',
            'data/ckeditor/plugins/showblocks/images/block_h3.png',
            'data/ckeditor/plugins/showblocks/images/block_h4.png',
            'data/ckeditor/plugins/showblocks/images/block_h5.png',
            'data/ckeditor/plugins/showblocks/images/block_h6.png',
            'data/ckeditor/plugins/showblocks/images/block_p.png',
            'data/ckeditor/plugins/showblocks/images/block_pre.png',
            'data/ckeditor/plugins/showblocks/images/index.html',
            'data/ckeditor/plugins/showcomcodeblocks/index.html',
            'data/ckeditor/plugins/showcomcodeblocks/plugin.js',
            'data/ckeditor/plugins/specialchar/dialogs/index.html',
            'data/ckeditor/plugins/specialchar/dialogs/lang/_translationstatus.txt',
            'data/ckeditor/plugins/specialchar/dialogs/lang/af.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ar.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/az.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/bg.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ca.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/cs.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/cy.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/da.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/de-ch.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/de.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/el.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/en-au.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/en-ca.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/en-gb.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/en.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/eo.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/es-mx.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/es.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/et.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/eu.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/fa.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/fi.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/fr-ca.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/fr.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/gl.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/he.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/hr.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/hu.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/id.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/index.html',
            'data/ckeditor/plugins/specialchar/dialogs/lang/it.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ja.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/km.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ko.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ku.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/lt.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/lv.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/nb.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/nl.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/no.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/oc.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/pl.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/pt-br.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/pt.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ro.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ru.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/si.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/sk.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/sl.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/sq.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/sr-latn.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/sr.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/sv.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/th.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/tr.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/tt.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/ug.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/uk.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/vi.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/zh-cn.js',
            'data/ckeditor/plugins/specialchar/dialogs/lang/zh.js',
            'data/ckeditor/plugins/specialchar/dialogs/specialchar.js',
            'data/ckeditor/plugins/spellchecktoggle/images/hidpi/index.html',
            'data/ckeditor/plugins/spellchecktoggle/images/hidpi/spellchecktoggle.png',
            'data/ckeditor/plugins/spellchecktoggle/images/hidpi/spellchecktoggle__moono.png',
            'data/ckeditor/plugins/spellchecktoggle/images/index.html',
            'data/ckeditor/plugins/spellchecktoggle/images/spellchecktoggle.png',
            'data/ckeditor/plugins/spellchecktoggle/images/spellchecktoggle__moono.png',
            'data/ckeditor/plugins/spellchecktoggle/index.html',
            'data/ckeditor/plugins/spellchecktoggle/plugin.js',
            'data/ckeditor/plugins/table/dialogs/index.html',
            'data/ckeditor/plugins/table/dialogs/table.js',
            'data/ckeditor/plugins/tableselection/styles/index.html',
            'data/ckeditor/plugins/tableselection/styles/tableselection.css',
            'data/ckeditor/plugins/tabletools/dialogs/index.html',
            'data/ckeditor/plugins/tabletools/dialogs/tableCell.js',
            'data/ckeditor/plugins/widget/images/handle.png',
            'data/ckeditor/plugins/widget/images/index.html',
            'data/ckeditor/skins/kama/dialog.css',
            'data/ckeditor/skins/kama/dialog_ie.css',
            'data/ckeditor/skins/kama/dialog_ie7.css',
            'data/ckeditor/skins/kama/dialog_ie8.css',
            'data/ckeditor/skins/kama/dialog_iequirks.css',
            'data/ckeditor/skins/kama/editor.css',
            'data/ckeditor/skins/kama/editor_ie.css',
            'data/ckeditor/skins/kama/editor_ie7.css',
            'data/ckeditor/skins/kama/editor_ie8.css',
            'data/ckeditor/skins/kama/editor_iequirks.css',
            'data/ckeditor/skins/kama/icons.png',
            'data/ckeditor/skins/kama/icons_hidpi.png',
            'data/ckeditor/skins/kama/images/dialog_sides.gif',
            'data/ckeditor/skins/kama/images/dialog_sides.png',
            'data/ckeditor/skins/kama/images/dialog_sides_rtl.png',
            'data/ckeditor/skins/kama/images/index.html',
            'data/ckeditor/skins/kama/images/mini.gif',
            'data/ckeditor/skins/kama/images/spinner.gif',
            'data/ckeditor/skins/kama/images/sprites.png',
            'data/ckeditor/skins/kama/images/sprites_ie6.png',
            'data/ckeditor/skins/kama/images/toolbar_start.gif',
            'data/ckeditor/skins/kama/index.html',
            'data/ckeditor/skins/kama/readme.md',
            'data/ckeditor/skins/moono/dev/icons16.png',
            'data/ckeditor/skins/moono/dev/icons16.svg',
            'data/ckeditor/skins/moono/dev/icons32.png',
            'data/ckeditor/skins/moono/dev/icons32.svg',
            'data/ckeditor/skins/moono/dev/index.html',
            'data/ckeditor/skins/moono/dev/locations.json',
            'data/ckeditor/skins/moono/dialog.css',
            'data/ckeditor/skins/moono/dialog_ie.css',
            'data/ckeditor/skins/moono/dialog_ie7.css',
            'data/ckeditor/skins/moono/dialog_ie8.css',
            'data/ckeditor/skins/moono/dialog_iequirks.css',
            'data/ckeditor/skins/moono/editor.css',
            'data/ckeditor/skins/moono/editor_gecko.css',
            'data/ckeditor/skins/moono/editor_ie.css',
            'data/ckeditor/skins/moono/editor_ie7.css',
            'data/ckeditor/skins/moono/editor_ie8.css',
            'data/ckeditor/skins/moono/editor_iequirks.css',
            'data/ckeditor/skins/moono/icons.png',
            'data/ckeditor/skins/moono/icons_hidpi.png',
            'data/ckeditor/skins/moono/images/anchor.png',
            'data/ckeditor/skins/moono/images/arrow.png',
            'data/ckeditor/skins/moono/images/close.png',
            'data/ckeditor/skins/moono/images/hidpi/anchor.png',
            'data/ckeditor/skins/moono/images/hidpi/close.png',
            'data/ckeditor/skins/moono/images/hidpi/index.html',
            'data/ckeditor/skins/moono/images/hidpi/lock-open.png',
            'data/ckeditor/skins/moono/images/hidpi/lock.png',
            'data/ckeditor/skins/moono/images/hidpi/refresh.png',
            'data/ckeditor/skins/moono/images/index.html',
            'data/ckeditor/skins/moono/images/lock-open.png',
            'data/ckeditor/skins/moono/images/lock.png',
            'data/ckeditor/skins/moono/images/refresh.png',
            'data/ckeditor/skins/moono/images/spinner.gif',
            'data/ckeditor/skins/moono/index.html',
            'data/ckeditor/skins/moono/readme.md',
            'data/ckeditor/skins/moono/skin.js',
            'data/ckeditor/skins/moonocolor/dialog.css',
            'data/ckeditor/skins/moonocolor/dialog_ie.css',
            'data/ckeditor/skins/moonocolor/dialog_ie7.css',
            'data/ckeditor/skins/moonocolor/dialog_ie8.css',
            'data/ckeditor/skins/moonocolor/dialog_iequirks.css',
            'data/ckeditor/skins/moonocolor/editor.css',
            'data/ckeditor/skins/moonocolor/editor_gecko.css',
            'data/ckeditor/skins/moonocolor/editor_ie.css',
            'data/ckeditor/skins/moonocolor/editor_ie7.css',
            'data/ckeditor/skins/moonocolor/editor_ie8.css',
            'data/ckeditor/skins/moonocolor/editor_iequirks.css',
            'data/ckeditor/skins/moonocolor/icons.png',
            'data/ckeditor/skins/moonocolor/icons_hidpi.png',
            'data/ckeditor/skins/moonocolor/images/arrow.png',
            'data/ckeditor/skins/moonocolor/images/close.png',
            'data/ckeditor/skins/moonocolor/images/hidpi/close.png',
            'data/ckeditor/skins/moonocolor/images/hidpi/index.html',
            'data/ckeditor/skins/moonocolor/images/hidpi/lock-open.png',
            'data/ckeditor/skins/moonocolor/images/hidpi/lock.png',
            'data/ckeditor/skins/moonocolor/images/hidpi/refresh.png',
            'data/ckeditor/skins/moonocolor/images/index.html',
            'data/ckeditor/skins/moonocolor/images/lock-open.png',
            'data/ckeditor/skins/moonocolor/images/lock.png',
            'data/ckeditor/skins/moonocolor/images/refresh.png',
            'data/ckeditor/skins/moonocolor/images/spinner.gif',
            'data/ckeditor/skins/moonocolor/index.html',
            'data/ckeditor/skins/moonocolor/readme.md',
            'data/ckeditor/skins/moonocolor/skin.js',
            'data/ckeditor/styles.js',
            'data/ckeditor/vendor/index.html',
            'data/ckeditor/vendor/promise.js',
            'data/namelike.php',
            'data/preview.php',
            'data/username_exists.php',
            'data_custom/spelling/index.html',
            'data_custom/spelling/personal_dicts/.htaccess',
            'data_custom/spelling/personal_dicts/index.html',
            'sources/form_templates.php',
            'sources/hooks/systems/addon_registry/core_form_interfaces.php',
            'sources/hooks/systems/symbols/CKEDITOR_PATH.php',
            'sources/hooks/systems/symbols/COMCODE_TAGS.php',
            'sources/preview.php',
            'sources/spelling.php',
            'themes/default/css/forms.css',
            'themes/default/css/widget_color.css',
            'themes/default/css/widget_select2.css',
            'themes/default/images/EN/comcode_editor/apply_changes.svg',
            'themes/default/images/EN/comcode_editor/b.svg',
            'themes/default/images/EN/comcode_editor/block.svg',
            'themes/default/images/EN/comcode_editor/box.svg',
            'themes/default/images/EN/comcode_editor/code.svg',
            'themes/default/images/EN/comcode_editor/comcode.svg',
            'themes/default/images/EN/comcode_editor/email.svg',
            'themes/default/images/EN/comcode_editor/hide.svg',
            'themes/default/images/EN/comcode_editor/html.svg',
            'themes/default/images/EN/comcode_editor/i.svg',
            'themes/default/images/EN/comcode_editor/img.svg',
            'themes/default/images/EN/comcode_editor/index.html',
            'themes/default/images/EN/comcode_editor/list.svg',
            'themes/default/images/EN/comcode_editor/page.svg',
            'themes/default/images/EN/comcode_editor/quote.svg',
            'themes/default/images/EN/comcode_editor/s.svg',
            'themes/default/images/EN/comcode_editor/thumb.svg',
            'themes/default/images/EN/comcode_editor/u.svg',
            'themes/default/images/EN/comcode_editor/url.svg',
            'themes/default/images/icons/tree_field/category.svg',
            'themes/default/images/icons/tree_field/collapse.svg',
            'themes/default/images/icons/tree_field/entry.svg',
            'themes/default/images/icons/tree_field/expand.svg',
            'themes/default/images/icons/tree_field/index.html',
            'themes/default/images/icons_monochrome/tree_field/category.svg',
            'themes/default/images/icons_monochrome/tree_field/collapse.svg',
            'themes/default/images/icons_monochrome/tree_field/entry.svg',
            'themes/default/images/icons_monochrome/tree_field/expand.svg',
            'themes/default/images/icons_monochrome/tree_field/index.html',
            'themes/default/javascript/_attachment_ui_defaults.js',
            'themes/default/javascript/_wysiwyg_settings.js',
            'themes/default/javascript/ajax_people_lists.js',
            'themes/default/javascript/checking.js',
            'themes/default/javascript/core_form_interfaces.js',
            'themes/default/javascript/editing.js',
            'themes/default/javascript/posting.js',
            'themes/default/javascript/select2.js',
            'themes/default/javascript/widget_color.js',
            'themes/default/templates/BLOCK_HELPER_BLOCK_CHOICE.tpl',
            'themes/default/templates/BLOCK_HELPER_BLOCK_GROUP.tpl',
            'themes/default/templates/BLOCK_HELPER_DONE.tpl',
            'themes/default/templates/BLOCK_HELPER_START.tpl',
            'themes/default/templates/COMCODE_EDITOR.tpl',
            'themes/default/templates/FORM.tpl',
            'themes/default/templates/FORM_DESCRIP_SEP.tpl',
            'themes/default/templates/FORM_FIELD_SET_GROUPER.tpl',
            'themes/default/templates/FORM_GROUP.tpl',
            'themes/default/templates/FORM_GROUPED.tpl',
            'themes/default/templates/FORM_SCREEN.tpl',
            'themes/default/templates/FORM_SCREEN_ARE_REQUIRED.tpl',
            'themes/default/templates/FORM_SCREEN_FIELD.tpl',
            'themes/default/templates/FORM_SCREEN_FIELDS_SET.tpl',
            'themes/default/templates/FORM_SCREEN_FIELDS_SET_ITEM.tpl',
            'themes/default/templates/FORM_SCREEN_FIELD_DESCRIPTION.tpl',
            'themes/default/templates/FORM_SCREEN_FIELD_SPACER.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_ALL_AND_NOT.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_AUTHOR.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_CODENAME.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_COLOUR.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_COMBO.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_DATE.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_DATE_COMPONENTS.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_DIMENSIONS.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_EMAIL.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_FLOAT.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_HIDDEN.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_HIDDEN_2.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_HUGE.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_HUGE_COMCODE.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_HUGE_LIST.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_INTEGER.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_LINE.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_LINE_MULTI.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_LIST.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_LIST_ENTRY.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_LIST_GROUP.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_MULTI_LIST.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_PASSWORD.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_PERMISSION.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_PERMISSION_ADMIN.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_PERMISSION_MATRIX.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_PERMISSION_MATRIX_OUTER.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_PERMISSION_OVERRIDE.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_RADIO_LIST.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_RADIO_LIST_COMBO_ENTRY.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_RADIO_LIST_ENTRY.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_RADIO_LIST_ENTRY_PICTURE.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_TEXT.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_TEXT_MULTI.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_THEME_IMAGE_CATEGORY.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_THEME_IMAGE_ENTRY.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_TICK.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_TIME.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_TREE_LIST.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_UPLOAD.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_UPLOAD_MULTI.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_URL.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_USERNAME.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_USERNAME_MULTI.tpl',
            'themes/default/templates/FORM_SCREEN_INPUT_VARIOUS_TICKS.tpl',
            'themes/default/templates/FORM_SINGLE_FIELD.tpl',
            'themes/default/templates/FORM_STANDARD_END.tpl',
            'themes/default/templates/FORM_STANDARD_START.tpl',
            'themes/default/templates/POSTING_FORM.tpl',
            'themes/default/templates/POSTING_SCREEN.tpl',
            'themes/default/templates/PREVIEW_SCRIPT.tpl',
            'themes/default/templates/PREVIEW_SCRIPT_CODE.tpl',
            'themes/default/templates/PREVIEW_SCRIPT_KEYWORD_DENSITY.tpl',
            'themes/default/templates/PREVIEW_SCRIPT_SPELLING.tpl',
            'themes/default/templates/WYSIWYG_LOAD.tpl',
        ];
    }

    /**
     * Get mapping between template names and the method of this class that can render a preview of them.
     *
     * @return array The mapping
     */
    public function tpl_previews() : array
    {
        return [
            'templates/FORM_SCREEN_INPUT_COMBO.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_FIELD_SPACER.tpl' => 'form_screen_1',
            'templates/FORM_STANDARD_START.tpl' => 'form_screen_1',
            'templates/FORM_STANDARD_END.tpl' => 'form_screen_1',
            'templates/FORM.tpl' => 'form_screen_1',
            'templates/FORM_SINGLE_FIELD.tpl' => 'form_single_field',
            'templates/POSTING_SCREEN.tpl' => 'posting_screen',
            'templates/FORM_SCREEN_INPUT_LIST_ENTRY.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_LIST_GROUP.tpl' => 'form_screen_1',
            'templates/FORM_GROUP.tpl' => 'form_group',
            'templates/FORM_GROUPED.tpl' => 'form_grouped',
            'templates/FORM_FIELD_SET_GROUPER.tpl' => 'form_field_set_grouper',
            'templates/PREVIEW_SCRIPT_SPELLING.tpl' => 'preview_script_spelling',
            'templates/PREVIEW_SCRIPT_KEYWORD_DENSITY.tpl' => 'preview_script_keyword_density',
            'templates/PREVIEW_SCRIPT_CODE.tpl' => 'preview_script_code',
            'templates/FORM_SCREEN_INPUT_RADIO_LIST_ENTRY_PICTURE.tpl' => 'form_screen',
            'templates/WYSIWYG_LOAD.tpl' => 'wysiwyg_load',
            'templates/POSTING_FORM.tpl' => 'posting_form',
            'templates/COMCODE_MESSAGE.tpl' => 'form_screen_1',
            'templates/COMCODE_EDITOR.tpl' => 'posting_form',
            'templates/COMCODE_EDITOR_BUTTON.tpl' => 'posting_form',
            'templates/COMCODE_EDITOR_MICRO_BUTTON.tpl' => 'posting_form',
            'templates/FORM_SCREEN_INPUT_CODENAME.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_LINE.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_URL.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_FIELDS_SET_ITEM.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_FIELDS_SET.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_USERNAME.tpl' => 'form_screen_2',
            'templates/FORM_DESCRIP_SEP.tpl' => 'form_descrip_sep',
            'templates/FORM_SCREEN_INPUT_AUTHOR.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_EMAIL.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_COLOUR.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_LINE_MULTI.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_TEXT_MULTI.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_USERNAME_MULTI.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_TEXT.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_HUGE_COMCODE.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_HUGE.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_PASSWORD.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_TICK.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_VARIOUS_TICKS.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_UPLOAD.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_UPLOAD_MULTI.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_LIST.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_TREE_LIST.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_HUGE_LIST.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_MULTI_LIST.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_ALL_AND_NOT.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_RADIO_LIST.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_THEME_IMAGE_ENTRY.tpl' => 'form_screen',
            'templates/FORM_SCREEN_INPUT_THEME_IMAGE_CATEGORY.tpl' => 'form_screen',
            'templates/FORM_SCREEN_FIELD_DESCRIPTION.tpl' => 'form_screen',
            'templates/FORM_SCREEN_ARE_REQUIRED.tpl' => 'form_screen',
            'templates/POSTING_FIELD.tpl' => 'form_screen',
            'templates/FORM_SCREEN_INPUT_TIME.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_DATE.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_DATE_COMPONENTS.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_INTEGER.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_DIMENSIONS.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_FLOAT.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_HIDDEN_2.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_FIELD.tpl' => 'form_screen_1',
            'templates/FORM_SCREEN_INPUT_RADIO_LIST_ENTRY.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_RADIO_LIST_COMBO_ENTRY.tpl' => 'form_screen_2',
            'templates/FORM_SCREEN_INPUT_PERMISSION_OVERRIDE.tpl' => 'administrative__form_screen_input_permission',
            'templates/FORM_SCREEN_INPUT_PERMISSION.tpl' => 'administrative__form_screen_input_permission',
            'templates/FORM_SCREEN_INPUT_PERMISSION_ADMIN.tpl' => 'administrative__form_screen_input_permission',
            'templates/FORM_SCREEN_INPUT_PERMISSION_MATRIX.tpl' => 'administrative__form_screen_input_permission',
            'templates/FORM_SCREEN_INPUT_PERMISSION_MATRIX_OUTER.tpl' => 'administrative__form_screen_input_permission_outer',
            'templates/FORM_SCREEN_INPUT_HIDDEN.tpl' => 'form_screen_2',
            'templates/PREVIEW_SCRIPT.tpl' => 'preview_script',
            'templates/BLOCK_HELPER_BLOCK_CHOICE.tpl' => 'administrative__block_helper_block_group',
            'templates/BLOCK_HELPER_BLOCK_GROUP.tpl' => 'administrative__block_helper_block_group',
            'templates/BLOCK_HELPER_START.tpl' => 'administrative__block_helper_start',
            'templates/BLOCK_HELPER_DONE.tpl' => 'administrative__block_helper_done',
            'javascript/_wysiwyg_settings.js' => 'wysiwyg_settings',
            'javascript/_attachment_ui_defaults.js' => 'attachment_ui_defaults',
        ];
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__wysiwyg_settings() : object
    {
        return do_lorem_template('_wysiwyg_settings', [], null, false, null, '.js', 'javascript');
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__attachment_ui_defaults() : object
    {
        return do_lorem_template('_attachment_ui_defaults', [], null, false, null, '.js', 'javascript');
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_single_field() : object
    {
        $fields = new Tempcode();

        $name = placeholder_codename();
        $fields->attach(do_lorem_template('FORM_SCREEN_INPUT_FLOAT', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
        ]));

        return lorem_globalise(do_lorem_template('FORM_SINGLE_FIELD', [
            'GET' => false,
            'HIDDEN' => '',
            'LABEL' => lorem_phrase(),
            'NAME' => $name,
            'URL' => placeholder_url(),
            'FIELD' => $fields,
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'TEXT' => lorem_sentence_html(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_screen_1() : object
    {
        require_lang('comcode');
        require_lang('dates');
        require_javascript('theme_colours');

        $fields = new Tempcode();

        $name = placeholder_random_id();
        $_input1 = do_lorem_template('FORM_SCREEN_INPUT_LINE', [
            'MAXLENGTH' => placeholder_number(),
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name . '1',
            'DEFAULT' => '',
            'SIZE' => '40',
        ]);
        $input1 = do_lorem_template('FORM_SCREEN_FIELDS_SET_ITEM', [
            'SET_NAME' => $name,
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name . '1',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $_input1,
            'COMCODE' => '',
        ]);
        $_input2 = do_lorem_template('FORM_SCREEN_INPUT_URL', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name . '2',
            'DEFAULT' => '',
        ]);
        $input2 = do_lorem_template('FORM_SCREEN_FIELDS_SET_ITEM', [
            'SET_NAME' => $name,
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name . '2',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $_input2,
            'COMCODE' => '',
        ]);
        $inputs = new Tempcode();
        $inputs->attach($input1);
        $inputs->attach($input2);
        $fieldset = do_lorem_template('FORM_SCREEN_FIELDS_SET', [
            'SET_NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'FIELDS' => $inputs,
            'REQUIRED' => true,
            'DEFAULT_SET' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $fieldset,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_TEXT', [
            'SCROLLS' => '',
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
            'RAW' => false,
            'DISPLAY_ONLY' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_CODENAME', [
            'MAXLENGTH' => placeholder_number(),
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
            'EXTRA_CHARS' => [],
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_TICK', [
            'CHECKED' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'READ_ONLY' => false,
            'DISABLED' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD_SPACER', [
            'SECTION_HIDDEN' => false,
            'TITLE' => lorem_phrase(),
            'HELP' => lorem_paragraph(),
        ]));
        $list = new Tempcode();
        foreach (placeholder_array() as $key => $value) {
            $list->attach(do_lorem_template('FORM_SCREEN_INPUT_LIST_ENTRY', [
                'SELECTED' => false,
                'DISABLED' => false,
                'CLASS' => '',
                'NAME' => placeholder_random_id(),
                'TEXT' => lorem_phrase(),
            ]));
        }
        $list = do_lorem_template('FORM_SCREEN_INPUT_LIST_GROUP', [
            'ENTRIES' => $list,
            'TITLE' => lorem_phrase(),
        ]);
        $radios = new Tempcode();
        $name = placeholder_random_id();
        $radios->attach(do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST_ENTRY', [
            'DESCRIPTION' => lorem_sentence(),
            'CHECKED' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'VALUE' => '1',
            'TEXT' => lorem_phrase(),
            'DISABLED' => false,
        ]));
        $radios->attach(do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST_ENTRY', [
            'DESCRIPTION' => lorem_sentence(),
            'CHECKED' => 'false',
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'VALUE' => '2',
            'TEXT' => lorem_phrase(),
            'DISABLED' => false,
        ]));
        $radios->attach(do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST_ENTRY', [
            'DESCRIPTION' => lorem_sentence(),
            'CHECKED' => 'false',
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'VALUE' => '3',
            'TEXT' => lorem_phrase(),
            'DISABLED' => false,
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_ALL_AND_NOT', [
            'TABINDEX' => placeholder_number(),
            'BASE' => $name,
            'RADIOS' => $radios,
            'LIST' => $list,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name . '_list',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_AUTHOR', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
        ]);
        $_description = new Tempcode();
        $_description->attach(lorem_phrase());
        $_description->attach(do_lorem_template('FORM_DESCRIP_SEP'));
        $extra = do_lorem_template('HYPERLINK_OVERLAY', [
            'URL' => placeholder_url(),
            'TITLE' => lorem_phrase(),
            'CAPTION' => lorem_phrase(),
        ]);
        $_description->attach($extra);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => $_description,
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $fields->attach(do_lorem_template('FORM_SCREEN_INPUT_COLOUR', [
            'RAW_FIELD' => false,
            'REQUIRED' => '',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence(),
            'TABINDEX' => placeholder_number(),
            '_REQUIRED' => '',
            'NAME' => placeholder_random_id(),
            'DEFAULT' => '',
        ]));

        $start_year = intval(date('Y')) - 1;
        $start_month = intval(date('m'));
        $start_day = intval(date('d'));
        $start_hour = intval(date('H'));
        $start_minute = intval(date('i'));
        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_TIME', [
            'REQUIRED' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'READ_ONLY' => false,

            'MINUTE' => placeholder_number(),
            'HOUR' => placeholder_number(),
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_DATE', [
            'REQUIRED' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'TYPE' => 'date',
            'READ_ONLY' => false,

            'YEAR' => placeholder_number(),
            'MONTH' => placeholder_number(),
            'DAY' => placeholder_number(),
            'MINUTE' => placeholder_number(),
            'HOUR' => placeholder_number(),

            'MIN_DATE_DAY' => '',
            'MIN_DATE_MONTH' => '',
            'MIN_DATE_YEAR' => '',
            'MAX_DATE_DAY' => '',
            'MAX_DATE_MONTH' => '',
            'MAX_DATE_YEAR' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_DATE_COMPONENTS', [
            'REQUIRED' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'WANT_YEAR' => true,
            'START_YEAR' => strval(intval(date('Y')) - 1),
            'END_YEAR' => strval(intval(date('Y')) + 1),
            'YEAR' => date('Y'),
            'WANT_MONTH' => true,
            'MONTH' => '1',
            'WANT_DAY' => true,
            'DAY' => '1',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_EMAIL', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_FLOAT', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $fields->attach(do_lorem_template('FORM_SCREEN_INPUT_HUGE', [
            'RAW' => true,
            'SCROLLS' => '',
            'DESCRIPTION_SIDE' => lorem_sentence(),
            'REQUIRED' => '',
            '_REQUIRED' => '',
            'TABINDEX' => placeholder_number(),
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence(),
            'NAME' => placeholder_random_id(),
            'DEFAULT' => '',
            'ROWS' => '20',
        ]));

        $_comcode = do_lorem_template('COMCODE_MESSAGE', [
            'NAME' => placeholder_random_id(),
            'W' => '',
            'URL' => placeholder_url(),
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_INPUT_HUGE_COMCODE', [
            'SCROLLS' => '',
            'DESCRIPTION_SIDE' => lorem_sentence(),
            'REQUIRED' => 'wysiwyg',
            '_REQUIRED' => 'wysiwyg',
            'TABINDEX' => placeholder_number(),
            'COMCODE' => $_comcode,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_phrase(),
            'NAME' => placeholder_random_id(),
            'DEFAULT' => lorem_word(),
            'DEFAULT_PARSED' => lorem_word(),
            'ROWS' => '20',
        ]));

        return lorem_globalise(do_lorem_template('FORM', [
            'GET' => false,
            'SKIP_WEBSTANDARDS' => true,
            'HIDDEN' => '',
            'URL' => placeholder_url(),
            'FIELDS' => $fields,
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'SUBMIT_BUTTON_CLASS' => null,
            'TEXT' => lorem_sentence_html(),
            'JS_FUNCTION_CALLS' => [],
            'SKIP_REQUIRED' => null,
            'SECONDARY_FORM' => false,
            'TABINDEX' => placeholder_number(),
            'SUPPORT_AUTOSAVE' => null,
            'ANALYTIC_EVENT_CATEGORY' => null,
            'MODSECURITY_WORKAROUND' => null,
            'NO_SIZING' => null,
            'PREVIEW' => null,
            'TITLE' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_screen_2() : object
    {
        require_javascript('plupload');

        require_lang('comcode');
        require_lang('dates');

        $fields = new Tempcode();
        $list = new Tempcode();

        $entries = new Tempcode();
        foreach (placeholder_array() as $val) {
            $entries->attach(do_lorem_template('FORM_SCREEN_INPUT_LIST_ENTRY', [
                'SELECTED' => '',
                'DISABLED' => '',
                'CLASS' => '',
                'NAME' => placeholder_random_id(),
                'TEXT' => lorem_phrase(),
            ]));
        }
        $fields->attach(do_lorem_template('FORM_SCREEN_INPUT_HUGE_LIST', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence(),
            'NAME' => placeholder_random_id(),
            'CONTENT' => $entries,
            'INLINE_LIST' => false,
        ]));

        $name_width = placeholder_random_id();
        $name_height = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_DIMENSIONS', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME_WIDTH' => $name_width,
            'DEFAULT_WIDTH' => '',
            'NAME_HEIGHT' => $name_height,
            'DEFAULT_HEIGHT' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name_width,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_INTEGER', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
            'READ_ONLY' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $placeholder_line_multi = [];
        for ($i = 0; $i < 3; $i++) {
            $placeholder_line_multi [] = [
                'NAME' => 'LOREM',
                'I' => strval($i),
                'REQUIRED' => '-required',
                'READONLY' => $i === 1
            ];
        }

        $input = new Tempcode();
        $name = placeholder_random_id();
        $input->attach(do_lorem_template('FORM_SCREEN_INPUT_LINE_MULTI', [
            '_GUID' => 'e2da34b7564cebfd83da2859e4abd020',
            'CLASS' => '',
            'MAXLENGTH' => '10',
            'PRETTY_NAME' => lorem_word(),
            'TABINDEX' => placeholder_number(),
            'NAME_STUB' => placeholder_random_id(),
            'DEFAULT_ARRAY' => $placeholder_line_multi,
            'PATTERN' => null,
            'NUM_REQUIRED' => strval(placeholder_number())
        ]));
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $input = new Tempcode();
        $name = placeholder_random_id();
        foreach (placeholder_array(2) as $k => $v) {
            $input->attach(do_lorem_template('FORM_SCREEN_INPUT_TEXT_MULTI', [
                'PRETTY_NAME' => lorem_word(),
                'TABINDEX' => placeholder_number(),
                'NAME_STUB' => $name,
                'I' => strval($k),
                'REQUIRED' => '-required',
                'DEFAULT' => ($k == 0) ? lorem_phrase() : '',
            ]));
        }
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => $name . '0',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $hidden = do_lorem_template('FORM_SCREEN_INPUT_HIDDEN_2', [
            'NAME' => $name,
            'VALUE' => lorem_phrase(),
        ]);

        foreach (placeholder_array() as $key => $value) {
            $list->attach(do_lorem_template('FORM_SCREEN_INPUT_LIST_ENTRY', [
                'SELECTED' => '',
                'DISABLED' => '',
                'CLASS' => '',
                'NAME' => $value,
                'TEXT' => lorem_phrase(),
            ]));
        }

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_COMBO', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'CONTENT' => $list,
            'DEFAULT' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_LIST', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'CONTENT' => $list,
            'INLINE_LIST' => false,
            'SIZE' => '9',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_MULTI_LIST', [
            'TABINDEX' => placeholder_number(),
            'SIZE' => '5',
            'CUSTOM_ACCEPT_MULTIPLE' => false,
            'CUSTOM_NAME' => placeholder_random_id(),
            'CUSTOM_VALUE' => '',
            'NAME' => $name,
            'CONTENT' => $list,
            'INLINE' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_PASSWORD', [
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'VALUE' => '',
            'PASSWORD_STRENGTH' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $radios = do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST_ENTRY', [
            'DESCRIPTION' => lorem_phrase(),
            'CHECKED' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'VALUE' => lorem_word(),
            'TEXT' => lorem_phrase(),
            'DISABLED' => false,
        ]);
        $radios->attach(do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST_COMBO_ENTRY', [
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'VALUE' => '',
        ]));
        $input = do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST', [
            'CONTENT' => $radios,
            'NAME' => $name,
            'REQUIRED' => true,
            'IMAGES' => null,
            'LINEAR' => false,
            'CODE' => '',
            'TABINDEX' => placeholder_number(),
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => '',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_TREE_LIST', [
            'NICE_LABEL' => '',
            'USE_SERVER_ID' => true,
            'TABINDEX' => placeholder_number(),
            'NAME' => $name,
            'REQUIRED' => '',
            '_REQUIRED' => false,
            'DEFAULT' => '',
            'HOOK' => 'choose_theme_files',
            'ROOT_ID' => '',
            'OPTIONS' => json_encode(['theme' => 'default']),
            'DESCRIPTION' => lorem_sentence_html(),
            'MULTI_SELECT' => false,
            'CONTENT_TYPE' => '',
            'END_OF_FORM' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => '',
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_UPLOAD', [
            'FILTER' => '',
            'EXISTING_URL' => placeholder_url(),
            'IS_IMAGE' => '',
            'PLUPLOAD' => false,
            'EDIT' => '',
            'TABINDEX' => placeholder_number(),
            'REQUIRED' => '',
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_UPLOAD_MULTI', [
            'TABINDEX' => placeholder_number(),
            'FILTER' => '',
            'PLUPLOAD' => false,
            'REQUIRED' => '',
            'NAME' => $name,
            'I' => $name,
            'NAME_STUB' => lorem_word(),
            'EDIT' => [],
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_USERNAME', [
            'TABINDEX' => placeholder_number(),
            'NEEDS_MATCH' => true,
            'REQUIRED' => '',
            'NAME' => $name,
            'DEFAULT' => '',
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => false,
            'NAME' => $name,
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $name = placeholder_random_id();
        $input = new Tempcode();
        foreach (placeholder_array() as $k => $v) {
            $input->attach(do_lorem_template('FORM_SCREEN_INPUT_USERNAME_MULTI', [
                'PRETTY_NAME' => lorem_word(),
                'TABINDEX' => placeholder_number(),
                'NEEDS_MATCH' => true,
                'NAME_STUB' => $name,
                'I' => strval($k),
                'REQUIRED' => '',
                'DEFAULT' => '',
            ]));
        }
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => placeholder_random_id(),
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        foreach ([true, false] as $simple_style) {
            $name = placeholder_random_id();
            $input = new Tempcode();
            $options = [
                [
                    'option1',
                    'opt1',
                    true,
                    '',
                ],
                [
                    'option2',
                    'opt2',
                    true,
                    '',
                ],
                [
                    'option3',
                    'opt3',
                    true,
                    '',
                ]
            ];
            if (count($options[0]) != 3) {
                $options = [
                    [
                        $options,
                        null,
                        new Tempcode(),
                    ],
                ];
            }

            foreach ($options as $_option) {
                $out = [];
                foreach ($_option[0] as $option) {
                    $out[] = [
                        'DISABLED' => false,
                        'CHECKED' => false,
                        'TABINDEX' => placeholder_number(),
                        'NAME' => placeholder_random_id(),
                        'PRETTY_NAME' => lorem_word(),
                        'DESCRIPTION' => lorem_sentence(),
                    ];
                }

                $input->attach(do_lorem_template('FORM_SCREEN_INPUT_VARIOUS_TICKS', [
                    'SECTION_TITLE' => $_option[2],
                    'EXPANDED' => $_option[1],
                    'SIMPLE_STYLE' => $simple_style,
                    'SIBLINGS' => '3',
                    'CUSTOM_ACCEPT_MULTIPLE' => false,
                    'CUSTOM_NAME' => placeholder_random_id(),
                    'CUSTOM_VALUE' => '',
                    'OUT' => $out,
                ]));
            }
            $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
                'REQUIRED' => true,
                'SKIP_LABEL' => true,
                'NAME' => $name,
                'PRETTY_NAME' => lorem_word(),
                'DESCRIPTION' => lorem_sentence_html(),
                'DESCRIPTION_SIDE' => '',
                'INPUT' => $input,
                'COMCODE' => '',
            ]));
        }

        $hidden->attach(do_lorem_template('FORM_SCREEN_INPUT_HIDDEN', [
            'NAME' => placeholder_random_id(),
            'VALUE' => lorem_word(),
        ]));

        return lorem_globalise(do_lorem_template('FORM_SCREEN', [
            'GET' => false,
            'SKIP_WEBSTANDARDS' => null,
            'HIDDEN' => $hidden,
            'TITLE' => lorem_screen_title(),
            'URL' => placeholder_url(),
            'FIELDS' => $fields,
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'TEXT' => lorem_sentence_html(),
            'SUPPORT_AUTOSAVE' => null,
            'JS_FUNCTION_CALLS' => [],
            'MODSECURITY_WORKAROUND' => false,
            'ANALYTIC_EVENT_CATEGORY' => null,
            'PREVIEW' => null,
            'STAFF_HELP_URL' => null,
            'PING_URL' => null,
            'CANCEL_URL' => null,
            'EXTRA_BUTTONS' => null,
            'THEME' => null,
            'SEPARATE_PREVIEW' => null,
            'BACK_URL' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__posting_screen() : object
    {
        return lorem_globalise(do_lorem_template('POSTING_SCREEN', [
            'POSTING_FORM' => placeholder_form(),
            'TITLE' => lorem_screen_title(),
            'TEXT' => lorem_sentence_html(),
            'PING_URL' => null,
            'PREVIEW' => null,
            'SEPARATE_PREVIEW' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_group() : object
    {
        return lorem_globalise(do_lorem_template('FORM_GROUP', [
            'FIELDS' => placeholder_fields(),
            'HIDDEN' => '',
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_grouped() : object
    {
        return lorem_globalise(do_lorem_template('FORM_GROUPED', [
            'TEXT' => lorem_sentence_html(),
            'FIELD_GROUPS' => lorem_phrase(),
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'URL' => placeholder_url(),
            'SUBMIT_BUTTON_CLASS' => null,
            'SUPPORT_AUTOSAVE' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_field_set_grouper() : object
    {
        return lorem_globalise(do_lorem_template('FORM_FIELD_SET_GROUPER', [
            'NAME' => lorem_word(),
            'ID' => placeholder_codename(),
            'FIELDS' => placeholder_fields(),
            'VISIBLE' => true,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__preview_script_spelling() : object
    {
        $misspellings = [
            [
                'WORD' => 'batt',
                'CORRECTIONS' => 'bat',
            ],
            [
                'WORD' => 'caat',
                'CORRECTIONS' => 'cat',
            ],
        ];

        return lorem_globalise(do_lorem_template('PREVIEW_SCRIPT_SPELLING', [
            'FIELD' => lorem_phrase(),
            'MISSPELLINGS' => $misspellings,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__preview_script_keyword_density() : object
    {
        $keywords = [];
        foreach (placeholder_array() as $k => $v) {
            $keywords[] = [
                'KEYWORD' => $v,
                'DENSITY' => placeholder_number(),
                'IDEAL_DENSITY' => placeholder_number(),
            ];
        }

        return lorem_globalise(do_lorem_template('PREVIEW_SCRIPT_KEYWORD_DENSITY', [
            'FIELD' => lorem_phrase(),
            'KEYWORDS' => $keywords,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__preview_script_code() : object
    {
        return lorem_globalise(do_lorem_template('PREVIEW_SCRIPT_CODE', [
            'NEW_POST_VALUE_HTML' => lorem_phrase(),
            'NEW_POST_VALUE' => lorem_phrase(),
            'ATTACHMENT_FIELD' => placeholder_codename(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_screen() : object
    {
        require_lang('comcode');
        require_lang('dates');

        $input = new Tempcode();
        $fields = new Tempcode();
        foreach (placeholder_array(1) as $key => $val) {
            $input = do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST_ENTRY_PICTURE', [
                'CHECKED' => true,
                'NAME' => 'path',
                'VALUE' => $val,
                'URL' => placeholder_image_url(),
                'PRETTY' => strval($key),
            ]);
            $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
                'REQUIRED' => true,
                'SKIP_LABEL' => false,
                'NAME' => 'path_' . $val,
                'PRETTY_NAME' => lorem_word(),
                'DESCRIPTION' => lorem_sentence_html(),
                'DESCRIPTION_SIDE' => '',
                'INPUT' => $input,
                'COMCODE' => '',
            ]));
        }
        $content = new Tempcode();
        $i = 0;
        foreach (placeholder_array() as $k => $v) {
            $cells = new Tempcode();
            foreach (placeholder_array() as $_k => $_v) {
                $temp = do_lorem_template('FORM_SCREEN_INPUT_THEME_IMAGE_ENTRY', [
                    'CHECKED' => true,
                    'PRETTY' => lorem_phrase(),
                    'NAME' => placeholder_random_id(),
                    'CODE' => $_v,
                    'URL' => placeholder_image_url(),
                    'VECTOR' => false,
                ]);
                $cells->attach($temp);
            }

            $input = do_lorem_template('FORM_SCREEN_INPUT_THEME_IMAGE_CATEGORY', [
                'DISPLAY' => 'block',
                'CATEGORY' => $cells,
                'FIELD_NAME' => placeholder_codename(),
                'CATEGORY_NAME' => lorem_word(),
            ]);
            $content->attach($input);
        }

        $name = placeholder_random_id();
        $input = do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST', [
            'NAME' => $name,
            'CODE' => '',
            'TABINDEX' => placeholder_number(),
            'CONTENT' => $content,
            'REQUIRED' => true,
            'IMAGES' => true,
            'LINEAR' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => '',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $fields->attach(do_lorem_template('POSTING_FIELD', [
            'REQUIRED' => true,
            'DESCRIPTION' => lorem_phrase(),
            'HIDDEN_FIELDS' => '',
            'PRETTY_NAME' => lorem_word(),
            'NAME' => placeholder_random_id(),
            'TABINDEX_PF' => placeholder_number(),
            'COMCODE_EDITOR' => placeholder_blank(),
            'COMCODE_EDITOR_SMALL' => placeholder_blank(),
            'CLASS' => lorem_word(),
            'COMCODE_URL' => placeholder_url(),
            'EMOTICON_CHOOSER' => placeholder_emoticon_chooser(),
            'POST' => placeholder_blank(),
            'DEFAULT_PARSED' => placeholder_blank(),
            'ATTACHMENTS' => placeholder_blank(),
        ]));

        return lorem_globalise(do_lorem_template('FORM_SCREEN', [
            'GET' => false,
            'HIDDEN' => '',
            'TITLE' => lorem_screen_title(),
            'URL' => placeholder_url(),
            'FIELDS' => $fields,
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'TEXT' => lorem_sentence_html(),
            'SKIP_WEBSTANDARDS' => null,
            'SUPPORT_AUTOSAVE' => null,
            'JS_FUNCTION_CALLS' => [],
            'MODSECURITY_WORKAROUND' => false,
            'ANALYTIC_EVENT_CATEGORY' => null,
            'PREVIEW' => null,
            'STAFF_HELP_URL' => null,
            'PING_URL' => null,
            'CANCEL_URL' => null,
            'EXTRA_BUTTONS' => null,
            'THEME' => null,
            'SEPARATE_PREVIEW' => null,
            'BACK_URL' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__wysiwyg_load() : object
    {
        return lorem_globalise(do_lorem_template('WYSIWYG_LOAD', []), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__posting_form() : object
    {
        require_lang('comcode');
        require_lang('dates');

        $buttons = new Tempcode();
        $_buttons = [
            'thumb',
            'url',
            'page',
            'code',
            'quote',
            'hide',
            'box',
            'block',
            'list',
            'html'
        ];
        foreach ($_buttons as $button) {
            $buttons->attach(do_lorem_template('COMCODE_EDITOR_BUTTON', [
                'DIVIDER' => true,
                'FIELD_NAME' => lorem_word(),
                'LABEL' => lorem_word(),
                'TITLE' => lorem_phrase(),
                'B' => $button,
                'IS_POSTING_FIELD' => true,
            ]));
        }

        $micro_buttons = new Tempcode();
        $_micro_buttons = [
            [
                't' => 'b',
            ],
            [
                't' => 'i',
            ]
        ];

        foreach ($_micro_buttons as $button) {
            $micro_buttons->attach(do_lorem_template('COMCODE_EDITOR_MICRO_BUTTON', [
                'FIELD_NAME' => lorem_word(),
                'TITLE' => lorem_phrase(),
                'B' => $button['t'],
                'IS_POSTING_FIELD' => true,
            ]));
        }

        $comcode = do_lorem_template('COMCODE_EDITOR', [
            'POSTING_FIELD' => lorem_word(),
            'BUTTONS' => $buttons,
            'MICRO_BUTTONS' => $micro_buttons,
            'IS_POSTING_FIELD' => true,
        ]);

        return lorem_globalise(do_lorem_template('POSTING_FORM', [
            'TABINDEX_PF' => placeholder_number(),
            'PREVIEW' => lorem_phrase(),
            'COMCODE_EDITOR' => $comcode,
            'COMCODE_EDITOR_SMALL' => lorem_phrase(),
            'CLASS' => lorem_phrase(),
            'COMCODE_URL' => placeholder_url(),
            'EXTRA' => lorem_phrase(),
            'POST_COMMENT' => lorem_phrase(),
            'EMOTICON_CHOOSER' => lorem_phrase(),
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'HIDDEN_FIELDS' => '',
            'URL' => placeholder_url(),
            'POST' => lorem_phrase(),
            'DEFAULT_PARSED' => lorem_phrase(),
            'ATTACHMENTS' => lorem_phrase(),
            'SPECIALISATION' => placeholder_fields(),
            'SPECIALISATION2' => placeholder_fields(),
            'REQUIRED' => true,
            'SUPPORT_AUTOSAVE' => false,
            'DESCRIPTION' => lorem_paragraph(),
            'JS_FUNCTION_CALLS' => [],
            'SPECIALISATION2_HIDDEN' => null,
            'CANCEL_URL' => null,
            'MODSECURITY_WORKAROUND' => false,
            'COMCODE_PAGE_HINTS' => null,
            'STAFF_HELP_URL' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__form_descrip_sep() : object
    {
        return lorem_globalise(do_lorem_template('FORM_DESCRIP_SEP', []), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__form_screen_input_permission() : object
    {
        require_lang('permissions');

        $permission_rows = new Tempcode();
        foreach (placeholder_array() as $k => $v) {
            $overrides = new Tempcode();
            $all_global = true;
            foreach (placeholder_array() as $_k => $_v) {
                $overrides->attach(do_lorem_template('FORM_SCREEN_INPUT_PERMISSION_OVERRIDE', [
                    'FORCE_PRESETS' => false,
                    'GROUP_NAME' => lorem_phrase(),
                    'VIEW_ACCESS' => '',
                    'TABINDEX' => placeholder_number(),
                    'GROUP_ID' => placeholder_random_id(),
                    'PRIVILEGE' => strval($_k),
                    'ALL_GLOBAL' => true,
                    'TITLE' => lorem_phrase(),
                    'DEFAULT_ACCESS' => placeholder_number(),
                    'CODE' => '-1',
                ]));
            }

            $permission_rows->attach(do_lorem_template('FORM_SCREEN_INPUT_PERMISSION', [
                'FORCE_PRESETS' => false,
                'GROUP_NAME' => lorem_phrase(),
                'OVERRIDES' => $overrides->evaluate()/*Conserve memory*/,
                'ALL_GLOBAL' => true,
                'VIEW_ACCESS' => '',
                'TABINDEX' => placeholder_number(),
                'GROUP_ID' => placeholder_random_id(),
                'PINTERFACE_VIEW' => '',
            ]));
        }

        $permission_rows->attach(do_lorem_template('FORM_SCREEN_INPUT_PERMISSION_ADMIN', [
            'PINTERFACE_VIEW' => lorem_word(),
            'GROUP_ID' => placeholder_random_id(),
            'GROUP_NAME' => lorem_phrase(),
            'FORCE_PRESETS' => false,
            'OVERRIDES' => [['TITLE' => lorem_phrase()]],
        ]));

        $field = do_lorem_template('FORM_SCREEN_INPUT_PERMISSION_MATRIX', [
            'SERVER_ID' => placeholder_codename(),
            'COLOR' => lorem_word(),
            'OVERRIDES' => [['TITLE' => lorem_phrase()]],
            'PERMISSION_ROWS' => $permission_rows,
        ]);

        return lorem_globalise(do_lorem_template('FORM_GROUPED', [
            'URL' => '#',
            'FIELD_GROUPS' => $field,
            'TEXT' => '',
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word_2(),
            'SUBMIT_BUTTON_CLASS' => null,
            'SUPPORT_AUTOSAVE' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__form_screen_input_permission_outer() : object
    {
        require_lang('permissions');

        $fields = new Tempcode();

        $name = placeholder_codename();
        $input = do_lorem_template('FORM_SCREEN_INPUT_RADIO_LIST', [
            'NAME' => $name,
            'CODE' => '',
            'TABINDEX' => placeholder_number(),
            'CONTENT' => lorem_phrase(),
            'REQUIRED' => true,
            'IMAGES' => null,
            'LINEAR' => false,
        ]);
        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => '',
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $input,
            'COMCODE' => '',
        ]));

        $fields->attach(do_lorem_template('FORM_SCREEN_INPUT_PERMISSION_MATRIX_OUTER', [
            'INNER' => lorem_word(),
        ]));

        return lorem_globalise(do_lorem_template('FORM', [
            'URL' => '#',
            'FIELDS' => $fields,
            'HIDDEN' => '',
            'TEXT' => '',
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word_2(),
            'SUBMIT_BUTTON_CLASS' => null,
            'JS_FUNCTION_CALLS' => null,
            'SKIP_REQUIRED' => null,
            'SECONDARY_FORM' => null,
            'TABINDEX' => null,
            'SUPPORT_AUTOSAVE' => null,
            'ANALYTIC_EVENT_CATEGORY' => null,
            'MODSECURITY_WORKAROUND' => null,
            'SKIP_WEBSTANDARDS' => null,
            'GET' => null,
            'NO_SIZING' => null,
            'PREVIEW' => null,
            'TITLE' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__preview_script() : object
    {
        return lorem_globalise(do_lorem_template('PREVIEW_SCRIPT', [
            'OUTPUT' => lorem_phrase(),
            'HIDDEN' => '',
            'WEBSTANDARDS' => lorem_phrase(),
            'KEYWORD_DENSITY' => lorem_phrase(),
            'SPELLING' => lorem_phrase(),
            'HEALTH_CHECK' => [lorem_phrase()],
            'HAS_DEVICE_PREVIEW_MODES' => true,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__block_helper_block_group() : object
    {
        require_lang('blocks');
        require_lang('comcode');

        $links = new Tempcode();
        foreach (placeholder_array() as $v) {
            $links->attach(do_lorem_template('BLOCK_HELPER_BLOCK_CHOICE', [
                'USAGE' => placeholder_array(),
                'DESCRIPTION' => lorem_paragraph_html(),
                'URL' => placeholder_url(),
                'LINK_CAPTION' => lorem_phrase(),
            ]));
        }
        return lorem_globalise(do_lorem_template('BLOCK_HELPER_BLOCK_GROUP', [
            'IMG' => 'themes/default/images/icons/admin/add.svg',
            'TITLE' => lorem_phrase(),
            'LINKS' => $links,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__block_helper_start() : object
    {
        require_lang('comcode');
        return lorem_globalise(do_lorem_template('BLOCK_HELPER_START', [
            'TITLE' => lorem_phrase(),
            'LINKS' => lorem_paragraph_html(),
            'GET' => false,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__block_helper_done() : object
    {
        return lorem_globalise(do_lorem_template('BLOCK_HELPER_DONE', [
            'TITLE' => lorem_phrase(),
            'FIELD_NAME' => placeholder_codename(),
            'BLOCK' => lorem_phrase(),
            'TAG_CONTENTS' => '',
            'SAVE_TO_ID' => '',
            'DELETE' => false,
            'COMCODE' => lorem_phrase(),
            'COMCODE_SEMIHTML' => lorem_phrase(),
            'PREFIX' => null,
        ]), null, '', true);
    }
}
