<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    newsletter
 */

/**
 * Hook class.
 */
class Hook_addon_registry_newsletter
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the version of the software this addon is for (used in generating the TAR filename).
     *
     * @return float Version number
     */
    public function get_version() : float
    {
        return cms_version_number();
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'New Features';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'Support for users to join newsletters, and for the staff to send out newsletters to subscribers, and to specific usergroups.';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [
            'tut_newsletter',
        ];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => [],
            'recommends' => ['System scheduler'],
            'conflicts_with' => [],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/menu/site_meta/newsletters.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'adminzone/pages/modules/admin_newsletter.php',
            'data/incoming_bounced_email.php',
            'lang/EN/newsletter.ini',
            'site/pages/modules/newsletter.php',
            'sources/blocks/main_newsletter_signup.php',
            'sources/hooks/blocks/main_staff_checklist/newsletter.php',
            'sources/hooks/blocks/main_staff_checklist/newsletter_paused.php',
            'sources/hooks/modules/admin_import/newsletter_subscribers.php',
            'sources/hooks/modules/admin_import_types/newsletter.php',
            'sources/hooks/modules/admin_newsletter/.htaccess',
            'sources/hooks/modules/admin_newsletter/index.html',
            'sources/hooks/modules/admin_setupwizard/newsletter.php',
            'sources/hooks/modules/admin_stats/newsletter_subscriptions.php',
            'sources/hooks/systems/actionlog/newsletter.php',
            'sources/hooks/systems/addon_registry/newsletter.php',
            'sources/hooks/systems/block_ui_renderers/newsletters.php',
            'sources/hooks/systems/commandr_fs/newsletter_subscribers.php',
            'sources/hooks/systems/commandr_fs/newsletters.php',
            'sources/hooks/systems/commandr_fs/periodic_newsletters.php',
            'sources/hooks/systems/commandr_scheduled/send_newsletter.php',
            'sources/hooks/systems/config/dual_format_newsletters.php',
            'sources/hooks/systems/config/mails_per_send.php',
            'sources/hooks/systems/config/max_newsletter_whatsnew.php',
            'sources/hooks/systems/config/minutes_between_sends.php',
            'sources/hooks/systems/config/newsletter_allow_ext_images.php',
            'sources/hooks/systems/config/newsletter_auto_pause.php',
            'sources/hooks/systems/config/newsletter_enveloper_override.php',
            'sources/hooks/systems/config/newsletter_paused.php',
            'sources/hooks/systems/config/newsletter_smtp_from_address.php',
            'sources/hooks/systems/config/newsletter_smtp_sockets_host.php',
            'sources/hooks/systems/config/newsletter_smtp_sockets_password.php',
            'sources/hooks/systems/config/newsletter_smtp_sockets_port.php',
            'sources/hooks/systems/config/newsletter_smtp_sockets_use.php',
            'sources/hooks/systems/config/newsletter_smtp_sockets_username.php',
            'sources/hooks/systems/config/newsletter_text.php',
            'sources/hooks/systems/config/newsletter_title.php',
            'sources/hooks/systems/config/newsletter_update_time.php',
            'sources/hooks/systems/config/newsletter_website_email.php',
            'sources/hooks/systems/cron/newsletter_drip_send.php',
            'sources/hooks/systems/cron/newsletter_periodic.php',
            'sources/hooks/systems/notifications/newsletter_paused.php',
            'sources/hooks/systems/page_groupings/newsletter.php',
            'sources/hooks/systems/preview/newsletter_whatsnew.php',
            'sources/hooks/systems/privacy/newsletter.php',
            'sources/hooks/systems/resource_meta_aware/newsletter.php',
            'sources/hooks/systems/resource_meta_aware/newsletter_subscriber.php',
            'sources/hooks/systems/resource_meta_aware/periodic_newsletter.php',
            'sources/hooks/systems/tasks/export_newsletter_subscribers.php',
            'sources/hooks/systems/tasks/import_newsletter_subscribers.php',
            'sources/hooks/systems/tasks/send_newsletter.php',
            'sources/newsletter.php',
            'sources/newsletter2.php',
            'sources_custom/hooks/modules/admin_newsletter/.htaccess',
            'sources_custom/hooks/modules/admin_newsletter/index.html',
            'themes/default/css/newsletter.css',
            'themes/default/images/icons/menu/adminzone/tools/newsletter/index.html',
            'themes/default/images/icons/menu/adminzone/tools/newsletter/newsletter_email_bounce.svg',
            'themes/default/images/icons/menu/adminzone/tools/newsletter/newsletter_from_changes.svg',
            'themes/default/images/icons/menu/adminzone/tools/newsletter/subscribers.svg',
            'themes/default/images/icons/menu/site_meta/newsletters.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/newsletter/index.html',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/newsletter/newsletter_email_bounce.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/newsletter/newsletter_from_changes.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/tools/newsletter/subscribers.svg',
            'themes/default/images/icons_monochrome/menu/site_meta/newsletters.svg',
            'themes/default/javascript/newsletter.js',
            'themes/default/templates/BLOCK_MAIN_NEWSLETTER_SIGNUP.tpl',
            'themes/default/templates/BLOCK_MAIN_NEWSLETTER_SIGNUP_DONE.tpl',
            'themes/default/templates/NEWSLETTER_CONFIRM_WRAP.tpl',
            'themes/default/templates/NEWSLETTER_PREVIEW.tpl',
            'themes/default/templates/NEWSLETTER_STATUS_OVERVIEW.tpl',
            'themes/default/templates/NEWSLETTER_SUBSCRIBER.tpl',
            'themes/default/templates/NEWSLETTER_SUBSCRIBERS_SCREEN.tpl',
            'themes/default/templates/PERIODIC_NEWSLETTER_REMOVE_SCREEN.tpl',
            'themes/default/text/NEWSLETTER_WHATSNEW_FCOMCODE.txt',
            'themes/default/text/NEWSLETTER_WHATSNEW_RESOURCE_FCOMCODE.txt',
            'themes/default/text/NEWSLETTER_WHATSNEW_SECTION_FCOMCODE.txt',
        ];
    }

    /**
     * Get mapping between template names and the method of this class that can render a preview of them.
     *
     * @return array The mapping
     */
    public function tpl_previews() : array
    {
        return [
            'text/NEWSLETTER_WHATSNEW_FCOMCODE.txt' => 'newsletter_automated_fcomcode',
            'text/NEWSLETTER_WHATSNEW_SECTION_FCOMCODE.txt' => 'newsletter_automated_fcomcode',
            'text/NEWSLETTER_WHATSNEW_RESOURCE_FCOMCODE.txt' => 'newsletter_automated_fcomcode',
            'templates/NEWSLETTER_SUBSCRIBER.tpl' => 'administrative__newsletter_subscribers_screen',
            'templates/NEWSLETTER_SUBSCRIBERS_SCREEN.tpl' => 'administrative__newsletter_subscribers_screen',
            'text/NEWSLETTER_DEFAULT_FCOMCODE.txt' => 'newsletter_default',
            'templates/NEWSLETTER_CONFIRM_WRAP.tpl' => 'administrative__newsletter_confirm_wrap',
            'templates/NEWSLETTER_PREVIEW.tpl' => 'administrative__newsletter_confirm_wrap',
            'templates/BLOCK_MAIN_NEWSLETTER_SIGNUP_DONE.tpl' => 'block_main_newsletter_signup_done',
            'templates/BLOCK_MAIN_NEWSLETTER_SIGNUP.tpl' => 'block_main_newsletter_signup',
            'templates/PERIODIC_NEWSLETTER_REMOVE_SCREEN.tpl' => 'periodic_newsletter_remove_screen',
            'templates/NEWSLETTER_STATUS_OVERVIEW.tpl' => 'newsletter_status_overview',
        ];
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__newsletter_automated_fcomcode() : object
    {
        $automatic = [];
        foreach (placeholder_array() as $k => $v) {
            $_content = do_lorem_template('NEWSLETTER_WHATSNEW_RESOURCE_FCOMCODE', [
                'MEMBER_ID' => placeholder_numeric_id(),
                'URL' => placeholder_url(),
                'NAME' => lorem_word(),
                'DESCRIPTION' => lorem_paragraph(),
                'IMAGE_URL' => placeholder_image_url(),
                'CONTENT_TYPE' => lorem_word(),
                'CONTENT_ID' => placeholder_codename(),
                'CATALOGUE' => null,
            ], null, false, null, '.txt', 'text');

            $tmp = do_lorem_template('NEWSLETTER_WHATSNEW_SECTION_FCOMCODE', [
                'I' => '1',
                'TITLE' => lorem_phrase(),
                'CONTENT' => $_content,
                'IMAGE_URL' => placeholder_image_url(),
            ], null, false, null, '.txt', 'text');
            $automatic[] = $tmp->evaluate();
        }

        $content = '';
        foreach ($automatic as $tp) {
            $content .= $tp;
        }

        return lorem_globalise(do_lorem_template('NEWSLETTER_WHATSNEW_FCOMCODE', [
            'CONTENT' => $content,
        ], null, false, null, '.txt', 'text'), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__newsletter_subscribers_screen() : object
    {
        $subscribers_table_rows = new Tempcode();
        foreach (placeholder_array() as $k => $v) {
            $subscribers_table_rows->attach(do_lorem_template('NEWSLETTER_SUBSCRIBER', [
                'EMAIL_ADDRESS' => lorem_word(),
                'FORENAME' => lorem_word(),
                'SURNAME' => lorem_word(),
                'NAME' => lorem_word(),
                'SEND_ID' => lorem_word(),
                'HASH' => lorem_word(),
            ]));
        }

        return lorem_globalise(do_lorem_template('NEWSLETTER_SUBSCRIBERS_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'SUBSCRIBERS_TABLE_ROWS' => $subscribers_table_rows,
            'PAGINATION' => '',
            'DOMAINS' => placeholder_array(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__newsletter_default() : object
    {
        return lorem_globalise(do_lorem_template('NEWSLETTER_DEFAULT_FCOMCODE', [
            'CONTENT' => lorem_phrase(),
            'LANG' => fallback_lang(),
            'SUBJECT' => lorem_phrase(),
        ], null, false, null, '.txt', 'text'), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__newsletter_confirm_wrap() : object
    {
        $preview = do_lorem_template('NEWSLETTER_CONFIRM_WRAP', [
            'TEXT_PREVIEW' => lorem_sentence(),
            'HTML_PREVIEW' => lorem_phrase(),
            'SUBJECT' => lorem_phrase(),
            'SPAM_REPORT' => lorem_paragraph(),
            'SPAM_SCORE' => placeholder_number(),
        ]);

        return lorem_globalise(do_lorem_template('CONFIRM_SCREEN', [
            'URL' => placeholder_url(),
            'BACK_URL' => placeholder_url(),
            'PREVIEW' => $preview,
            'FIELDS' => new Tempcode(),
            'TITLE' => lorem_screen_title(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_newsletter_signup() : object
    {
        return lorem_globalise(do_lorem_template('BLOCK_MAIN_NEWSLETTER_SIGNUP', [
            'BLOCK_ID' => lorem_word(),
            'BLOCK_PARAMS' => '',
            'URL' => placeholder_url(),
            'NEWSLETTER_TITLE' => lorem_phrase(),
            'NID' => placeholder_numeric_id(),
            'BUTTON_ONLY' => false,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__block_main_newsletter_signup_done() : object
    {
        return lorem_globalise(do_lorem_template('BLOCK_MAIN_NEWSLETTER_SIGNUP_DONE', [
            'BLOCK_ID' => lorem_word(),
            'PASSWORD' => lorem_phrase(),
            'NEWSLETTER_TITLE' => lorem_phrase(),
            'PATH_EXISTS' => true,
            'BLOCK_PARAMS' => '',
            'NID' => placeholder_numeric_id(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__periodic_newsletter_remove_screen() : object
    {
        return lorem_globalise(do_lorem_template('PERIODIC_NEWSLETTER_REMOVE_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'URL' => placeholder_url(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__newsletter_status_overview() : object
    {
        return lorem_globalise(do_lorem_template('NEWSLETTER_STATUS_OVERVIEW', [
            'UPDATE_URL' => placeholder_url(),
            'NUM_IN_QUEUE' => placeholder_number(),
            '_NUM_IN_QUEUE' => placeholder_number(),
            'ETA' => placeholder_date(),
            '_ETA' => placeholder_date_raw(),
            'PAUSED' => true,
            'QUEUE_URL' => placeholder_url(),
        ]), null, '', true);
    }
}
