<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    stats
 */

/**
 * Hook class.
 */
class Hook_addon_registry_stats
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the version of the software this addon is for (used in generating the TAR filename).
     *
     * @return float Version number
     */
    public function get_version() : float
    {
        return cms_version_number();
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'Admin Utilities';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'Show advanced graphs (analytics) and dumps of raw data relating to your website activity.';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [
            'tut_statistics',
        ];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => ['System scheduler'],
            'recommends' => [],
            'conflicts_with' => [],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/menu/adminzone/audit/statistics/statistics.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'adminzone/pages/modules/admin_stats.php',
            'data/modules/admin_stats/.htaccess',
            'data/modules/admin_stats/IP_Country.txt',
            'data/modules/admin_stats/index.html',
            'data/tracked_redirect.php',
            'lang/EN/stats.ini',
            'sources/blocks/main_staff_stats_graph.php',
            'sources/hooks/blocks/main_staff_checklist/kpis.php',
            'sources/hooks/modules/admin_import_types/stats.php',
            'sources/hooks/modules/admin_stats/.htaccess',
            'sources/hooks/modules/admin_stats/comments.php',
            'sources/hooks/modules/admin_stats/content.php',
            'sources/hooks/modules/admin_stats/emails.php',
            'sources/hooks/modules/admin_stats/events.php',
            'sources/hooks/modules/admin_stats/index.html',
            'sources/hooks/modules/admin_stats/links.php',
            'sources/hooks/modules/admin_stats/ratings.php',
            'sources/hooks/modules/admin_stats/sitemap.php',
            'sources/hooks/modules/admin_stats/users_online.php',
            'sources/hooks/modules/admin_stats/views.php',
            'sources/hooks/modules/admin_stats_redirects/.htaccess',
            'sources/hooks/modules/admin_stats_redirects/google_analytics.php',
            'sources/hooks/modules/admin_stats_redirects/index.html',
            'sources/hooks/systems/actionlog/kpis.php',
            'sources/hooks/systems/addon_registry/stats.php',
            'sources/hooks/systems/block_ui_renderers/stats.php',
            'sources/hooks/systems/cleanup/page_stats.php',
            'sources/hooks/systems/config/stats_store_time.php',
            'sources/hooks/systems/config/super_logging.php',
            'sources/hooks/systems/cron/stats_preprocess_raw_data.php',
            'sources/hooks/systems/cron/stats_purging.php',
            'sources/hooks/systems/non_active_urls/stats.php',
            'sources/hooks/systems/notifications/kpis.php',
            'sources/hooks/systems/page_groupings/stats.php',
            'sources/hooks/systems/privacy/stats.php',
            'sources/hooks/systems/realtime_rain/stats.php',
            'sources/hooks/systems/resource_meta_aware/kpi.php',
            'sources/hooks/systems/sitemap/stats_category.php',
            'sources/hooks/systems/snippets/stats_event.php',
            'sources/hooks/systems/symbols/HAS_TRACKING_CODE.php',
            'sources/hooks/systems/symbols/LOG_STATS_EVENT.php',
            'sources/hooks/systems/symbols/SET_TRACKING_CODE.php',
            'sources/hooks/systems/symbols/SPLIT_TEST.php',
            'sources/hooks/systems/symbols/STATS_TRACK_URL.php',
            'sources/hooks/systems/tasks/install_geolocation_data.php',
            'sources/stats.php',
            'sources_custom/hooks/modules/admin_stats/.htaccess',
            'sources_custom/hooks/modules/admin_stats/index.html',
            'text/search_engine_domains.txt',
            'text/social_media_domains.txt',
            'themes/default/css/stats.css',
            'themes/default/images/icons/menu/adminzone/audit/statistics/geolocate.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/index.html',
            'themes/default/images/icons/menu/adminzone/audit/statistics/load_times.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/page_views.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/statistics.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/submits.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/top_referrers.svg',
            'themes/default/images/icons/menu/adminzone/audit/statistics/users_online.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/geolocate.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/index.html',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/load_times.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/page_views.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/statistics.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/submits.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/top_referrers.svg',
            'themes/default/images/icons_monochrome/menu/adminzone/audit/statistics/users_online.svg',
            'themes/default/templates/FORM_SCREEN_INPUT_STATS_DATE_RANGE.tpl',
            'themes/default/templates/KPI_SCREEN.tpl',
            'themes/default/templates/STATS_GRAPH.tpl',
            'themes/default/templates/STATS_SCREEN.tpl',
            'themes/default/text/KPI_UPDATE_MAIL.txt',
        ];
    }

    /**
     * Get mapping between template names and the method of this class that can render a preview of them.
     *
     * @return array The mapping
     */
    public function tpl_previews() : array
    {
        return [
            'templates/STATS_SCREEN.tpl' => 'administrative__stats_screen',
            'templates/STATS_GRAPH.tpl' => 'administrative__stats_graph_screen',
            'templates/KPI_SCREEN.tpl' => 'administrative__kpi_screen',
            'templates/FORM_SCREEN_INPUT_STATS_DATE_RANGE.tpl' => 'administrative__form_screen_input_stats_date_range_screen',
            'text/KPI_UPDATE_MAIL.txt' => 'administrative__kpi_update_mail',
        ];
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__stats_screen() : object
    {
        $existing_kpis = [];
        $existing_kpis[] = [
            'KPI_ID' => placeholder_numeric_id(),
            'KPI_TITLE' => lorem_phrase(),
            'KPI_EDIT_URL' => placeholder_url(),
        ];

        $graphs = [];
        $graphs[] = [
            'GRAPH_NAME' => lorem_word(),
            'GRAPH_LABEL' => lorem_phrase(),
            'GRAPH_FORM' => placeholder_form(),
            'GRAPH_RENDERED' => lorem_chunk_html(),
            'RESULTS_TABLE' => placeholder_table(),
            'EXISTING_KPIS' => $existing_kpis,
            'KPI_ADD_URL' => placeholder_url(),
        ];

        return lorem_globalise(do_lorem_template('STATS_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'GRAPHS' => $graphs,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__stats_graph_screen() : object
    {
        $existing_kpis = [];
        $existing_kpis[] = [
            'KPI_ID' => placeholder_numeric_id(),
            'KPI_TITLE' => lorem_phrase(),
            'KPI_EDIT_URL' => placeholder_url(),
        ];

        return lorem_globalise(do_lorem_template('STATS_GRAPH', [
            'GRAPH_NAME' => lorem_word(),
            'GRAPH_LABEL' => lorem_phrase(),
            'GRAPH_FORM' => placeholder_form(),
            'GRAPH_RENDERED' => lorem_chunk_html(),
            'RESULTS_TABLE' => placeholder_table(),
            'EXISTING_KPIS' => $existing_kpis,
            'KPI_ADD_URL' => placeholder_url(),
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__kpi_screen() : object
    {
        $existing_kpis = [];
        $existing_kpis[] = [
            'KPI_ID' => placeholder_numeric_id(),
            'KPI_TITLE' => lorem_phrase(),
            'KPI_EDIT_URL' => placeholder_url(),
        ];

        $graphs = [];
        $graphs[] = [
            'GRAPH_NAME' => lorem_word(),
            'GRAPH_LABEL' => lorem_phrase(),
            'GRAPH_RENDERED' => lorem_chunk_html(),
            'GRAPH_FORM' => placeholder_form(),
            'RESULTS_TABLE' => placeholder_table(),
            'EXISTING_KPIS' => $existing_kpis,
            'KPI_ADD_URL' => placeholder_url(),
            'SPREADSHEET_URL' => placeholder_url(),
        ];

        $kpis = [];
        $kpis[] = [
            'TITLE' => lorem_phrase(),
            'CURRENT' => placeholder_number(),
            'HITS_TARGET' => true,
            'TARGET' => placeholder_number(),
            'KPI_EDIT_URL' => placeholder_url(),
            'GRAPH_NAME' => lorem_word(),
        ];

        return lorem_globalise(do_lorem_template('KPI_SCREEN', [
            'TITLE' => lorem_screen_title(),
            'GRAPHS' => $graphs,
            'KPIS' => $kpis,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__form_screen_input_stats_date_range_screen() : object
    {
        $field = do_lorem_template('FORM_SCREEN_INPUT_STATS_DATE_RANGE', [
            'TABINDEX' => placeholder_number(),
            'NAME' => lorem_word(),
            'MONTHS' => ['10' => lorem_phrase()],
            'START' => '10',
            'END' => '10',
        ]);

        $fields = new Tempcode();

        $fields->attach(do_lorem_template('FORM_SCREEN_FIELD', [
            'REQUIRED' => true,
            'SKIP_LABEL' => true,
            'NAME' => lorem_word(),
            'PRETTY_NAME' => lorem_word(),
            'DESCRIPTION' => lorem_sentence_html(),
            'DESCRIPTION_SIDE' => '',
            'INPUT' => $field,
            'COMCODE' => '',
        ]));

        return lorem_globalise(do_lorem_template('FORM', [
            'GET' => false,
            'SKIP_WEBSTANDARDS' => true,
            'HIDDEN' => '',
            'URL' => placeholder_url(),
            'FIELDS' => $fields,
            'SUBMIT_ICON' => 'buttons/proceed',
            'SUBMIT_NAME' => lorem_word(),
            'SUBMIT_BUTTON_CLASS' => null,
            'TEXT' => lorem_sentence_html(),
            'JS_FUNCTION_CALLS' => [],
            'SKIP_REQUIRED' => null,
            'SECONDARY_FORM' => false,
            'TABINDEX' => placeholder_number(),
            'SUPPORT_AUTOSAVE' => null,
            'ANALYTIC_EVENT_CATEGORY' => null,
            'MODSECURITY_WORKAROUND' => null,
            'NO_SIZING' => null,
            'PREVIEW' => null,
            'TITLE' => null,
        ]), null, '', true);
    }

    /**
     * Get a preview(s) of a (group of) template(s), as a full standalone piece of HTML in Tempcode format.
     * Uses sources/lorem.php functions to place appropriate stock-text. Should not hard-code things, as the code is intended to be declarative.
     * Assumptions: You can assume all Lang/CSS/JavaScript files in this addon have been pre-required.
     *
     * @return Tempcode Preview
     */
    public function tpl_preview__administrative__kpi_update_mail() : object
    {
        $kpis = [];
        $kpis[] = [
            'TITLE' => lorem_phrase(),
            'CURRENT' => placeholder_number(),
            'HITS_TARGET' => true,
            'TARGET' => placeholder_number(),
            'EDIT_URL' => placeholder_url(),
            'VIEW_URL' => placeholder_url(),
            'GRAPH_NAME' => lorem_phrase(),
            'USERNAME' => lorem_word(),
            'ADDED' => placeholder_date(),
            'NOTES' => lorem_phrase(),
        ];

        return lorem_globalise(do_lorem_template('KPI_UPDATE_MAIL', [
            'KPIS' => $kpis,
        ], null, false, null, '.txt', 'text'), null, '', true);
    }
}
