<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    cns_forum
 */

/**
 * Hook class.
 */
class Hook_attachments_cns_post
{
    /**
     * Run function for attachment hooks. They see if permission to an attachment of an ID relating to this content is present for a member.
     *
     * @param  ID_TEXT $id The ID
     * @param  object $db The database connector to check on
     * @param  MEMBER $member_id The member to check for
     * @return boolean Whether there is permission
     */
    public function run(string $id, object $db, int $member_id) : bool
    {
        if (!addon_installed('cns_forum')) {
            return false;
        }

        if (get_forum_type() != 'cns') {
            return false; // Shouldn't be here, but maybe it's left over somehow
        }

        require_code('cns_forums');
        require_code('cns_topics');
        $info = $GLOBALS['FORUM_DB']->query_select('f_posts', ['p_cache_forum_id', 'p_whisper_to_member', 'p_posting_member', 'p_topic_id'], ['id' => intval($id)], '', 1);
        if (!array_key_exists(0, $info)) {
            return false;
        }
        $forum_id = $info[0]['p_cache_forum_id'];
        $forum_id_parent = ($forum_id === null) ? null : $GLOBALS['FORUM_DB']->query_select_value_if_there('f_forums', 'f_parent_forum_id', ['id' => $forum_id]);
        $forum_id_parent_parent = ($forum_id_parent === null) ? null : $GLOBALS['FORUM_DB']->query_select_value_if_there('f_forums', 'f_parent_forum_id', ['id' => $forum_id_parent]);
        $poster = $info[0]['p_posting_member'];
        $whisper_to_member = $info[0]['p_whisper_to_member'];
        if (($whisper_to_member !== null) && ($poster != $member_id) && ($whisper_to_member != $member_id)) {
            return false;
        }
        if (cns_may_access_topic($info[0]['p_topic_id'])) {
            return true;
        }
        if (addon_installed('tickets')) {
            $tf = get_option('ticket_forum_name', true);
            if ($tf !== null) {
                $forum2 = $GLOBALS['FORUM_DRIVER']->forum_id_from_name($tf);
            } else {
                $forum2 = null;
            }
            if (($forum2 === $forum_id) || ($forum2 === $forum_id_parent) || ($forum2 === $forum_id_parent_parent)) {
                $title = $GLOBALS['FORUM_DB']->query_select_value('f_topics', 't_cache_first_title', ['id' => $info[0]['p_topic_id']]);
                if (substr($title, 0, strlen(strval($member_id)) + 1) == strval($member_id) . '_') {
                    return true;
                }
                require_lang('tickets');

                $description = $GLOBALS['FORUM_DB']->query_select_value('f_topics', 't_description', ['id' => $info[0]['p_topic_id']]);
                if (substr($description, 0, strlen(do_lang('SUPPORT_TICKET') . ': #' . strval($member_id)) + 1) == do_lang('SUPPORT_TICKET') . ': #' . strval($member_id) . '_') {
                    return true;
                }
            }
        }
        return false;
    }
}
