<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    cns_forum
 */

/**
 * Hook class.
 */
class Hook_cleanup_cns
{
    /**
     * Find details about this cleanup hook.
     *
     * @return ?array Map of cleanup hook info (null: hook is disabled)
     */
    public function info() : ?array
    {
        if (!addon_installed('cns_forum')) {
            return null;
        }

        if (get_forum_type() != 'cns') {
            return null;
        }

        cns_require_all_forum_stuff();

        require_lang('cns');

        $info = [];
        $info['title'] = do_lang_tempcode('SECTION_FORUMS');
        $info['description'] = do_lang_tempcode('DESCRIPTION_FORUMS_CACHE');
        $info['type'] = 'cache';

        return $info;
    }

    /**
     * Run the cleanup hook action.
     *
     * @return Tempcode Results
     */
    public function run() : object
    {
        if (get_forum_type() != 'cns') {
            return new Tempcode();
        }

        require_lang('cns');

        require_code('tasks');
        return call_user_func_array__long_task(do_lang('CACHE_FORUMS'), null, 'cns_recache');
    }
}
