<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_cleanup_tools
 */

/**
 * Hook class.
 */
class Hook_cleanup_disposable_values
{
    /**
     * Find details about this cleanup hook.
     *
     * @return ?array Map of cleanup hook info (null: hook is disabled)
     */
    public function info() : ?array
    {
        if (!addon_installed('stats')) {
            return null;
        }

        if ($GLOBALS['CURRENT_SHARE_USER'] !== null) {
            return null;
        }

        $info = [];
        $info['title'] = do_lang_tempcode('DISPOSABLE_VALUES_CACHE');
        $info['description'] = do_lang_tempcode('DESCRIPTION_DISPOSABLE_VALUES_CACHE');
        $info['type'] = 'cache';

        return $info;
    }

    /**
     * Run the cleanup hook action.
     *
     * @return Tempcode Results
     */
    public function run() : object
    {
        if ($GLOBALS['CURRENT_SHARE_USER'] !== null) {
            return new Tempcode();
        }

        $hooks = find_all_hooks('systems', 'disposable_values');
        foreach (array_keys($hooks) as $hook) {
            $GLOBALS['SITE_DB']->query_delete('values', ['the_name' => $hook], '', 1);
        }
        persistent_cache_delete('VALUES');

        return new Tempcode();
    }
}
