<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    points
 */

/**
 * Hook class.
 */
class Hook_cleanup_points_recalculate_cpf
{
    /**
     * Find details about this cleanup hook.
     *
     * @return ?array Map of cleanup hook info (null: hook is disabled)
     */
    public function info() : ?array
    {
        if (!addon_installed('points')) {
            return null;
        }

        $info = [];
        $info['title'] = do_lang_tempcode('points:POINTS_CACHE');
        $info['description'] = do_lang_tempcode('points:DESCRIPTION_POINTS_CACHE');
        $info['type'] = 'cache';

        return $info;
    }

    /**
     * Run the cleanup hook action.
     *
     * @return Tempcode Results
     */
    public function run() : object
    {
        if (!addon_installed('points')) {
            return new Tempcode();
        }

        require_code('tasks');
        call_user_func_array__long_task(do_lang('points:POINTS_CACHE'), get_screen_title('points:POINTS_CACHE'), 'points_recalculate_cpf', []);

        return new Tempcode();
    }
}
