<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    commandr
 */

/**
 * Hook class.
 */
class Hook_commandr_command_mvdir
{
    /**
     * Run function for Commandr hooks.
     *
     * @param  array $options The options with which the command was called
     * @param  array $parameters The parameters with which the command was called
     * @param  object $commandr_fs A reference to the Commandr filesystem object
     * @return array Array of stdcommand, stdhtml, stdout, and stderr responses
     */
    public function run(array $options, array $parameters, object &$commandr_fs) : array
    {
        if ((array_key_exists('h', $options)) || (array_key_exists('help', $options))) {
            return ['', do_command_help('mvdir', ['h'], [true, true, true]), '', ''];
        }

        if (!array_key_exists(0, $parameters)) {
            return ['', '', '', do_lang('MISSING_PARAM', '1', 'mvdir')];
        } else {
            $parameters[0] = $commandr_fs->_pwd_to_array($parameters[0]);
        }
        if (!array_key_exists(1, $parameters)) {
            $parameters[1] = $commandr_fs->print_working_directory(true);
        } else {
            $parameters[1] = $commandr_fs->_pwd_to_array($parameters[1]);
        }

        if (!$commandr_fs->_is_dir($parameters[0])) {
            return ['', '', '', do_lang('NOT_A_DIR', '1')];
        }
        if (!$commandr_fs->_is_dir($parameters[1])) {
            return ['', '', '', do_lang('NOT_A_DIR', '2')];
        }

        if (!array_key_exists(2, $parameters)) {
            $parameters[2] = end($parameters[1]);
        }

        $success = $commandr_fs->move_directory($parameters[0], array_merge($parameters[1], [$parameters[2]]));
        if (!$success) {
            return ['', '', '', do_lang('INCOMPLETE_ERROR')];
        }

        return ['', '', do_lang('SUCCESS'), ''];
    }
}
