<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    commandr
 */

/**
 * Hook class.
 */
class Hook_commandr_command_rm
{
    /**
     * Run function for Commandr hooks.
     *
     * @param  array $options The options with which the command was called
     * @param  array $parameters The parameters with which the command was called
     * @param  object $commandr_fs A reference to the Commandr filesystem object
     * @return array Array of stdcommand, stdhtml, stdout, and stderr responses
     */
    public function run(array $options, array $parameters, object &$commandr_fs) : array
    {
        if ((array_key_exists('h', $options)) || (array_key_exists('help', $options))) {
            return ['', do_command_help('rm', ['h'], [true]), '', ''];
        }

        if (!array_key_exists(0, $parameters)) {
            return ['', '', '', do_lang('MISSING_PARAM', '1', 'rm')];
        }

        $success = true;

        foreach ($parameters as $i => $param) {
            $param = $commandr_fs->_pwd_to_array($param);

            if (!$commandr_fs->_is_file($param)) {
                $success = false;
                if (($i == 0) && (count($parameters) == 1)) {
                    return ['', '', '', do_lang('NOT_A_FILE', strval($i + 1))];
                }
            }

            $success = $success && $commandr_fs->remove_file($param);
        }

        if ($success) {
            return ['', '', do_lang('SUCCESS'), ''];
        }

        return ['', '', '', do_lang('INCOMPLETE_ERROR')];
    }
}
