<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    commandr
 */

/**
 * Hook class.
 */
class Hook_commandr_command_test_mail
{
    /**
     * Run function for Commandr hooks.
     *
     * @param  array $options The options with which the command was called
     * @param  array $parameters The parameters with which the command was called
     * @param  object $commandr_fs A reference to the Commandr filesystem object
     * @return array Array of stdcommand, stdhtml, stdout, and stderr responses
     */
    public function run(array $options, array $parameters, object &$commandr_fs) : array
    {
        if ((array_key_exists('h', $options)) || (array_key_exists('help', $options))) {
            return ['', do_command_help('test_mail', ['h'], [true]), '', ''];
        }

        $to_email = $GLOBALS['FORUM_DRIVER']->get_member_email_address(get_member());

        if (!array_key_exists(0, $parameters)) {
            if ($to_email == '') {
                return ['', '', '', do_lang('MISSING_PARAM', '1', 'test_mail')];
            }
        } else {
            $to_email = $parameters[0];
        }

        require_code('mail');

        global $ATTACHED_MESSAGES_RAW;
        $ATTACHED_MESSAGES_RAW = [];

        if (@$GLOBALS['SITE_INFO']['no_email_output'] === '1') {
            return ['', '', '', 'no_email_output==1'];
        }

        $mail_ob = dispatch_mail('Testing', 'Testing', '', [$to_email], 'Tester', '', '', ['priority' => 1, 'bypass_queue' => true]);
        return ['', '', $mail_ob->log, ''];
    }
}
