<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    themewizard
 */

/**
 * Hook class.
 */
class Hook_commandr_command_themewizard_compute_equation
{
    /**
     * Run function for Commandr hooks.
     *
     * @param  array $options The options with which the command was called
     * @param  array $parameters The parameters with which the command was called
     * @param  object $commandr_fs A reference to the Commandr filesystem object
     * @return array Array of stdcommand, stdhtml, stdout, and stderr responses
     */
    public function run(array $options, array $parameters, object &$commandr_fs) : array
    {
        if (!addon_installed('themewizard')) {
            return ['', '', '', do_lang('INTERNAL_ERROR', comcode_escape('16b443b2a32a1a0611e0336c8db192b1'))];
        }

        require_lang('themes');

        if ((array_key_exists('h', $options)) || (array_key_exists('help', $options))) {
            return ['', do_command_help('themewizard_compute_equation', ['h'], [true, true]), '', ''];
        }

        if (!array_key_exists(0, $parameters)) {
            return ['', '', '', do_lang('MISSING_PARAM', '1', 'themewizard_compute_equation')];
        }

        $equation = $parameters[0];

        $theme = array_key_exists(1, $parameters) ? $parameters[1] : 'default';

        require_code('themewizard');
        require_code('themes2');

        $seed = find_theme_seed($theme);
        $dark = find_theme_dark($theme);

        list($colours, $landscape) = calculate_themewizard_css_colours($seed, $dark, $theme, 'equations');
        $parsed_equation = parse_themewizard_css_colour_expression($equation);
        if ($parsed_equation === null) {
            return ['', '', '', '?'];
        }
        $answer = execute_themewizard_css_colour_expression($parsed_equation, $colours);
        if ($answer === null) {
            return ['', '', '', '?'];
        }

        return ['', '<span style="padding: 0.5em 0.8em; display: inline-block; background: white"><span style="border: 1px solid black; width: 2em; height: 1em; display: inline-block; background: #' . escape_html($answer) . '"></span></span>', '#' . $answer, ''];
    }
}
