<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    commandr
 */

// For opposite command see 'fs_search'.

// Does not use a task because that just makes things complex and this is meant to be quick and easy. If a more sophisticated system is needed, use something like SSH or cPanel.

/**
 * Hook class.
 */
class Hook_commandr_command_untar
{
    /**
     * Run function for Commandr hooks.
     *
     * @param  array $options The options with which the command was called
     * @param  array $parameters The parameters with which the command was called
     * @param  object $commandr_fs A reference to the Commandr filesystem object
     * @return array Array of stdcommand, stdhtml, stdout, and stderr responses
     */
    public function run(array $options, array $parameters, object &$commandr_fs) : array
    {
        if ((array_key_exists('h', $options)) || (array_key_exists('help', $options))) {
            return ['', do_command_help('untar', ['h'], [true]), '', ''];
        }

        if (!array_key_exists(0, $parameters)) {
            return ['', '', '', do_lang('MISSING_PARAM', '1', 'untar')];
        }

        $path = $parameters[0];

        if (!is_file(get_custom_file_base() . '/' . $path)) {
            return ['', '', '', do_lang('MISSING_RESOURCE')];
        }

        disable_php_memory_limit();

        require_code('tar');

        $myfile = tar_open(get_custom_file_base() . '/' . $path, 'rb');
        tar_extract_to_folder($myfile, '');
        tar_close($myfile);

        return ['', '', do_lang('SUCCESS'), ''];
    }
}
