<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_configuration
 */

/**
 * Hook class.
 */
class Hook_config_forum_in_portal
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'FORUM_IN_PORTAL',
            'type' => 'tick',
            'category' => 'SITE',
            'group' => 'ADVANCED',
            'explanation' => (get_forum_type() == 'cns') ? 'CONFIG_OPTION_forum_in_portal__cns' : 'CONFIG_OPTION_forum_in_portal',
            'shared_hosting_restricted' => '1',
            'list_options' => '',
            'required' => true,
            'public' => false,
            'addon' => 'core_configuration',

            'maintenance_code' => 'forum_in_portal',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        return has_no_forum() ? null : '0';
    }

    /**
     * Code to run after the option is saved, if the value was changed or we are not formally setting it.
     *
     * @param  string $new_value The new value
     */
    public function postsave_handler(string $new_value)
    {
        // Need to clear some additional caches so URLs render properly
        require_code('caches3');

        erase_block_cache(true);
        erase_cached_templates();
        erase_persistent_cache();
    }
}
