<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    health_check
 */

/**
 * Hook class.
 */
class Hook_config_hc_cron_sections_to_run
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'HC_CRON_SECTIONS_TO_RUN',
            'type' => 'special',
            'category' => 'HEALTH_CHECK',
            'group' => 'AUTOMATIC_CHECKS',
            'explanation' => 'CONFIG_OPTION_hc_cron_sections_to_run',
            'shared_hosting_restricted' => '0',
            'list_options' => '',
            'order_in_category_group' => 1,
            'required' => false,
            'public' => false,
            'addon' => 'health_check',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        if (!addon_installed('health_check')) {
            return null;
        }

        return '';
    }

    /**
     * Field inputter (because the_type=special).
     *
     * @param  ID_TEXT $name The config option name
     * @param  array $myrow The config row
     * @param  ID_TEXT $config_field_name The form field name to use for this config
     * @param  Tempcode $human_name The field title
     * @param  Tempcode $explanation The field description
     * @return Tempcode The inputter
     */
    public function field_inputter(string $name, array $myrow, string $config_field_name, object $human_name, object $explanation) : object
    {
        require_code('health_check');

        $current = (get_option($name) == '') ? [] : explode(',', get_option($name));
        $list = create_selection_list_health_check_sections($current);

        return form_input_multi_list($human_name, $explanation, $config_field_name, $list, null, 15);
    }

    /**
     * Code to run after the option is saved, if the value was changed or we are not formally setting it.
     *
     * @param  string $new_value The new value
     */
    public function postsave_handler(string $new_value)
    {
        // Clear the checklist cache so we have an updated account of when health check is due
        require_code('caches');
        delete_cache_entry('main_staff_checklist');
    }
}
