<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_configuration
 */

/**
 * Hook class.
 */
class Hook_config_mail_username
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'USERNAME',
            'type' => 'line',
            'category' => 'SERVER',
            'group' => 'IMAP',
            'explanation' => 'CONFIG_OPTION_mail_username',
            'shared_hosting_restricted' => '1',
            'list_options' => '',
            'order_in_category_group' => 5,
            'required' => true,
            'public' => false,
            'addon' => 'core_configuration',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        if (!function_exists('imap_open') && !addon_installed('imap')) {
            return null;
        }

        $website_email = get_option('website_email');
        return preg_replace('#@.*$#', '', $website_email);
    }
}
