<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    newsletter
 */

/**
 * Hook class.
 */
class Hook_config_newsletter_website_email
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'WEBSITE_EMAIL',
            'type' => 'line',
            'category' => 'MESSAGES',
            'group' => 'NEWSLETTER',
            'explanation' => 'CONFIG_OPTION_website_email',
            'shared_hosting_restricted' => '0',
            'list_options' => '',
            'order_in_category_group' => 11,
            'required' => true,
            'public' => false,
            'addon' => 'newsletter',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        if (!addon_installed('newsletter')) {
            return null;
        }

        $staff_address = get_option('staff_address');
        $website_email = 'website@' . get_domain();
        if (substr($staff_address, -strlen(get_domain()) - 1) == '@' . get_domain()) {
            $website_email = $staff_address;
        }
        return $website_email;
    }
}
