<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    ecommerce
 */

/**
 * Hook class.
 */
class Hook_config_payment_gateway_username
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'PAYMENT_GATEWAY_USERNAME',
            'type' => 'line',
            'category' => 'ECOMMERCE',
            'group' => 'PAYMENT_GATEWAY',
            'explanation' => 'CONFIG_OPTION_payment_gateway_username',
            'shared_hosting_restricted' => '0',
            'list_options' => '',
            'order_in_category_group' => 3,
            'required' => false,
            'public' => false,
            'ecommerce' => true,
            'addon' => 'ecommerce',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        if (!addon_installed('ecommerce')) {
            return null;
        }

        return ''; // We used to default the option, but this will turn on the CSP stuff for PayPal immediately and bloat up our headers even if PayPal not used, so now require explicit configuration -- and besides, not all payment gateways take an e-mail like this
        //return get_option('staff_address');
    }
}
