<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    ecommerce
 */

/**
 * Hook class.
 */
class Hook_config_shipping_density
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'SHIPPING_DENSITY',
            'type' => 'float',
            'category' => 'ECOMMERCE',
            'group' => 'SHIPPING',
            'explanation' => 'CONFIG_OPTION_shipping_density',
            'shared_hosting_restricted' => '1',
            'list_options' => '',
            'order_in_category_group' => 7,
            'required' => true,
            'public' => false,
            'addon' => 'ecommerce',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        if (!addon_installed('ecommerce')) {
            return null;
        }

        require_code('locations');
        if (geolocate_ip() == 'US' || get_option('yeehaw') == '1') {
            return '139.0'; // 139 in3/lb (https://en.wikipedia.org/wiki/Dimensional_weight)
        }
        return '5000.0'; // 5000 cm3/kg (https://en.wikipedia.org/wiki/Dimensional_weight)
    }
}
