<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_configuration
 */

/**
 * Hook class.
 */
class Hook_config_yeehaw
{
    /**
     * Gets the details relating to the config option.
     *
     * @return array The details
     */
    public function get_details() : array
    {
        return [
            'human_name' => 'AMERICAN_ENGLISH',
            'type' => 'tick',
            'category' => 'SITE',
            'group' => 'INTERNATIONALISATION',
            'explanation' => 'CONFIG_OPTION_yeehaw',
            'shared_hosting_restricted' => '0',
            'list_options' => '',
            'order_in_category_group' => 5,
            'required' => true,
            'public' => false,
            'addon' => 'core_configuration',

            'maintenance_code' => 'lang_us',
        ];
    }

    /**
     * Gets the default value for the config option.
     *
     * @return ?string The default value (null: option is disabled)
     */
    public function get_default() : ?string
    {
        require_code('locations');
        if (geolocate_ip() == 'US') {
            return '1';
        }
        return '0';
    }

    /**
     * Code to run before the option is saved.
     * If there is some kind of problem with the new value then we could attach an error message.
     *
     * @param  string $new_value The new value
     * @param  string $old_value The old value
     * @return boolean Whether to allow the save
     */
    public function presave_handler(string $new_value, string $old_value) : bool
    {
        erase_cached_language();

        return true;
    }
}
