<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    banners
 */

/**
 * Hook class.
 */
class Hook_content_meta_aware_banner extends Hook_CMA
{
    /**
     * Get content type details.
     *
     * @param  ?ID_TEXT $zone The zone to link through to (null: autodetect)
     * @param  boolean $get_extended_data Populate additional data that is somewhat costly to compute (add_url, archive_url)
     * @return ?array Map of content-type info (null: disabled)
     */
    public function info(?string $zone = null, bool $get_extended_data = false) : ?array
    {
        if (!addon_installed('banners')) {
            return null;
        }

        return [
            'support_custom_fields' => false,

            'content_type_label' => 'global:BANNER',
            'content_type_universal_label' => 'Banner',

            'db' => $GLOBALS['SITE_DB'],
            'table' => 'banners',
            'id_field' => 'name',
            'id_field_numeric' => false,
            'parent_category_field' => 'b_type',
            'parent_category_meta_aware_type' => 'banner_type',
            'is_category' => false,
            'is_entry' => true,
            'category_field' => 'b_type', // For category permissions
            'permission_module' => 'banners', // For category permissions
            'parent_spec__table_name' => null,
            'parent_spec__parent_name' => null,
            'parent_spec__field_name' => null,
            'category_is_string' => true,

            'title_field' => 'name',
            'title_field_dereference' => false,
            'description_field' => 'caption',
            'description_field_dereference' => true,
            'description_field_supports_comcode' => true,
            'image_field' => 'img_url',
            'image_field_is_theme_image' => false,
            'alternate_icon_theme_image' => 'icons/menu/cms/banners',

            'view_page_link_pattern' => '_SEARCH:banners:view:source=_WILD',
            'edit_page_link_pattern' => '_SEARCH:cms_banners:_edit:_WILD',
            'view_category_page_link_pattern' => null,
            'add_url' => ($get_extended_data && has_submit_permission('mid', get_member(), get_ip_address(), 'cms_banners')) ? (get_module_zone('cms_banners') . ':cms_banners:add') : null,
            'archive_url' => null,

            'support_url_monikers' => false,

            'views_field' => null,
            'order_field' => null,
            'submitter_field' => 'submitter',
            'author_field' => null,
            'add_time_field' => 'add_date',
            'edit_time_field' => 'edit_date',
            'date_field' => 'add_date',
            'validated_field' => 'validated',
            'additional_sort_fields' => [
                'b_type' => [
                    'ASC' => 'r.b_type ASC,r.name ASC',
                    'DESC' => 'r.b_type ASC,r.name DESC',
                ],
                'deployment_agreement' => null,
                'campaign_remaining' => null,
                'display_likelihood' => null,
                'expiry_date' => null,
            ],

            'seo_type_code' => null,

            'feedback_type_code' => null,

            'search_hook' => null,
            'rss_hook' => null,
            'attachment_hook' => null,
            'notification_hook' => null,
            'sitemap_hook' => 'banner',

            'addon_name' => 'banners',

            'cms_page' => 'cms_banners',
            'module' => 'banners',

            'commandr_filesystem_hook' => 'banners',
            'commandr_filesystem__is_folder' => false,

            'support_revisions' => false,

            'support_privacy' => false,

            'support_content_reviews' => true,

            'support_spam_heuristics' => null,

            'actionlog_regexp' => '\w+_BANNER',

            'default_prominence_weight' => PROMINENCE_WEIGHT_NONE,
            'default_prominence_flags' => 0,
        ];
    }

    /**
     * Render a content box for a content row.
     *
     * @param  array $row The database row for the content
     * @param  ID_TEXT $zone The zone to display in
     * @param  boolean $give_context Whether to include context (i.e. say WHAT this is, not just show the actual content)
     * @param  boolean $include_breadcrumbs Whether to include breadcrumbs (if there are any)
     * @param  ?ID_TEXT $root Virtual root to use (null: none)
     * @param  boolean $attach_to_url_filter Whether to copy through any filter parameters in the URL, under the basis that they are associated with what this box is browsing
     * @param  ID_TEXT $guid Overridden GUID to send to templates (blank: none)
     * @return Tempcode Results
     */
    public function render_box(array $row, string $zone, bool $give_context = true, bool $include_breadcrumbs = true, ?string $root = null, bool $attach_to_url_filter = false, string $guid = '') : object
    {
        require_code('banners2');

        return render_banner_box($row, $zone, $give_context, $guid);
    }

    /**
     * Create a selection list.
     *
     * @param  ?string $id The pre-selected ID (null: none selected)
     * @return Tempcode List
     */
    public function create_selection_list(?string $id = null) : object
    {
        require_code('banners2');

        return create_selection_list_banners($id);
    }
}
