<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    cns_forum
 */

/**
 * Hook class.
 */
class Hook_cron_cns_forum_email_integration
{
    /**
     * Get info from this hook.
     *
     * @param  ?TIME $last_run Last time run (null: never)
     * @param  ?boolean $calculate_num_queued Calculate the number of items queued, if possible (null: the hook may decide / low priority)
     * @return ?array Return a map of info about the hook (null: disabled)
     */
    public function info(?int $last_run, ?bool $calculate_num_queued) : ?array
    {
        if (get_forum_type() != 'cns') {
            return null;
        }

        if (!addon_installed('cns_forum')) {
            return null;
        }

        require_code('cns_forums2');
        $test = cns_has_mailing_list_style();
        if ($test[0] == 0) {
            return null; // Possibly due to not being fully configured yet
        }

        return [
            'label' => 'Forum email integration',
            'num_queued' => null,
            'minutes_between_runs' => 1,
            'enabled_by_default' => true,
        ];
    }

    /**
     * Run function for system scheduler hooks. Searches for tasks to perform.
     */
    public function run()
    {
        require_code('mail_integration');
        require_code('cns_forum_email_integration');
        $email_ob = new ForumEmailIntegration();
        $email_ob->incoming_scan();
    }
}
