<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_cron_decache_events
{
    /**
     * Get info from this hook.
     *
     * @param  ?TIME $last_run Last time run (null: never)
     * @param  ?boolean $calculate_num_queued Calculate the number of items queued, if possible (null: the hook may decide / low priority)
     * @return ?array Return a map of info about the hook (null: disabled)
     */
    public function info(?int $last_run, ?bool $calculate_num_queued) : ?array
    {
        return [
            'label' => 'Event-based decaching',
            'num_queued' => null,
            'minutes_between_runs' => 29, // So it runs before Health Check
            'enabled_by_default' => true,
        ];
    }

    /**
     * Run function for system scheduler hooks. Searches for things to do. ->info(..., true) must be called before this method.
     *
     * @param  ?TIME $last_run Last time run (null: never)
     */
    public function run(?int $last_run)
    {
        // We cache the copyright symbol, but each year it needs replacing...
        $current_year = intval(date('Y', tz_time(time(), get_server_timezone())));
        $current_year_recently = intval(date('Y', tz_time(time() - 60 * 60, get_server_timezone())));
        if ($current_year != $current_year_recently) {
            require_code('caches3');
            erase_cached_templates(false, null, '\$COPYRIGHT');
        }
    }
}
