<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_notification_core_staff extends Hook_notification__Staff
{
    /**
     * Find the initial setting that members have for a notification code (only applies to the member_could_potentially_enable members).
     *
     * @param  ID_TEXT $notification_code Notification code
     * @param  ?SHORT_TEXT $category The category within the notification code (null: none)
     * @param  MEMBER $member_id The member the notification would be for
     * @return integer Initial setting
     */
    public function get_initial_setting(string $notification_code, ?string $category, int $member_id) : int
    {
        if (($this->is_first_admin($member_id)) && (($notification_code == 'spam_check_block') || ($notification_code == 'auto_ban'))) {
            return A_WEEKLY_EMAIL_DIGEST;
        }
        return A_NA;
    }

    /**
     * Get a list of all the notification codes this hook can handle.
     * (Addons can define hooks that handle whole sets of codes, so hooks are written so they can take wide authority).
     *
     * @return array List of codes (mapping between code names, and a pair: section and labelling for those codes)
     */
    public function list_handled_codes() : array
    {
        $list = [];
        $list['auto_ban'] = [do_lang('SECURITY'), do_lang('NOTIFICATION_TYPE_auto_ban')];
        $list['hack_attack'] = [do_lang('SECURITY'), do_lang('NOTIFICATION_TYPE_hack_attack')];
        $list['spam_check_block'] = [do_lang('SECURITY'), do_lang('NOTIFICATION_TYPE_spam_check_block')];
        $list['adminzone_dashboard_accessed'] = [do_lang('SECURITY'), do_lang('NOTIFICATION_TYPE_adminzone_dashboard_accessed')];
        return $list;
    }
}
