<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_page_groupings_core
{
    /**
     * Run function for do_next_menu hooks. They find links to put on standard navigation menus of the system.
     *
     * @param  ?MEMBER $member_id Member ID to run as (null: current member)
     * @param  boolean $extensive_docs Whether to use extensive documentation tooltips, rather than short summaries
     * @return array List of tuple of links (page grouping, icon, do-next-style linking data), label, help (optional) and/or nulls
     */
    public function run(?int $member_id = null, bool $extensive_docs = false) : array
    {
        require_code('site');

        if ($member_id === null) {
            $member_id = get_member();
        }

        return [
            ['', 'menu/adminzone/' . DEFAULT_ZONE_PAGE_NAME, [DEFAULT_ZONE_PAGE_NAME, [], 'adminzone'], do_lang_tempcode('menus:DASHBOARD'), $extensive_docs ? 'menus:DOC_DASHBOARD' : 'menus:MM_TOOLTIP_DASHBOARD'],
            ['', 'menu/adminzone/audit', ['admin', ['type' => 'audit'], get_module_zone('admin')], do_lang_tempcode('menus:AUDIT'), $extensive_docs ? 'menus:DOC_AUDIT' : 'menus:MM_TOOLTIP_AUDIT'],
            ['', 'menu/adminzone/security', ['admin', ['type' => 'security'], get_module_zone('admin')], do_lang_tempcode('SECURITY'), $extensive_docs ? 'menus:DOC_SECURITY' : 'menus:MM_TOOLTIP_SECURITY'],
            ['', 'menu/adminzone/setup', ['admin', ['type' => 'setup'], get_module_zone('admin')], do_lang_tempcode('SETUP'), $extensive_docs ? 'menus:DOC_SETUP' : 'menus:MM_TOOLTIP_SETUP'],
            ['', 'menu/adminzone/structure', ['admin', ['type' => 'structure'], get_module_zone('admin')], do_lang_tempcode('menus:STRUCTURE'), $extensive_docs ? 'menus:DOC_STRUCTURE' : 'menus:MM_TOOLTIP_STRUCTURE'],
            ['', 'menu/adminzone/style', ['admin', ['type' => 'style'], get_module_zone('admin')], do_lang_tempcode('menus:STYLE'), $extensive_docs ? 'menus:DOC_STYLE' : 'menus:MM_TOOLTIP_STYLE'],
            ['', 'menu/adminzone/tools', ['admin', ['type' => 'tools'], get_module_zone('admin')], do_lang_tempcode('TOOLS'), $extensive_docs ? 'menus:DOC_TOOLS' : 'menus:MM_TOOLTIP_TOOLS'],
            (_request_page('website', 'adminzone') === null) ? ['', 'help', get_brand_base_url() . '/docs' . strval(cms_version()) . '/', do_lang_tempcode('menus:DOCS'), $extensive_docs ? 'menus:DOCS' : 'menus:MM_TOOLTIP_DOCS'] : null,
            (_request_page('website', 'adminzone') === null) ? null : ['', 'help', ['website', [], 'adminzone'], do_lang_tempcode('menus:DOCS'), $extensive_docs ? 'menus:DOCS' : 'menus:MM_TOOLTIP_DOCS'],
            ['', 'menu/cms/cms', ['cms', ['type' => 'cms'], get_module_zone('cms')], do_lang_tempcode('CONTENT'), $extensive_docs ? 'CONTENT' : 'menus:MM_TOOLTIP_CMS'],

            ((has_some_edit_comcode_page_permission(COMCODE_EDIT_OWN | COMCODE_EDIT_ANY)) || (!empty(get_comcode_page_editability_per_zone()))) ? ['cms', 'menu/cms/comcode_page_edit', ['cms_comcode_pages', ['type' => 'browse'], get_module_zone('cms_comcode_pages')], do_lang_tempcode('ITEMS_HERE', do_lang_tempcode('menus:_COMCODE_PAGES'), make_string_tempcode(escape_html(integer_format(intval($GLOBALS['SITE_DB']->query_value_if_there('SELECT COUNT(*) FROM (SELECT DISTINCT the_zone,the_page FROM ' . get_table_prefix() . 'comcode_pages WHERE ' . db_string_not_equal_to('the_zone', '!') . ') x')), 0)))), 'zones:DOC_COMCODE_PAGE_EDIT'] : null,
            (get_forum_type() != 'cns' || !addon_installed('cns_clubs')) ? null : ['cms', 'menu/cms/clubs', ['cms_cns_groups', ['type' => 'browse'], get_module_zone('cms_cns_groups')], do_lang_tempcode('ITEMS_HERE', do_lang_tempcode('cns:CLUBS'), make_string_tempcode(escape_html(integer_format(intval($GLOBALS['FORUM_DB']->query_select_value('f_groups', 'COUNT(*)', ['g_is_private_club' => 1])), 0)))), 'cns:DOC_CLUBS'],

            ['structure', 'menu/adminzone/structure/zones/zones', ['admin_zones', ['type' => 'browse'], get_module_zone('admin_zones')], do_lang_tempcode('ZONES'), 'zones:DOC_ZONES'],
            (get_value('hide_zone_editor') === '1') ? null : ['structure', 'menu/adminzone/structure/zones/zone_editor', ['admin_zones', ['type' => 'editor'], get_module_zone('admin_zones')], do_lang_tempcode('zones:ZONE_EDITOR'), 'zones:DOC_ZONE_EDITOR'],
            ['structure', 'menu/adminzone/structure/menus', ['admin_menus', ['type' => 'browse'], get_module_zone('admin_menus')], do_lang_tempcode('menus:MENU_MANAGEMENT'), 'menus:DOC_MENUS'],
            addon_installed('page_management') ? ['structure', 'menu/adminzone/structure/sitemap/sitemap_editor', ['admin_sitemap', ['type' => 'browse'], get_module_zone('admin_sitemap')], do_lang_tempcode('zones:SITEMAP_EDITOR'), 'zones:DOC_SITEMAP_EDITOR'] : null,
            addon_installed('redirects_editor') ? ['structure', 'menu/adminzone/structure/redirects', ['admin_redirects', ['type' => 'browse'], get_module_zone('admin_redirects')], do_lang_tempcode('REDIRECTS'), 'redirects:DOC_REDIRECTS'] : null,
            addon_installed('breadcrumbs') ? ['structure', 'menu/adminzone/structure/breadcrumbs', ['admin_config', ['type' => 'xml_breadcrumbs'], get_module_zone('admin_config')], do_lang_tempcode('config:BREADCRUMB_OVERRIDES'), 'config:DOC_BREADCRUMB_OVERRIDES'] : null,
            addon_installed('securitylogging') ? ['security', 'menu/adminzone/audit/security_log', ['admin_config', ['type' => 'advanced_banning'], get_module_zone('admin_config')], do_lang_tempcode('config:ADVANCED_BANNING'), 'config:DOC_ADVANCED_BANNING'] : null,
            addon_installed('robots_txt') ? ['structure', 'spare/seo', ['admin_robots_txt', [], get_module_zone('admin_robots_txt')], do_lang_tempcode('robots_txt:ROBOTS_TXT'), 'robots_txt:DOC_ROBOTS_TXT'] : null,
            ['structure', 'menu/adminzone/structure/addons', ['admin_addons', ['type' => 'browse'], get_module_zone('admin_addons')], do_lang_tempcode('addons:ADDONS'), 'addons:DOC_ADDONS'],

            (get_forum_type() != 'cns' || !addon_installed('cns_cpfs')) ? null : ['audit', 'menu/adminzone/tools/users/custom_profile_fields', ['admin_cns_customprofilefields', ['type' => 'stats'], get_module_zone('admin_cns_customprofilefields')], do_lang_tempcode('cns:CUSTOM_PROFILE_FIELD_STATS'), 'cns:DOC_CUSTOM_PROFILE_FIELDS_STATS'],
            addon_installed('errorlog') ? ['audit', 'menu/adminzone/audit/errorlog', ['admin_errorlog', [], get_module_zone('admin_errorlog')], do_lang_tempcode('errorlog:ERRORLOG'), 'errorlog:DOC_ERRORLOG'] : null,
            addon_installed('actionlog') ? ['audit', 'menu/adminzone/audit/actionlog', ['admin_actionlog', ['type' => 'browse'], get_module_zone('admin_actionlog')], do_lang_tempcode('actionlog:VIEW_ACTIONLOGS'), 'actionlog:DOC_ACTIONLOG'] : null,
            addon_installed('securitylogging') ? ['audit', 'menu/adminzone/audit/security_log', ['admin_security', ['type' => 'browse'], get_module_zone('admin_security')], do_lang_tempcode('security:SECURITY_LOG'), 'security:DOC_SECURITY_LOG'] : null,
            ['audit', 'menu/adminzone/audit/email_log', ['admin_email_log', ['type' => 'browse'], get_module_zone('admin_email_log')], do_lang_tempcode('email_log:EMAIL_LOG'), 'email_log:DOC_EMAIL_LOG'],
            (addon_installed('actionlog') && get_option('store_revisions', true) === '1') ? ['audit', 'admin/revisions', ['admin_revisions', ['type' => 'browse'], get_module_zone('admin_revisions')], do_lang_tempcode('actionlog:REVISIONS'), 'actionlog:DOC_REVISIONS'] : null,
            (!addon_installed('content_reviews')) ? null : ['audit', 'menu/adminzone/audit/content_reviews', ['admin_content_reviews', ['type' => 'browse'], get_module_zone('admin_content_reviews')], do_lang_tempcode('content_reviews:CONTENT_REVIEWS'), 'content_reviews:DOC_CONTENT_REVIEWS'],

            ['style', 'menu/adminzone/style/themes/themes', ['admin_themes', ['type' => 'browse'], get_module_zone('admin_themes')], do_lang_tempcode('themes:THEMES'), 'themes:DOC_THEMES'],
            ['style', 'admin/tool', ['admin_svg_sprites', [], get_page_zone('admin_svg_sprites')], do_lang_tempcode('themes:SVG_SPRITES'), 'themes:DOC_SVG_SPRITES'],
            (get_forum_type() != 'cns') ? null : ['style', 'menu/adminzone/style/emoticons', ['admin_cns_emoticons', ['type' => 'browse'], get_module_zone('admin_cns_emoticons')], do_lang_tempcode('ITEMS_HERE', do_lang_tempcode('EMOTICONS'), make_string_tempcode(escape_html(integer_format(intval($GLOBALS['FORUM_DB']->query_select_value('f_emoticons', 'COUNT(*)')), 0)))), 'cns:DOC_EMOTICONS'],

            ['setup', 'menu/adminzone/setup/config/config', ['admin_config', ['type' => 'browse'], get_module_zone('admin_config')], do_lang_tempcode('config:CONFIGURATION'), 'config:DOC_CONFIGURATION'],
            ['setup', 'menu/adminzone/setup/oauth', ['admin_oauth', ['type' => 'browse'], get_module_zone('admin_oauth')], do_lang_tempcode('oauth:_OAUTH_TITLE'), 'oauth:DOC_OAUTH'],
            addon_installed('awards') ? ['setup', 'menu/adminzone/setup/awards', ['admin_awards', ['type' => 'browse'], get_module_zone('admin_awards')], do_lang_tempcode('awards:AWARDS'), 'awards:DOC_AWARDS'] : null,
            addon_installed('awards') ? ['site_meta', 'menu/adminzone/setup/awards', ['awards', ['type' => 'browse'], get_module_zone('awards')], do_lang_tempcode('awards:AWARDS')] : null,
            (get_forum_type() == 'cns' || !addon_installed('welcome_emails')) ? /*Is on members menu*/null : ['setup', 'menu/adminzone/setup/welcome_emails', ['admin_cns_welcome_emails', ['type' => 'browse'], get_module_zone('admin_cns_welcome_emails')], do_lang_tempcode('cns_welcome_emails:WELCOME_EMAILS'), 'cns_welcome_emails:DOC_WELCOME_EMAILS'],
            ((get_forum_type() == 'cns')/*Is on members menu*/ || (addon_installed('securitylogging'))) ? null : ['tools', 'menu/adminzone/tools/users/investigate_user', ['admin_lookup', [], get_module_zone('admin_lookup')], do_lang_tempcode('lookup:INVESTIGATE_USER'), 'lookup:DOC_INVESTIGATE_USER'],
            addon_installed('xml_fields') ? ['setup', 'menu/adminzone/setup/xml_fields', ['admin_config', ['type' => 'xml_fields'], get_module_zone('admin_config')], do_lang_tempcode('config:FIELD_FILTERS'), 'config:DOC_FIELD_FILTERS'] : null,

            //((get_forum_type() != 'cns') || (!has_privilege($member_id, 'control_usergroups'))) ? null : ['tools', 'menu/social/groups', ['groups', ['type' => 'browse'], get_module_zone('groups'), do_lang_tempcode('SWITCH_ZONE_WARNING')], do_lang_tempcode('SECONDARY_GROUP_MEMBERSHIP'), 'cns:DOC_SECONDARY_GROUP_MEMBERSHIP'], Excessive
            ['tools', 'menu/adminzone/tools/cleanup', ['admin_cleanup', ['type' => 'browse'], get_module_zone('admin_cleanup')], do_lang_tempcode('cleanup:CLEANUP_TOOLS'), 'cleanup:DOC_CLEANUP_TOOLS'],
            ['tools', 'menu/adminzone/tools/broken_urls', ['admin_broken_urls', ['type' => 'browse'], get_module_zone('admin_broken_urls')], do_lang_tempcode('cleanup:BROKEN_URLS'), 'cleanup:DOC_BROKEN_URLS'],
            ['tools', 'menu/pages/privacy_policy', ['admin_privacy', ['type' => 'browse'], get_module_zone('admin_privacy')], do_lang_tempcode('PRIVACY'), 'privacy:DOC_GDPR'],
            (get_value('brand_base_url') === null) ? ['tools', 'menu/adminzone/tools/upgrade', ['admin_config', ['type' => 'upgrader'], get_module_zone('admin_config')], do_lang_tempcode('upgrade:UPGRADER_UPGRADER_TITLE'), 'upgrade:UPGRADER_UPGRADER_INTRO'] : null,
            ((addon_installed('syndication')) && (get_value('hide_rss') === '0')) ? ['tools', 'links/rss', ['admin_config', ['type' => 'backend'], get_module_zone('admin_config')], do_lang_tempcode('FEEDS'), 'rss:OPML_INDEX_DESCRIPTION'] : null,
            (addon_installed('code_editor')) ? ['tools', 'menu/adminzone/tools/code_editor', ['admin_config', ['type' => 'code_editor'], get_module_zone('admin_config')], do_lang_tempcode('CODE_EDITOR'), 'DOC_CODE_EDITOR'] : null,

            ['security', 'menu/adminzone/security/permissions/permission_tree_editor', ['admin_permissions', ['type' => 'browse'], get_module_zone('admin_permissions')], do_lang_tempcode('permissions:PERMISSIONS_TREE'), 'permissions:DOC_PERMISSIONS_TREE'],
            addon_installed('match_key_permissions') ? ['security', 'menu/adminzone/security/permissions/match_keys', ['admin_permissions', ['type' => 'match_keys'], get_module_zone('admin_permissions')], do_lang_tempcode('permissions:PAGE_MATCH_KEY_ACCESS'), 'permissions:DOC_PAGE_MATCH_KEY_ACCESS'] : null,
            ['security', 'spare/content', ['admin_permissions', ['type' => 'content_access'], get_module_zone('admin_permissions')], do_lang_tempcode('permissions:CONTENT_ACCESS'), 'permissions:DOC_CONTENT_PERMISSIONS'],
            addon_installed('securitylogging') ? ['security', 'menu/adminzone/security/ip_ban', ['admin_ip_ban', ['type' => 'browse'], get_module_zone('admin_ip_ban')], do_lang_tempcode('submitban:BANNED_ADDRESSES'), 'submitban:DOC_IP_BAN'] : null,
            ['security', 'menu/adminzone/security/permissions/privileges', ['admin_permissions', ['type' => 'privileges'], get_module_zone('admin_permissions')], do_lang_tempcode('permissions:GLOBAL_PRIVILEGES'), 'permissions:DOC_PRIVILEGES'],
            (get_forum_type() != 'cns') ? null : ['security', 'menu/social/members', ['admin_cns_members', [], get_module_zone('admin_cns_members')], do_lang_tempcode('MEMBERS'), 'cns:DOC_MEMBERS'],
            (get_forum_type() != 'cns') ? null : ['security', 'menu/social/groups', ['admin_cns_groups', ['type' => 'browse'], get_module_zone('admin_cns_groups')], do_lang_tempcode('USERGROUPS'), 'cns:DOC_GROUPS'],
            (get_forum_type() != 'cns') ? null : ['security', 'menu/adminzone/security/usergroups_temp', ['admin_group_member_timeouts', ['type' => 'browse'], get_module_zone('admin_group_member_timeouts')], do_lang_tempcode('group_member_timeouts:GROUP_MEMBER_TIMEOUTS'), 'group_member_timeouts:DOC_MANAGE_GROUP_MEMBER_TIMEOUTS'],
            (get_forum_type() == 'cns') ? null : ['security', 'menu/social/groups', ['admin_permissions', ['type' => 'absorb'], get_module_zone('admin_security')], do_lang_tempcode('permissions:ABSORB_PERMISSIONS'), 'permissions:DOC_ABSORB_PERMISSIONS'],

            //(get_comcode_zone(DEFAULT_ZONE_PAGE_NAME, false) === null) ? null : ['', 'menu/' . DEFAULT_ZONE_PAGE_NAME, [DEFAULT_ZONE_PAGE_NAME, [], get_comcode_zone(DEFAULT_ZONE_PAGE_NAME)], do_lang_tempcode('HOME')],  Attached to zone, so this is not needed
            ['', 'menu/pages', ['admin', ['type' => 'pages'], 'adminzone'], do_lang_tempcode('PAGES')],
            ['', 'menu/rich_content', ['admin', ['type' => 'rich_content'], 'adminzone'], do_lang_tempcode('menus:RICH_CONTENT')],
            ['', 'menu/site_meta', ['admin', ['type' => 'site_meta'], 'adminzone'], do_lang_tempcode('menus:SITE_META')],
            ['', 'menu/social', ['admin', ['type' => 'social'], 'adminzone'], do_lang_tempcode('SECTION_SOCIAL')],

            (get_comcode_zone('about', false) === null) ? null : ['pages', 'menu/pages/about_us', ['about', [], get_comcode_zone('about')], do_lang_tempcode('menus:ABOUT')],

            //(get_comcode_zone('keymap', false) === null) ? null : ['site_meta',/*'menu/pages/keymap'*/null, ['keymap', [], get_comcode_zone('keymap')], do_lang_tempcode('KEYBOARD_MAP')],   Shows as child of help page
            (get_comcode_zone('privacy', false) === null || get_option('bottom_show_privacy_link') == '1') ? null : ['site_meta', 'menu/pages/privacy_policy', ['privacy', [], get_comcode_zone('privacy')], do_lang_tempcode('PRIVACY')],
            (get_comcode_zone('rules', false) === null || get_option('bottom_show_rules_link') == '1') ? null : ['site_meta', 'menu/pages/rules', ['rules', [], get_comcode_zone('rules')], do_lang_tempcode('RULES')],
            (get_comcode_zone('feedback', false) === null || get_option('bottom_show_feedback_link') == '1') ? null : ['site_meta', 'menu/site_meta/contact_us', ['feedback', [], get_comcode_zone('feedback')], do_lang_tempcode('FEEDBACK')],
            //(get_comcode_zone('sitemap', false) === null || get_option('bottom_show_sitemap_button') == '1') ? null : ['site_meta', 'tool_buttons/sitemap', ['sitemap', [], get_comcode_zone('sitemap')], do_lang_tempcode('SITEMAP')],   Redundant, menu itself is a sitemap
            // userguide_comcode is child of help_page
            (get_forum_type() == 'none' || !is_guest($member_id)) ? null : ['site_meta', 'menu/site_meta/user_actions/login', ['login', [], ''], do_lang_tempcode('_LOGIN')],
            (get_forum_type() != 'none' || is_guest($member_id)) ? null : ['site_meta', 'tool_buttons/notifications', ['notifications', [], get_module_zone('notifications')], do_lang_tempcode('notifications:NOTIFICATIONS')],
            //(get_forum_type() == 'none' || is_guest($member_id)) ? null : ['site_meta', 'menu/site_meta/user_actions/logout', ['login', [], ''), do_lang_tempcode('LOGOUT')], Don't show an immediate action, don't want accidental preloading
            (get_forum_type() != 'cns') ? null : ['site_meta', 'menu/site_meta/user_actions/join', ['join', [], get_module_zone('join')], do_lang_tempcode('_JOIN')],
            (get_forum_type() != 'cns') ? null : ['site_meta', 'menu/site_meta/user_actions/lost_password', ['lost_password', [], get_module_zone('lost_password')], do_lang_tempcode('cns_lost_password:LOST_PASSWORD')],

            (get_forum_type() != 'cns') ? null : ['social', 'menu/social/groups', ['groups', [], get_module_zone('groups')], do_lang_tempcode('USERGROUPS')],
            (get_forum_type() != 'cns') ? null : ['social', 'menu/social/members', ['members', [], get_module_zone('members')], do_lang_tempcode('cns:MEMBER_DIRECTORY')],
            (get_forum_type() != 'cns') ? null : ['social', 'menu/social/users_online', ['users_online', [], get_module_zone('users_online')], do_lang_tempcode('USERS_ONLINE')],

            (get_forum_type() == 'cns' || get_forum_type() == 'none') ? null : ['social', 'menu/social/forum/forums', (get_option('forum_in_portal') == '1') ? build_url(['page' => 'forums'], get_module_zone('forums')) : (get_forum_base_url() . '/'), do_lang_tempcode('SECTION_FORUMS')],
            (get_forum_type() == 'cns' || get_forum_type() == 'none') ? null : ['social', 'menu/social/members', $GLOBALS['FORUM_DRIVER']->member_profile_url($member_id, true), do_lang_tempcode('MY_PROFILE')],
            (get_forum_type() == 'cns' || get_forum_type() == 'none') ? null : ['site_meta', 'menu/site_meta/user_actions/join', $GLOBALS['FORUM_DRIVER']->join_url(), do_lang_tempcode('_JOIN')],
            (get_forum_type() == 'cns' || get_forum_type() == 'none') ? null : ['social', 'menu/social/users_online', $GLOBALS['FORUM_DRIVER']->users_online_url(), do_lang_tempcode('USERS_ONLINE')],
        ];
    }
}
