<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    ecommerce
 */

/**
 * Hook class.
 */
class Hook_page_groupings_ecommerce
{
    /**
     * Run function for do_next_menu hooks. They find links to put on standard navigation menus of the system.
     *
     * @param  ?MEMBER $member_id Member ID to run as (null: current member)
     * @param  boolean $extensive_docs Whether to use extensive documentation tooltips, rather than short summaries
     * @return array List of tuple of links (page grouping, icon, do-next-style linking data), label, help (optional) and/or nulls
     */
    public function run(?int $member_id = null, bool $extensive_docs = false) : array
    {
        if (!addon_installed('ecommerce')) {
            return [];
        }

        $ret = [
            ['setup', 'menu/adminzone/audit/ecommerce/ecommerce', ['admin_ecommerce', [], get_module_zone('admin_ecommerce')], do_lang_tempcode('ecommerce:ECOMMERCE'), 'ecommerce:DOC_ECOMMERCE'],
            ['audit', 'menu/adminzone/audit/ecommerce/ecommerce', ['admin_ecommerce_reports', ['type' => 'browse'], get_module_zone('admin_ecommerce_reports')], do_lang_tempcode('ecommerce:ECOMMERCE'), 'ecommerce:DOC_ECOMMERCE'],
            ['rich_content', 'menu/rich_content/ecommerce/purchase', ['purchase', [], get_module_zone('purchase')], do_lang_tempcode('ecommerce:PURCHASING')],
        ];
        if ((!is_guest($member_id)) && (intval($GLOBALS['SITE_DB']->query_select_value('ecom_invoices', 'COUNT(*)', ['i_member_id' => get_member()])) > 0)) {
            $ret = array_merge($ret, [
                ['rich_content', 'menu/adminzone/audit/ecommerce/invoices', ['invoices', [], get_module_zone('invoices')], do_lang_tempcode('ecommerce:MY_INVOICES')],
            ]);
        }
        if ((!is_guest($member_id)) && (intval($GLOBALS['SITE_DB']->query_select_value('ecom_subscriptions', 'COUNT(*)', ['s_member_id' => get_member()])) > 0)) {
            $ret = array_merge($ret, [
                ['rich_content', 'menu/adminzone/audit/ecommerce/subscriptions', ['subscriptions', [], get_module_zone('subscriptions')], do_lang_tempcode('ecommerce:MY_SUBSCRIPTIONS')],
            ]);
        }
        if (addon_installed('shopping')) {
            $ret = array_merge($ret, [
                ['rich_content', 'menu/rich_content/ecommerce/shopping_cart', ['shopping', [], get_module_zone('shopping')], do_lang_tempcode('shopping:SHOPPING')],
            ]);
        }
        return $ret;
    }
}
