<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_preview_block_comcode
{
    /**
     * Find whether this preview hook applies.
     *
     * @return array Triplet: Whether it applies, the attachment ID type (may be null), whether the forum DB is used [optional]
     */
    public function applies() : array
    {
        if (!has_privilege(get_member(), 'comcode_dangerous')) {
            return [false, null, false];
        }

        $applies = (post_param_string('block', null) !== null);
        return [$applies, null, false];
    }

    /**
     * Run function for preview hooks.
     *
     * @return array A pair: The preview, the updated post Comcode (may be null)
     */
    public function run() : array
    {
        if (!has_privilege(get_member(), 'comcode_dangerous')) {
            access_denied('I_ERROR');
        }

        require_code('zones2');
        require_code('zones3');

        $bparameters = '';
        $block = post_param_string('block');
        $parameters = get_block_parameters($block, true);
        foreach ($parameters as $parameter) {
            $value = post_param_string($parameter, null);
            if ($value === null) {
                if (post_param_integer('tick_on_form__' . $parameter, null) === null) {
                    $value = '';
                } else {
                    $value = '0';
                }
            }

            if ($value != '') {
                $bparameters .= ' ' . $parameter . '="' . str_replace('"', '\"', $value) . '"';
            }
        }

        $comcode = '[block' . $bparameters . ']' . $block . '[/block]';

        $preview = comcode_to_tempcode($comcode);

        return [$preview, null];
    }
}
