<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    galleries
 */

/**
 * Hook class.
 */
class Hook_preview_image
{
    /**
     * Find whether this preview hook applies.
     *
     * @return array Triplet: Whether it applies, the attachment ID type (may be null), whether the forum DB is used [optional]
     */
    public function applies() : array
    {
        $applies = (addon_installed('galleries')) && (get_page_name() == 'cms_galleries') && ((get_param_string('type', '') == 'add') || (get_param_string('type', '') == '_edit'));
        return [$applies, null, false];
    }

    /**
     * Run function for preview hooks.
     *
     * @return array A pair: The preview, the updated post Comcode (may be null)
     */
    public function run() : array
    {
        require_code('uploads');
        require_code('galleries2');

        list(
            $url,
            $filename,
            $title,
            $cat,
        ) = image_get_defaults__post();

        if ($url == '') {
            if (post_param_integer('id', null) !== null) {
                $rows = $GLOBALS['SITE_DB']->query_select('images', ['url'], ['id' => post_param_integer('id')], '', 1);
                $urls = $rows[0];

                $url = $urls['url'];
            } else {
                warn_exit(do_lang_tempcode('IMPROPERLY_FILLED_IN_UPLOAD'));
            }
        }

        require_code('images');
        $thumb = do_image_thumb(url_is_local($url) ? (get_custom_base_url() . '/' . $url) : $url, protect_from_escaping(comcode_to_tempcode(post_param_string('description'))), true);
        $preview = hyperlink(url_is_local($url) ? (get_custom_base_url() . '/' . $url) : $url, $thumb, false, true);

        return [$preview, null];
    }
}
