<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    themewizard
 */

/**
 * Hook class.
 */
class Hook_preview_logo
{
    /**
     * Find whether this preview hook applies.
     *
     * @return array Triplet: Whether it applies, the attachment ID type (may be null), whether the forum DB is used [optional]
     */
    public function applies() : array
    {
        $applies = (addon_installed('themewizard')) && (get_page_name() == 'admin_themewizard') && (get_param_string('type', '') == 'make_logo');
        return [$applies, null, false];
    }

    /**
     * Run function for preview hooks.
     *
     * @return array A pair: The preview, the updated post Comcode (may be null)
     */
    public function run() : array
    {
        $theme = post_param_string('theme');

        require_lang('themes');

        require_css('themes_editor');

        if (get_theme_option('enable_logowizard', null, $theme) == '0') {
            warn_exit(do_lang_tempcode('THEME_NOT_SUPPORTING_LOGOWIZARD', escape_html(get_theme_option('title', null, $theme)), escape_html($theme)));
        }

        $preview = new Tempcode();
        foreach (['large' => '-logo', 'standalone' => 'standalone_logo', 'small' => 'small_logo', 'small_white' => 'small_white_logo'] as $logo_type => $logo_output_theme_image) {
            $preview->attach(do_template('LOGOWIZARD_2', [
                '_GUID' => '6e5a442860e5b7644b50c2345c3c8dee',
                'NAME' => post_param_string('logo_name'),
                'FONT' => post_param_string('font'),
                'COLOUR' => post_param_string('colour'),
                'LOGO_TYPE' => $logo_type,
                'LOGO_OUTPUT_THEME_IMAGE' => $logo_output_theme_image,
                'LOGO_THEME_IMAGE' => post_param_string('logo_theme_image'),
                'BACKGROUND_THEME_IMAGE' => post_param_string('background_theme_image'),
                'THEME' => $theme,
            ]));
        }

        return [$preview, null];
    }
}
