<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    themewizard
 */

/**
 * Hook class.
 */
class Hook_preview_themewizard
{
    /**
     * Find whether this preview hook applies.
     *
     * @return array Triplet: Whether it applies, the attachment ID type (may be null), whether the forum DB is used [optional]
     */
    public function applies() : array
    {
        $applies = (addon_installed('themewizard')) && (get_page_name() == 'admin_themewizard') && (get_param_string('type', '') == '');
        return [$applies, null, false];
    }

    /**
     * Run function for preview hooks.
     *
     * @return array A pair: The preview, the updated post Comcode (may be null)
     */
    public function run() : array
    {
        require_lang('themes');
        require_code('themewizard');
        require_css('themes_editor');

        $source_theme = post_param_string('source_theme');
        $algorithm = post_param_string('algorithm');
        $seed = preg_replace('/^\#/', '', post_param_string('seed'));
        $dark = (post_param_integer('dark', 0) == 1);

        check_themewizard_theme($source_theme, $algorithm, $seed, $dark);

        list($colours) = calculate_themewizard_css_colours($seed, $dark, $source_theme, $algorithm);

        $preview = do_template('THEMEWIZARD_PREVIEW', [
            '_GUID' => 'c7b91a925047e6f4bd1208e8ccd9e434',
            'SOURCE_THEME' => $source_theme,
            'ALGORITHM' => $algorithm,
            'RED' => $colours['red'],
            'GREEN' => $colours['green'],
            'BLUE' => $colours['blue'],
            'DOMINANT' => $colours['dominant'],
            'LD' => $colours['LD'],
            'DARK' => $colours['dark'],
            'SEED' => $colours['seed'],
        ]);

        return [$preview, null];
    }
}
