<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_notifications
 */

/**
 * Hook class.
 */
class Hook_profiles_tabs_edit_notifications
{
    /**
     * Find whether this hook is active.
     *
     * @param  MEMBER $member_id_of The ID of the member who is being viewed
     * @param  MEMBER $member_id_viewing The ID of the member who is doing the viewing
     * @return boolean Whether this hook is active
     */
    public function is_active(int $member_id_of, int $member_id_viewing) : bool
    {
        return (($member_id_of == $member_id_viewing) || (has_privilege($member_id_viewing, 'assume_any_member')) || (has_privilege($member_id_viewing, 'member_maintenance')));
    }

    /**
     * Render function for profile tabs edit hooks.
     *
     * @param  MEMBER $member_id_of The ID of the member who is being viewed
     * @param  MEMBER $member_id_viewing The ID of the member who is doing the viewing
     * @param  boolean $leave_to_ajax_if_possible Whether to leave the tab contents null, if this hook supports it, so that AJAX can load it later
     * @return ?array A tuple: The tab title, the tab body text (may be blank), the tab fields, extra JavaScript (may be blank) the suggested tab order, hidden fields (optional) (null: if $leave_to_ajax_if_possible was set), the icon
     */
    public function render_tab(int $member_id_of, int $member_id_viewing, bool $leave_to_ajax_if_possible = false) : ?array
    {
        $memory_debugging = (get_param_integer('keep_show_loading', 0) == 1);
        if ($memory_debugging) {
            $usage_before = memory_get_usage();
        }

        require_lang('notifications');
        $title = do_lang_tempcode('NOTIFICATIONS');

        $order = 100;

        // The actualiser is handled by the settings tab
        /*
        if (get_param_integer('submitting_settings_tab', 0) == 1) {
            $auto_monitor_contrib_content = post_param_integer('auto_monitor_contrib_content', 0);
            $smart_topic_notification = post_param_integer('smart_topic_notification', 0);
            $mailing_list_style = post_param_integer('mailing_list_style', 0);

            $update_map = [
                'm_auto_monitor_contrib_content' => $auto_monitor_contrib_content,
                'm_smart_topic_notification' => $smart_topic_notification,
                'm_mailing_list_style' => $mailing_list_style,
            ];

            $GLOBALS['FORUM_DB']->query_update('f_members', $update_map, ['id' => $member_id_of], '', 1);

            // Decache from run-time cache
            unset($GLOBALS['FORUM_DRIVER']->MEMBER_ROWS_CACHED[$member_id_of]);
            unset($GLOBALS['MEMBER_CACHE_FIELD_MAPPINGS'][$member_id_of]);
        }
        */

        if (($leave_to_ajax_if_possible) && ($_SERVER['REQUEST_METHOD'] != 'POST')) {
            return null;
        }

        require_code('notifications2');

        $text = notifications_ui($member_id_of);
        if ($text->is_empty()) {
            return null;
        }

        if ($memory_debugging) {
            require_code('files');
            $usage_after = memory_get_usage();
            $usage = clean_file_size($usage_after - $usage_before);
            $text->attach(paragraph('Memory debugging: ' . $usage . ' used, now at ' . clean_file_size($usage_after)));
            $title->attach(' (' . $usage . ')');
        }

        return [$title, new Tempcode(), $text, null, $order, null, 'tool_buttons/notifications2'];
    }
}
