<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    points
 */

/**
 * Hook class.
 */
class Hook_realtime_rain_points
{
    /**
     * Run function for realtime-rain hooks.
     *
     * @param  TIME $from Start of time range
     * @param  TIME $to End of time range
     * @return array A list of template parameter sets for rendering a 'drop'
     */
    public function run(int $from, int $to) : array
    {
        if (!addon_installed('points')) {
            return [];
        }

        $drops = [];

        if (has_actual_page_access(get_member(), 'points')) {
            require_lang('points');
            require_code('points');

            // point debits
            $rows = $GLOBALS['SITE_DB']->query('SELECT reason,amount_gift_points,amount_points,date_and_time AS timestamp,sending_member,receiving_member FROM ' . $GLOBALS['SITE_DB']->get_table_prefix() . 'points_ledger WHERE receiving_member=' . strval($GLOBALS['FORUM_DRIVER']->get_guest_id()) . ' AND date_and_time BETWEEN ' . strval($from) . ' AND ' . strval($to));

            foreach ($rows as $row) {
                $timestamp = $row['timestamp'];
                $sending_member = $row['sending_member'];

                $drops[] = rain_get_special_icons(null, $timestamp) + [
                    'TYPE' => 'point_debits',
                    'FROM_MEMBER_ID' => strval($sending_member),
                    'TO_MEMBER_ID' => null,
                    'TITLE' => do_lang('MEMBER_SPENT_POINTS', integer_format($row['amount_gift_points'] + $row['amount_points'], 0), get_translated_text($row['reason'])),
                    'IMAGE' => $GLOBALS['FORUM_DRIVER']->get_member_avatar_url($sending_member),
                    'TIMESTAMP' => strval($timestamp),
                    'RELATIVE_TIMESTAMP' => strval($timestamp - $from),
                    'TICKER_TEXT' => null,
                    'URL' => points_url($sending_member),
                    'IS_POSITIVE' => false,
                    'IS_NEGATIVE' => true,

                    // These are for showing connections between drops. They are not discriminated, it's just three slots to give an ID code that may be seen as a commonality with other drops.
                    'FROM_ID' => 'member_' . strval($sending_member),
                    'TO_ID' => false,
                    'GROUP_ID' => false,
                ];
            }

            // Point credits
            $rows = $GLOBALS['SITE_DB']->query('SELECT reason,amount_gift_points,amount_points,sending_member AS member_id,receiving_member,date_and_time AS timestamp,anonymous FROM ' . $GLOBALS['SITE_DB']->get_table_prefix() . 'points_ledger WHERE sending_member=' . strval($GLOBALS['FORUM_DRIVER']->get_guest_id()) . ' AND date_and_time BETWEEN ' . strval($from) . ' AND ' . strval($to));

            foreach ($rows as $row) {
                $timestamp = $row['timestamp'];
                $member_id = $row['member_id'];
                $receiving_member = $row['receiving_member'];

                $drops[] = rain_get_special_icons(null, $timestamp) + [
                    'TYPE' => 'point_credits',
                    'FROM_MEMBER_ID' => ($row['anonymous'] == 1) ? null : strval($member_id),
                    'TO_MEMBER_ID' => strval($receiving_member),
                    'TITLE' => do_lang('MEMBER_SENT_POINTS', integer_format($row['amount_gift_points'] + $row['amount_points'], 0), get_translated_text($row['reason'])),
                    'IMAGE' => $GLOBALS['FORUM_DRIVER']->get_member_avatar_url($receiving_member),
                    'TIMESTAMP' => strval($timestamp),
                    'RELATIVE_TIMESTAMP' => strval($timestamp - $from),
                    'TICKER_TEXT' => null,
                    'URL' => points_url($receiving_member),
                    'IS_POSITIVE' => true,
                    'IS_NEGATIVE' => false,

                    // These are for showing connections between drops. They are not discriminated, it's just three slots to give an ID code that may be seen as a commonality with other drops.
                    'FROM_ID' => ($row['anonymous'] == 1) ? null : ('member_' . strval($member_id)),
                    'TO_ID' => 'member_' . strval($receiving_member),
                    'GROUP_ID' => null,
                ];
            }

            // Point transactions
            $rows = $GLOBALS['SITE_DB']->query('SELECT reason,amount_gift_points,amount_points,sending_member AS member_id,receiving_member,date_and_time AS timestamp,anonymous FROM ' . $GLOBALS['SITE_DB']->get_table_prefix() . 'points_ledger WHERE sending_member<>' . strval($GLOBALS['FORUM_DRIVER']->get_guest_id()) . ' AND receiving_member<>' . strval($GLOBALS['FORUM_DRIVER']->get_guest_id()) . ' AND date_and_time BETWEEN ' . strval($from) . ' AND ' . strval($to));

            foreach ($rows as $row) {
                $timestamp = $row['timestamp'];
                $member_id = $row['member_id'];
                $receiving_member = $row['receiving_member'];

                $drops[] = rain_get_special_icons(null, $timestamp) + [
                    'TYPE' => 'point_transactions',
                    'FROM_MEMBER_ID' => ($row['anonymous'] == 1) ? null : strval($member_id),
                    'TO_MEMBER_ID' => strval($receiving_member),
                    'TITLE' => do_lang('MEMBER_SENT_POINTS', integer_format($row['amount_gift_points'] + $row['amount_points'], 0), get_translated_text($row['reason'])),
                    'IMAGE' => $GLOBALS['FORUM_DRIVER']->get_member_avatar_url($receiving_member),
                    'TIMESTAMP' => strval($timestamp),
                    'RELATIVE_TIMESTAMP' => strval($timestamp - $from),
                    'TICKER_TEXT' => null,
                    'URL' => points_url($receiving_member),
                    'IS_POSITIVE' => false,
                    'IS_NEGATIVE' => false,

                    // These are for showing connections between drops. They are not discriminated, it's just three slots to give an ID code that may be seen as a commonality with other drops.
                    'FROM_ID' => ($row['anonymous'] == 1) ? null : ('member_' . strval($member_id)),
                    'TO_ID' => 'member_' . strval($receiving_member),
                    'GROUP_ID' => null,
                ];
            }
        }

        return $drops;
    }
}
