<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    backup
 */

/**
 * Hook class.
 */
class Hook_snippet_backup_size
{
    /**
     * Run function for snippet hooks. Generates HTML to insert into a page using AJAX.
     *
     * @return Tempcode The snippet
     */
    public function run() : object
    {
        if (!addon_installed('backup')) {
            return new Tempcode();
        }

        cms_extend_time_limit(TIME_LIMIT_EXTEND__CRAWL);

        require_code('files2');
        require_code('backup');

        $directories_to_backup = array_flip(directories_to_backup());

        $size = 0;
        $max_size = get_param_integer('max_size') * 1024 * 1024;
        $files = get_directory_contents(get_custom_file_base(), '', IGNORE_REBUILDABLE_OR_TEMP_FILES_FOR_BACKUP);
        foreach ($files as $path) {
            $first_dir = preg_replace('#/.*#', '', $path);

            if (!isset($directories_to_backup[$first_dir])) {
                continue;
            }

            $file_size = filesize(get_custom_file_base() . '/' . $path);
            if ($file_size < $max_size) {
                $size += $file_size;
            }
        }

        return make_string_tempcode(clean_file_size($size));
    }
}
