<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core_feedback_features
 */

/**
 * Hook class.
 */
class Hook_snippet_comments
{
    /**
     * Run function for snippet hooks. Generates HTML to insert into a page using AJAX.
     *
     * @return Tempcode The snippet
     */
    public function run() : object
    {
        if (get_option('is_on_comments') == '0') {
            warn_exit(do_lang_tempcode('INTERNAL_ERROR', escape_html('a33597747b575767b9e9625cb5e7f3e0')));
        }

        $serialized_options = get_param_string('serialized_options', false, INPUT_FILTER_GET_COMPLEX);
        $hash = get_param_string('hash');

        require_code('crypt');
        if (!ratchet_hash_verify($serialized_options, get_site_salt(), $hash)) {
            warn_exit(do_lang_tempcode('INTERNAL_ERROR', escape_html('612cdd3a673c56779ece352780967923')));
        }

        list($topic_id, $num_to_show_limit, $allow_comments, $invisible_if_no_comments, $forum, $reverse, $may_reply, $highlight_by_user, $allow_reviews) = json_decode($serialized_options, true);

        $posts = array_map('intval', explode(',', get_param_string('ids', false, INPUT_FILTER_GET_COMPLEX)));

        $_parent_id = get_param_string('id', '');
        $parent_id = ($_parent_id == '') ? null : intval($_parent_id);

        require_code('topics');
        $renderer = new CMS_Topic();
        return $renderer->render_posts_from_topic($topic_id, $num_to_show_limit, $allow_comments, $invisible_if_no_comments, $forum, null, $reverse, $may_reply, $highlight_by_user, $allow_reviews, $posts, $parent_id);
    }
}
