<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    themewizard
 */

/**
 * Hook class.
 */
class Hook_snippet_themewizard_equation
{
    /**
     * Run function for snippet hooks. Generates HTML to insert into a page using AJAX.
     *
     * @return Tempcode The snippet
     */
    public function run() : object
    {
        if (!addon_installed('themewizard')) {
            return new Tempcode();
        }

        $theme = get_param_string('theme');
        $equation = get_param_string('css_equation', false, INPUT_FILTER_GET_COMPLEX);

        require_code('themewizard');
        require_code('themes2');

        $seed = find_theme_seed($theme);
        $dark = find_theme_dark($theme);

        list($colours, $landscape) = calculate_themewizard_css_colours($seed, $dark, $theme, 'equations');
        $parsed_equation = parse_themewizard_css_colour_expression($equation);
        if ($parsed_equation === null) {
            return make_string_tempcode('');
        }
        $answer = execute_themewizard_css_colour_expression($parsed_equation, $colours);
        if ($answer === null) {
            return make_string_tempcode('');
        }

        return make_string_tempcode('#' . $answer);
    }
}
