<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_spam_heuristics_frequency
{
    /**
     * Find the confidence score for a particular spam heuristic as applied to the current context.
     *
     * @param  string $post_data Posted data
     * @return integer Confidence score
     */
    public function assess_confidence(string $post_data) : int
    {
        $score = intval(get_option('spam_heuristic_confidence_frequency'));
        if ($score == 0) {
            return 0;
        }

        if (is_guest()) {
            return 0;
        }

        $previous = 0;

        require_code('content');
        foreach (array_keys(find_all_hooks('systems', 'content_meta_aware')) as $hook) {
            $cma_ob = get_content_object($hook);
            $cma_info = $cma_ob->info();
            if (!empty($cma_info['support_spam_heuristics'])) {
                $data = post_param_string($cma_info['support_spam_heuristics'], null);
                if (!cms_empty_safe($data)) {
                    $where = [
                        $cma_info['submitter_field'] => get_member(),
                    ];

                    $threshold = 60 * intval(get_option('spam_heuristic_frequency'));

                    $previous += $cma_info['db']->query_select_value(
                        $cma_info['table'],
                        'COUNT(*)',
                        $where,
                        ' AND ' . $cma_info['add_time_field'] . '>' . strval(time() - $threshold)
                    );
                }
            }
        }

        if ($previous > intval(get_option('spam_heuristic_frequency_threshold'))) {
            return $score;
        }

        return 0;
    }
}
