<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    stats
 */

/**
 * Hook class.
 */
class Hook_task_install_geolocation_data
{
    /**
     * Run the task hook.
     *
     * @return ?array A tuple of at least 2: Return mime-type, content (either Tempcode, or a string, or a filename and file-path pair to a temporary file), map of HTTP headers if transferring immediately, map of ini_set commands if transferring immediately (null: show standard success message)
     */
    public function run() : ?array
    {
        if (!addon_installed('stats')) {
            return null;
        }

        push_query_limiting(false);

        $GLOBALS['SITE_DB']->query_delete('ip_country');

        // We need to read in IP_Country.txt, line-by-line...

        require_code('files');

        $path = get_file_base() . '/data/modules/admin_stats/IP_Country.txt';
        $file_charset = null;
        $file = @cms_fopen_text_read($path, $file_charset, true);
        if ($file === false) {
            warn_exit(do_lang_tempcode('READ_ERROR', escape_html($path)), false, true);
        }
        $i = 0;
        $to_insert = ['begin_num' => [], 'end_num' => [], 'country' => []];
        while (!feof($file)) {
            $data = cms_fgets($file, $file_charset);
            if ($data === false) {
                continue;
            }

            $data = trim($data);

            task_log($this, 'Preprocessing geolocation data row', $i);

            $_data = explode(',', $data);
            if (count($_data) == 3) {
                if (($_data[0][0] == '-') || ($_data[1][0] == '-')) {
                    fatal_exit(do_lang_tempcode('INTERNAL_ERROR', escape_html('e3542e5d80d0514cba8bc69cda73b537')));
                }

                $to_insert['begin_num'][] = $_data[0]; // FUDGE. Intentionally passes in as strings, to workaround problem in PHP integer sizes (can't store unsigned data type) #3046
                $to_insert['end_num'][] = $_data[1];
                $to_insert['country'][] = substr($_data[2], 0, 2);

                if (count($to_insert['begin_num']) == 100) { // Batches of 100
                    $GLOBALS['SITE_DB']->query_insert('ip_country', $to_insert);
                    $to_insert = ['begin_num' => [], 'end_num' => [], 'country' => []];
                }
            }

            $i++;
        }
        flock($file, LOCK_UN);
        fclose($file);

        if (!empty($to_insert['begin_num'])) { // Final batch, if there is one
            $GLOBALS['SITE_DB']->query_insert('ip_country', $to_insert);
        }

        return null;
    }
}
