<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    core
 */

/**
 * Hook class.
 */
class Hook_trusted_sites_core
{
    /**
     * Detect what needs to be 'added' to the trusted_sites_1 option.
     *
     * @param  array $sites List of trusted sites (written by reference)
     */
    public function find_trusted_sites_1(array &$sites)
    {
        $sites[] = str_replace('www.', '', cms_parse_url_safe(get_brand_base_url(), PHP_URL_HOST));
    }

    /**
     * Detect what needs to be 'added' to the trusted_sites_2 option.
     *
     * @param  array $sites List of trusted sites (written by reference)
     */
    public function find_trusted_sites_2(array &$sites)
    {
        if (!empty(get_option('google_fonts'))) {
            $sites[] = 'fonts.googleapis.com';
        }

        if ((get_option('google_apis_api_key') != '') && (get_option('google_translate_enabled') == '1') && (get_option('csp_on_forms') == '1')) {
            $sites[] = 'translate.googleusercontent.com';
        }

        if (get_option('csp_on_forms') == '1') {
            $sites[] = 'validator.w3.org';
        }
    }
}
